@@ -27,7 +27,10 @@ import (
 	"github.com/apache/skywalking-banyandb/banyand/measure"
 	"github.com/apache/skywalking-banyandb/banyand/metadata"
 	"github.com/apache/skywalking-banyandb/banyand/observability"
+	"github.com/apache/skywalking-banyandb/banyand/property"
+	"github.com/apache/skywalking-banyandb/banyand/protector"
 	"github.com/apache/skywalking-banyandb/banyand/query"
+	"github.com/apache/skywalking-banyandb/banyand/queue"
 	"github.com/apache/skywalking-banyandb/banyand/queue/sub"
 	"github.com/apache/skywalking-banyandb/banyand/stream"
 	"github.com/apache/skywalking-banyandb/pkg/logger"
@@ -38,40 +41,46 @@ import (
 func newDataCmd(runners ...run.Unit) *cobra.Command {
 	l := logger.GetLogger("bootstrap")
 	ctx := context.Background()
-	metaSvc, err := metadata.NewClient(false)
+	metaSvc, err := metadata.NewClient(true, false)
 	if err != nil {
 		l.Fatal().Err(err).Msg("failed to initiate metadata service")
 	}
-	pipeline := sub.NewServer()
-	streamSvc, err := stream.NewService(ctx, metaSvc, pipeline)
+	localPipeline := queue.Local()
+	metricSvc := observability.NewMetricService(metaSvc, localPipeline, "data", nil)
+	pm := protector.NewMemory(metricSvc)
+	pipeline := sub.NewServer(metricSvc)
+	propertySvc, err := property.NewService(metaSvc, pipeline, metricSvc)
+	if err != nil {
+		l.Fatal().Err(err).Msg("failed to initiate property service")
+	}
+	streamSvc, err := stream.NewService(metaSvc, pipeline, metricSvc, pm)
 	if err != nil {
 		l.Fatal().Err(err).Msg("failed to initiate stream service")
 	}
-	measureSvc, err := measure.NewService(ctx, metaSvc, pipeline)
+	measureSvc, err := measure.NewService(metaSvc, pipeline, localPipeline, metricSvc, pm)
 	if err != nil {
 		l.Fatal().Err(err).Msg("failed to initiate measure service")
 	}
-	// TODO: remove streamSVC and measureSvc from query processor. To use metaSvc instead.
 	q, err := query.NewService(ctx, streamSvc, measureSvc, metaSvc, pipeline)
 	if err != nil {
 		l.Fatal().Err(err).Msg("failed to initiate query processor")
 	}
 	profSvc := observability.NewProfService()
-	metricSvc := observability.NewMetricService(metaSvc)
 
 	var units []run.Unit
 	units = append(units, runners...)
 	units = append(units,
 		metaSvc,
+		localPipeline,
+		metricSvc,
+		pm,
 		pipeline,
+		propertySvc,
 		measureSvc,
 		streamSvc,
 		q,
 		profSvc,
 	)
-	if metricSvc != nil {
-		units = append(units, metricSvc)
-	}
 	dataGroup := run.NewGroup("data")
 	dataGroup.Register(units...)
 	dataCmd := &cobra.Command{
