@@ -24,6 +24,7 @@ import (
 	"github.com/spf13/cobra"
 
 	"github.com/apache/skywalking-banyandb/api/common"
+	"github.com/apache/skywalking-banyandb/api/data"
 	"github.com/apache/skywalking-banyandb/banyand/dquery"
 	"github.com/apache/skywalking-banyandb/banyand/liaison/grpc"
 	"github.com/apache/skywalking-banyandb/banyand/liaison/http"
@@ -40,21 +41,25 @@ import (
 func newLiaisonCmd(runners ...run.Unit) *cobra.Command {
 	l := logger.GetLogger("bootstrap")
 	ctx := context.Background()
-	metaSvc, err := metadata.NewClient(false)
+	metaSvc, err := metadata.NewClient(true, false)
 	if err != nil {
 		l.Fatal().Err(err).Msg("failed to initiate metadata service")
 	}
 	pipeline := pub.New(metaSvc)
 	localPipeline := queue.Local()
-	nodeSel, err := node.NewMaglevSelector()
-	if err != nil {
-		l.Fatal().Err(err).Msg("failed to initiate required node selector")
-	}
-	grpcServer := grpc.NewServer(ctx, pipeline, localPipeline, metaSvc, grpc.NewClusterNodeRegistry(pipeline, nodeSel))
+	measureNodeSel := node.NewRoundRobinSelector(data.TopicMeasureWrite.String(), metaSvc)
+	measureNodeRegistry := grpc.NewClusterNodeRegistry(data.TopicMeasureWrite, pipeline, measureNodeSel)
+	metricSvc := observability.NewMetricService(metaSvc, pipeline, "liaison", measureNodeRegistry)
+	streamNodeSel := node.NewRoundRobinSelector(data.TopicStreamWrite.String(), metaSvc)
+	propertyNodeSel := node.NewRoundRobinSelector(data.TopicPropertyUpdate.String(), metaSvc)
+	grpcServer := grpc.NewServer(ctx, pipeline, localPipeline, metaSvc, grpc.NodeRegistries{
+		MeasureNodeRegistry:  measureNodeRegistry,
+		StreamNodeRegistry:   grpc.NewClusterNodeRegistry(data.TopicStreamWrite, pipeline, streamNodeSel),
+		PropertyNodeRegistry: grpc.NewClusterNodeRegistry(data.TopicPropertyUpdate, pipeline, propertyNodeSel),
+	}, metricSvc)
 	profSvc := observability.NewProfService()
-	metricSvc := observability.NewMetricService(metaSvc)
 	httpServer := http.NewServer()
-	dQuery, err := dquery.NewService(metaSvc, localPipeline, pipeline)
+	dQuery, err := dquery.NewService(metaSvc, localPipeline, pipeline, metricSvc)
 	if err != nil {
 		l.Fatal().Err(err).Msg("failed to initiate distributed query service")
 	}
@@ -64,21 +69,33 @@ func newLiaisonCmd(runners ...run.Unit) *cobra.Command {
 		metaSvc,
 		localPipeline,
 		pipeline,
+		measureNodeSel,
+		streamNodeSel,
+		propertyNodeSel,
+		metricSvc,
 		dQuery,
 		grpcServer,
 		httpServer,
 		profSvc,
 	)
-	if metricSvc != nil {
-		units = append(units, metricSvc)
-	}
 	liaisonGroup := run.NewGroup("liaison")
 	liaisonGroup.Register(units...)
+	var nodeSelector string
 	liaisonCmd := &cobra.Command{
 		Use:     "liaison",
 		Version: version.Build(),
 		Short:   "Run as the liaison server",
 		RunE: func(_ *cobra.Command, _ []string) (err error) {
+			if nodeSelector != "" {
+				var ls *pub.LabelSelector
+				ls, err = pub.ParseLabelSelector(nodeSelector)
+				if err != nil {
+					return err
+				}
+				for _, sel := range []node.Selector{measureNodeSel, streamNodeSel, propertyNodeSel} {
+					sel.SetNodeSelector(ls)
+				}
+			}
 			node, err := common.GenerateNode(grpcServer.GetPort(), httpServer.GetPort())
 			if err != nil {
 				return err
@@ -93,5 +110,7 @@ func newLiaisonCmd(runners ...run.Unit) *cobra.Command {
 		},
 	}
 	liaisonCmd.Flags().AddFlagSet(liaisonGroup.RegisterFlags().FlagSet)
+	liaisonCmd.PersistentFlags().StringVar(&nodeSelector, "data-node-selector", "",
+		"the data node selector. e.g. key1=value1,key2=value2. If not set, all nodes are selected")
 	return liaisonCmd
 }
