@@ -21,6 +21,7 @@ import (
 	"context"
 	"os"
 
+	grpcprom "github.com/grpc-ecosystem/go-grpc-middleware/providers/prometheus"
 	"github.com/spf13/cobra"
 
 	"github.com/apache/skywalking-banyandb/api/common"
@@ -29,6 +30,8 @@ import (
 	"github.com/apache/skywalking-banyandb/banyand/measure"
 	"github.com/apache/skywalking-banyandb/banyand/metadata/embeddedserver"
 	"github.com/apache/skywalking-banyandb/banyand/observability"
+	"github.com/apache/skywalking-banyandb/banyand/property"
+	"github.com/apache/skywalking-banyandb/banyand/protector"
 	"github.com/apache/skywalking-banyandb/banyand/query"
 	"github.com/apache/skywalking-banyandb/banyand/queue"
 	"github.com/apache/skywalking-banyandb/banyand/stream"
@@ -45,11 +48,20 @@ func newStandaloneCmd(runners ...run.Unit) *cobra.Command {
 	if err != nil {
 		l.Fatal().Err(err).Msg("failed to initiate metadata service")
 	}
-	streamSvc, err := stream.NewService(ctx, metaSvc, pipeline)
+	metricSvc := observability.NewMetricService(metaSvc, pipeline, "standalone", nil)
+	pm := protector.NewMemory(metricSvc)
+	propertySvc, err := property.NewService(metaSvc, pipeline, metricSvc)
+	if err != nil {
+		l.Fatal().Err(err).Msg("failed to initiate property service")
+	}
+	streamSvc, err := stream.NewService(metaSvc, pipeline, metricSvc, pm)
 	if err != nil {
 		l.Fatal().Err(err).Msg("failed to initiate stream service")
 	}
-	measureSvc, err := measure.NewService(ctx, metaSvc, pipeline)
+	var srvMetrics *grpcprom.ServerMetrics
+	srvMetrics.UnaryServerInterceptor()
+	srvMetrics.UnaryServerInterceptor()
+	measureSvc, err := measure.NewService(metaSvc, pipeline, nil, metricSvc, pm)
 	if err != nil {
 		l.Fatal().Err(err).Msg("failed to initiate measure service")
 	}
@@ -57,9 +69,13 @@ func newStandaloneCmd(runners ...run.Unit) *cobra.Command {
 	if err != nil {
 		l.Fatal().Err(err).Msg("failed to initiate query processor")
 	}
-	grpcServer := grpc.NewServer(ctx, pipeline, pipeline, metaSvc, grpc.NewLocalNodeRegistry())
+	nr := grpc.NewLocalNodeRegistry()
+	grpcServer := grpc.NewServer(ctx, pipeline, pipeline, metaSvc, grpc.NodeRegistries{
+		MeasureNodeRegistry:  nr,
+		StreamNodeRegistry:   nr,
+		PropertyNodeRegistry: nr,
+	}, metricSvc)
 	profSvc := observability.NewProfService()
-	metricSvc := observability.NewMetricService(metaSvc)
 	httpServer := http.NewServer()
 
 	var units []run.Unit
@@ -67,6 +83,9 @@ func newStandaloneCmd(runners ...run.Unit) *cobra.Command {
 	units = append(units,
 		pipeline,
 		metaSvc,
+		metricSvc,
+		pm,
+		propertySvc,
 		measureSvc,
 		streamSvc,
 		q,
@@ -74,9 +93,6 @@ func newStandaloneCmd(runners ...run.Unit) *cobra.Command {
 		httpServer,
 		profSvc,
 	)
-	if metricSvc != nil {
-		units = append(units, metricSvc)
-	}
 	standaloneGroup := run.NewGroup("standalone")
 	// Meta the run Group units.
 	standaloneGroup.Register(units...)
