@@ -19,17 +19,17 @@
 package inverted
 
 import (
-	"bytes"
 	"context"
 	"io"
 	"log"
-	"math"
+	"strconv"
 	"time"
 
 	"github.com/blugelabs/bluge"
 	"github.com/blugelabs/bluge/analysis"
 	"github.com/blugelabs/bluge/analysis/analyzer"
 	blugeIndex "github.com/blugelabs/bluge/index"
+	"github.com/blugelabs/bluge/numeric"
 	"github.com/blugelabs/bluge/search"
 	"github.com/pkg/errors"
 	"go.uber.org/multierr"
@@ -48,17 +48,15 @@ import (
 const (
 	docIDField     = "_id"
 	batchSize      = 1024
-	seriesIDField  = "series_id"
-	entityField    = "entity"
-	idField        = "id"
-	timestampField = "timestamp"
+	seriesIDField  = "_series_id"
+	timestampField = "_timestamp"
+	versionField   = "_version"
+	sourceField    = "_source"
 )
 
 var (
-	defaultUpper            = convert.Uint64ToBytes(math.MaxUint64)
-	defaultLower            = convert.Uint64ToBytes(0)
 	defaultRangePreloadSize = 1000
-	defaultProjection       = []string{docIDField}
+	defaultProjection       = []string{docIDField, timestampField}
 )
 
 // Analyzers is a map that associates each IndexRule_Analyzer type with a corresponding Analyzer.
@@ -77,10 +75,11 @@ var _ index.Store = (*store)(nil)
 
 // StoreOpts wraps options to create an inverted index repository.
 type StoreOpts struct {
-	Logger       *logger.Logger
-	Metrics      *Metrics
-	Path         string
-	BatchWaitSec int64
+	Logger        *logger.Logger
+	Metrics       *Metrics
+	Path          string
+	BatchWaitSec  int64
+	CacheMaxBytes int
 }
 
 type store struct {
@@ -113,15 +112,19 @@ func (s *store) Batch(batch index.Batch) error {
 	b := generateBatch()
 	defer releaseBatch(b)
 	for _, d := range batch.Documents {
-		var fk *index.FieldKey
-		if len(d.Fields) > 0 {
-			fk = &d.Fields[0].Key
-		}
 		doc := bluge.NewDocument(convert.BytesToString(convert.Uint64ToBytes(d.DocID)))
-		for _, f := range d.Fields {
-			tf := bluge.NewKeywordFieldBytes(f.Key.Marshal(), f.Term)
+		for i, f := range d.Fields {
+			var tf *bluge.TermField
+			switch f.GetTerm().(type) {
+			case *index.BytesTermValue:
+				tf = bluge.NewKeywordFieldBytes(f.Key.Marshal(), f.GetBytes())
+			case *index.FloatTermValue:
+				tf = bluge.NewNumericField(f.Key.Marshal(), f.GetFloat())
+			default:
+				return errors.Errorf("unexpected field type: %T", f.GetTerm())
+			}
 			if !f.NoSort {
-				tf.StoreValue().Sortable()
+				tf.Sortable()
 			}
 			if f.Store {
 				tf.StoreValue()
@@ -130,17 +133,15 @@ func (s *store) Batch(batch index.Batch) error {
 				tf = tf.WithAnalyzer(Analyzers[f.Key.Analyzer])
 			}
 			doc.AddField(tf)
+			if i == 0 {
+				doc.AddField(bluge.NewKeywordFieldBytes(seriesIDField, f.Key.SeriesID.Marshal()).StoreValue())
+			}
 		}
 
-		if d.EntityValues != nil {
-			doc.AddField(bluge.NewKeywordFieldBytes(entityField, d.EntityValues).StoreValue())
-		} else if fk != nil && fk.HasSeriesID() {
-			doc.AddField(bluge.NewKeywordFieldBytes(seriesIDField, fk.SeriesID.Marshal()).StoreValue())
-		}
 		if d.Timestamp > 0 {
 			doc.AddField(bluge.NewDateTimeField(timestampField, time.Unix(0, d.Timestamp)).StoreValue())
 		}
-		b.Update(doc.ID(), doc)
+		b.Insert(doc)
 	}
 	return s.writer.Batch(b)
 }
@@ -155,6 +156,7 @@ func NewStore(opts StoreOpts) (index.SeriesStore, error) {
 		indexConfig = indexConfig.WithUnsafeBatches().
 			WithPersisterNapTimeMSec(int(opts.BatchWaitSec * 1000))
 	}
+	indexConfig.CacheMaxBytes = opts.CacheMaxBytes
 	config := bluge.DefaultConfigWithIndexConfig(indexConfig)
 	config.DefaultSearchAnalyzer = Analyzers[index.AnalyzerKeyword]
 	config.Logger = log.New(opts.Logger, opts.Logger.Module(), 0)
@@ -177,12 +179,14 @@ func (s *store) Close() error {
 	return s.writer.Close()
 }
 
+func (s *store) Reset() {
+	s.writer.ResetCache()
+}
+
 func (s *store) Iterator(ctx context.Context, fieldKey index.FieldKey, termRange index.RangeOpts, order modelv1.Sort,
-	preLoadSize int, indexQuery index.Query, fieldKeys []index.FieldKey,
+	preLoadSize int,
 ) (iter index.FieldIterator[*index.DocumentResult], err error) {
-	if termRange.Lower != nil &&
-		termRange.Upper != nil &&
-		bytes.Compare(termRange.Lower, termRange.Upper) > 0 {
+	if !termRange.IsEmpty() && !termRange.Valid() {
 		return index.DummyFieldIterator, nil
 	}
 	if !s.closer.AddRunning() {
@@ -196,124 +200,168 @@ func (s *store) Iterator(ctx context.Context, fieldKey index.FieldKey, termRange
 	fk := fieldKey.Marshal()
 	rangeQuery := bluge.NewBooleanQuery()
 	rangeNode := newMustNode()
-	addRange := func(query *bluge.BooleanQuery, termRange index.RangeOpts) *bluge.BooleanQuery {
-		if termRange.Upper == nil {
-			termRange.Upper = defaultUpper
-		}
-		if termRange.Lower == nil {
-			termRange.Lower = defaultLower
-		}
-		query.AddMust(bluge.NewTermRangeInclusiveQuery(
-			string(termRange.Lower),
-			string(termRange.Upper),
-			termRange.IncludesLower,
-			termRange.IncludesUpper,
-		).
-			SetField(fk))
-		rangeNode.Append(newTermRangeInclusiveNode(string(termRange.Lower), string(termRange.Upper), termRange.IncludesLower, termRange.IncludesUpper, nil))
-		return query
-	}
 
-	if fieldKey.HasSeriesID() {
-		rangeQuery = rangeQuery.AddMust(bluge.NewTermQuery(string(fieldKey.SeriesID.Marshal())).
-			SetField(seriesIDField))
-		rangeNode.Append(newTermNode(string(fieldKey.SeriesID.Marshal()), nil))
-		if termRange.Lower != nil || termRange.Upper != nil {
-			rangeQuery = addRange(rangeQuery, termRange)
+	rangeQuery = rangeQuery.AddMust(bluge.NewTermQuery(string(fieldKey.SeriesID.Marshal())).
+		SetField(seriesIDField))
+	rangeNode.Append(newTermNode(string(fieldKey.SeriesID.Marshal()), nil))
+	if !termRange.IsEmpty() && termRange.Valid() {
+		switch lower := termRange.Lower.(type) {
+		case *index.BytesTermValue:
+			upper := termRange.Upper.(*index.BytesTermValue)
+			rangeQuery.AddMust(bluge.NewTermRangeInclusiveQuery(
+				string(lower.Value),
+				string(upper.Value),
+				termRange.IncludesLower,
+				termRange.IncludesUpper,
+			).
+				SetField(fk))
+			rangeNode.Append(newTermRangeInclusiveNode(
+				string(lower.Value),
+				string(upper.Value),
+				termRange.IncludesLower,
+				termRange.IncludesUpper, nil,
+				false,
+			))
+
+		case *index.FloatTermValue:
+			upper := termRange.Upper.(*index.FloatTermValue)
+			rangeQuery.AddMust(bluge.NewNumericRangeInclusiveQuery(
+				lower.Value,
+				upper.Value,
+				termRange.IncludesLower,
+				termRange.IncludesUpper,
+			).
+				SetField(fk))
+			rangeNode.Append(newTermRangeInclusiveNode(
+				strconv.FormatFloat(lower.Value, 'f', -1, 64),
+				strconv.FormatFloat(upper.Value, 'f', -1, 64),
+				termRange.IncludesLower,
+				termRange.IncludesUpper,
+				nil,
+				false,
+			))
+		default:
+			logger.Panicf("unexpected field type: %T", lower)
 		}
-	} else {
-		rangeQuery = addRange(rangeQuery, termRange)
+	}
+	if n := appendTimeRangeToQuery(rangeQuery, fieldKey); n != nil {
+		rangeNode.Append(n)
 	}
 
 	sortedKey := fk
 	if order == modelv1.Sort_SORT_DESC {
 		sortedKey = "-" + sortedKey
 	}
-	query := bluge.NewBooleanQuery().AddMust(rangeQuery)
-	node := newMustNode()
-	node.Append(rangeNode)
-	if indexQuery != nil && indexQuery.(*queryNode).query != nil {
-		query.AddMust(indexQuery.(*queryNode).query)
-		node.Append(indexQuery.(*queryNode).node)
-	}
-	fields := make([]string, 0, len(fieldKeys))
-	for i := range fieldKeys {
-		fields = append(fields, fieldKeys[i].Marshal())
-	}
 	result := &sortIterator{
-		query:     &queryNode{query, node},
-		fields:    fields,
-		reader:    reader,
-		sortedKey: sortedKey,
-		size:      preLoadSize,
-		closer:    s.closer,
-		ctx:       ctx,
+		query:       &queryNode{rangeQuery, rangeNode},
+		reader:      reader,
+		sortedKey:   sortedKey,
+		size:        preLoadSize,
+		closer:      s.closer,
+		ctx:         ctx,
+		newIterator: newBlugeMatchIterator,
 	}
 	return result, nil
 }
 
-func (s *store) MatchField(fieldKey index.FieldKey) (list posting.List, err error) {
+func appendTimeRangeToQuery(query *bluge.BooleanQuery, fieldKey index.FieldKey) node {
+	if fieldKey.TimeRange == nil || !fieldKey.TimeRange.Valid() {
+		return nil
+	}
+	lower := numeric.Float64ToInt64(fieldKey.TimeRange.Lower.(*index.FloatTermValue).Value)
+	upper := numeric.Float64ToInt64(fieldKey.TimeRange.Upper.(*index.FloatTermValue).Value)
+	query.AddMust(bluge.NewDateRangeInclusiveQuery(
+		time.Unix(0, lower),
+		time.Unix(0, upper),
+		fieldKey.TimeRange.IncludesLower,
+		fieldKey.TimeRange.IncludesUpper,
+	).SetField(timestampField))
+	return newTermRangeInclusiveNode(
+		strconv.FormatInt(lower, 10),
+		strconv.FormatInt(upper, 10),
+		fieldKey.TimeRange.IncludesLower,
+		fieldKey.TimeRange.IncludesUpper,
+		nil,
+		true,
+	)
+}
+
+func (s *store) MatchField(fieldKey index.FieldKey) (posting.List, posting.List, error) {
 	return s.Range(fieldKey, index.RangeOpts{})
 }
 
-func (s *store) MatchTerms(field index.Field) (list posting.List, err error) {
+func (s *store) MatchTerms(field index.Field) (list posting.List, timestamps posting.List, err error) {
 	reader, err := s.writer.Reader()
 	if err != nil {
-		return nil, err
+		return nil, nil, err
 	}
-	fk := field.Key.Marshal()
-	query := bluge.NewBooleanQuery().
-		AddMust(bluge.NewTermQuery(string(field.Term)).SetField(fk))
-	if field.Key.HasSeriesID() {
-		query = query.AddMust(bluge.NewTermQuery(string(field.Key.SeriesID.Marshal())).
-			SetField(seriesIDField))
+
+	var query *bluge.BooleanQuery
+	switch field.GetTerm().(type) {
+	case *index.BytesTermValue:
+		query = bluge.NewBooleanQuery()
+		query.AddMust(bluge.NewTermQuery(string(field.GetBytes())).SetField(field.Key.Marshal()))
+	case *index.FloatTermValue:
+		query = bluge.NewBooleanQuery()
+		query.AddMust(bluge.NewTermQuery(
+			strconv.FormatFloat(field.GetFloat(), 'f', -1, 64)).
+			SetField(field.Key.Marshal()))
+	case nil:
+		return roaring.DummyPostingList, roaring.DummyPostingList, nil
+	default:
+		return nil, nil, errors.Errorf("unexpected field type: %T", field.GetTerm())
 	}
+	query.AddMust(bluge.NewTermQuery(string(field.Key.SeriesID.Marshal())).
+		SetField(seriesIDField))
+	_ = appendTimeRangeToQuery(query, field.Key)
+
 	documentMatchIterator, err := reader.Search(context.Background(), bluge.NewAllMatches(query))
 	if err != nil {
-		return nil, err
+		return nil, nil, err
 	}
 	iter := newBlugeMatchIterator(documentMatchIterator, reader, defaultProjection)
 	defer func() {
 		err = multierr.Append(err, iter.Close())
 	}()
-	list = roaring.NewPostingList()
+	list, timestamps = roaring.NewPostingList(), roaring.NewPostingList()
 	for iter.Next() {
 		list.Insert(iter.Val().DocID)
+		timestamps.Insert(uint64(iter.Val().Timestamp))
 	}
-	return list, err
+	return list, timestamps, err
 }
 
-func (s *store) Match(fieldKey index.FieldKey, matches []string, opts *modelv1.Condition_MatchOption) (posting.List, error) {
+func (s *store) Match(fieldKey index.FieldKey, matches []string, opts *modelv1.Condition_MatchOption) (posting.List, posting.List, error) {
 	if len(matches) == 0 || fieldKey.Analyzer == index.AnalyzerUnspecified {
-		return roaring.DummyPostingList, nil
+		return roaring.DummyPostingList, roaring.DummyPostingList, nil
 	}
 	reader, err := s.writer.Reader()
 	if err != nil {
-		return nil, err
+		return nil, nil, err
 	}
 	analyzer, operator := getMatchOptions(fieldKey.Analyzer, opts)
 	fk := fieldKey.Marshal()
 	query := bluge.NewBooleanQuery()
-	if fieldKey.HasSeriesID() {
-		query.AddMust(bluge.NewTermQuery(string(fieldKey.SeriesID.Marshal())).SetField(seriesIDField))
-	}
+	query.AddMust(bluge.NewTermQuery(string(fieldKey.SeriesID.Marshal())).SetField(seriesIDField))
 	for _, m := range matches {
 		query.AddMust(bluge.NewMatchQuery(m).SetField(fk).
 			SetAnalyzer(analyzer).SetOperator(operator))
 	}
+	_ = appendTimeRangeToQuery(query, fieldKey)
 	documentMatchIterator, err := reader.Search(context.Background(), bluge.NewAllMatches(query))
 	if err != nil {
-		return nil, err
+		return nil, nil, err
 	}
 	iter := newBlugeMatchIterator(documentMatchIterator, reader, defaultProjection)
 	defer func() {
 		err = multierr.Append(err, iter.Close())
 	}()
-	list := roaring.NewPostingList()
+	list, timestamps := roaring.NewPostingList(), roaring.NewPostingList()
 	for iter.Next() {
 		list.Insert(iter.Val().DocID)
+		timestamps.Insert(uint64(iter.Val().Timestamp))
 	}
-	return list, err
+	return list, timestamps, err
 }
 
 func getMatchOptions(analyzerOnIndexRule string, opts *modelv1.Condition_MatchOption) (*analysis.Analyzer, bluge.MatchQueryOperator) {
@@ -332,39 +380,48 @@ func getMatchOptions(analyzerOnIndexRule string, opts *modelv1.Condition_MatchOp
 	return analyzer, bluge.MatchQueryOperator(operator)
 }
 
-func (s *store) Range(fieldKey index.FieldKey, opts index.RangeOpts) (list posting.List, err error) {
-	iter, err := s.Iterator(context.TODO(), fieldKey, opts, modelv1.Sort_SORT_ASC, defaultRangePreloadSize, nil, nil)
+func (s *store) Range(fieldKey index.FieldKey, opts index.RangeOpts) (list posting.List, timestamps posting.List, err error) {
+	iter, err := s.Iterator(context.TODO(), fieldKey, opts, modelv1.Sort_SORT_ASC, defaultRangePreloadSize)
 	if err != nil {
-		return roaring.DummyPostingList, err
+		return roaring.DummyPostingList, roaring.DummyPostingList, err
 	}
-	list = roaring.NewPostingList()
+	list, timestamps = roaring.NewPostingList(), roaring.NewPostingList()
 	for iter.Next() {
 		list.Insert(iter.Val().DocID)
+		timestamps.Insert(uint64(iter.Val().Timestamp))
 	}
 	err = multierr.Append(err, iter.Close())
 	return
 }
 
+func (s *store) TakeFileSnapshot(dst string) error {
+	reader, err := s.writer.Reader()
+	if err != nil {
+		return err
+	}
+	defer reader.Close()
+	return reader.Backup(dst, nil)
+}
+
 type blugeMatchIterator struct {
-	delegated        search.DocumentMatchIterator
-	err              error
-	closer           io.Closer
-	needToLoadFields []string
-	current          index.DocumentResult
-	hit              int
+	delegated     search.DocumentMatchIterator
+	err           error
+	closer        io.Closer
+	ctx           *search.Context
+	loadDocValues []string
+	current       index.DocumentResult
+	hit           int
 }
 
 func newBlugeMatchIterator(delegated search.DocumentMatchIterator, closer io.Closer,
-	needToLoadFields []string,
-) blugeMatchIterator {
-	bmi := blugeMatchIterator{
-		delegated:        delegated,
-		closer:           closer,
-		needToLoadFields: needToLoadFields,
-		current:          index.DocumentResult{Values: make(map[string][]byte, len(needToLoadFields))},
-	}
-	for _, f := range needToLoadFields {
-		bmi.current.Values[f] = nil
+	loadDocValues []string,
+) blugeIterator {
+	bmi := &blugeMatchIterator{
+		delegated:     delegated,
+		closer:        closer,
+		current:       index.DocumentResult{},
+		ctx:           search.NewSearchContext(1, 0),
+		loadDocValues: loadDocValues,
 	}
 	return bmi
 }
@@ -391,30 +448,42 @@ func (bmi *blugeMatchIterator) Next() bool {
 	if len(match.SortValue) > 0 {
 		bmi.current.SortedValue = match.SortValue[0]
 	}
-	err := match.VisitStoredFields(func(field string, value []byte) bool {
-		switch field {
-		case entityField:
-			bmi.current.EntityValues = value
-		case docIDField:
-			bmi.current.DocID = convert.BytesToUint64(value)
-		case seriesIDField:
-			bmi.current.SeriesID = common.SeriesID(convert.BytesToUint64(value))
-		case timestampField:
-			ts, err := bluge.DecodeDateTime(value)
-			if err != nil {
-				bmi.err = err
-				return false
-			}
-			bmi.current.Timestamp = ts.UnixNano()
-		default:
-			if _, ok := bmi.current.Values[field]; ok {
-				bmi.current.Values[field] = bytes.Clone(value)
-			}
+	if len(bmi.loadDocValues) == 0 {
+		err := match.VisitStoredFields(bmi.setVal)
+		bmi.err = multierr.Combine(bmi.err, err)
+		return bmi.err == nil
+	}
+	if err := match.LoadDocumentValues(bmi.ctx, bmi.loadDocValues); err != nil {
+		bmi.err = multierr.Combine(bmi.err, err)
+		return false
+	}
+	for _, dv := range bmi.loadDocValues {
+		vv := match.DocValues(dv)
+		if len(vv) == 0 {
+			continue
 		}
-		return true
-	})
-	bmi.err = errors.WithMessagef(err, "visit stored fields, hit: %d", bmi.hit)
-	return bmi.err == nil
+		bmi.setVal(dv, vv[0])
+	}
+	return true
+}
+
+func (bmi *blugeMatchIterator) setVal(field string, value []byte) bool {
+	switch field {
+	case docIDField:
+		bmi.current.DocID = convert.BytesToUint64(value)
+	case seriesIDField:
+		bmi.current.SeriesID = common.SeriesID(convert.BytesToUint64(value))
+	case timestampField:
+		ts, errTime := bluge.DecodeDateTime(value)
+		if errTime != nil {
+			bmi.err = errTime
+			return false
+		}
+		bmi.current.Timestamp = ts.UnixNano()
+	default:
+		bmi.err = errors.Errorf("unexpected field: %s", field)
+	}
+	return true
 }
 
 func (bmi *blugeMatchIterator) Val() index.DocumentResult {
