@@ -31,15 +31,18 @@ import (
 	"github.com/apache/skywalking-banyandb/pkg/logger"
 	pbv1 "github.com/apache/skywalking-banyandb/pkg/pb/v1"
 	"github.com/apache/skywalking-banyandb/pkg/query"
+	"github.com/apache/skywalking-banyandb/pkg/timestamp"
 )
 
+const seriesIndexDirName = "sidx"
+
 func (s *segment[T, O]) IndexDB() IndexDB {
 	return s.index
 }
 
 func (s *segment[T, O]) Lookup(ctx context.Context, series []*pbv1.Series) (pbv1.SeriesList, error) {
-	sl, _, err := s.index.filter(ctx, series, nil, nil)
-	return sl, err
+	sl, err := s.index.filter(ctx, series, nil, nil, nil)
+	return sl.SeriesList, err
 }
 
 type seriesIndex struct {
@@ -49,15 +52,18 @@ type seriesIndex struct {
 	p       common.Position
 }
 
-func newSeriesIndex(ctx context.Context, root string, flushTimeoutSeconds int64, metrics *inverted.Metrics) (*seriesIndex, error) {
+func newSeriesIndex(ctx context.Context, root string, flushTimeoutSeconds int64, cacheMaxBytes int,
+	metrics *inverted.Metrics,
+) (*seriesIndex, error) {
 	si := &seriesIndex{
 		l: logger.Fetch(ctx, "series_index"),
 		p: common.GetPosition(ctx),
 	}
 	opts := inverted.StoreOpts{
-		Path:         path.Join(root, "sidx"),
-		Logger:       si.l,
-		BatchWaitSec: flushTimeoutSeconds,
+		Path:          path.Join(root, seriesIndexDirName),
+		Logger:        si.l,
+		BatchWaitSec:  flushTimeoutSeconds,
+		CacheMaxBytes: cacheMaxBytes,
 	}
 	if metrics != nil {
 		opts.Metrics = metrics
@@ -70,36 +76,47 @@ func newSeriesIndex(ctx context.Context, root string, flushTimeoutSeconds int64,
 	return si, nil
 }
 
-func (s *seriesIndex) Write(docs index.Documents) error {
-	return s.store.Batch(index.Batch{
+func (s *seriesIndex) Insert(docs index.Documents) error {
+	return s.store.InsertSeriesBatch(index.Batch{
 		Documents: docs,
 	})
 }
 
-var rangeOpts = index.RangeOpts{}
+func (s *seriesIndex) Update(docs index.Documents) error {
+	return s.store.UpdateSeriesBatch(index.Batch{
+		Documents: docs,
+	})
+}
 
 func (s *seriesIndex) filter(ctx context.Context, series []*pbv1.Series,
-	projection []index.FieldKey, secondaryQuery index.Query,
-) (sl pbv1.SeriesList, fields FieldResultList, err error) {
-	seriesMatchers := make([]index.SeriesMatcher, len(series))
-	for i := range series {
-		seriesMatchers[i], err = convertEntityValuesToSeriesMatcher(series[i])
-		if err != nil {
-			return nil, nil, err
+	projection []index.FieldKey, secondaryQuery index.Query, timeRange *timestamp.TimeRange,
+) (data SeriesData, err error) {
+	if len(series) == 0 && secondaryQuery == nil {
+		return data, nil
+	}
+	var seriesMatchers []index.SeriesMatcher
+	if len(series) > 0 {
+		seriesMatchers = make([]index.SeriesMatcher, len(series))
+		for i := range series {
+			seriesMatchers[i], err = convertEntityValuesToSeriesMatcher(series[i])
+			if err != nil {
+				return SeriesData{}, err
+			}
 		}
 	}
-	indexQuery, err := s.store.BuildQuery(seriesMatchers, secondaryQuery)
+
+	indexQuery, err := s.store.BuildQuery(seriesMatchers, secondaryQuery, timeRange)
 	if err != nil {
-		return nil, nil, err
+		return SeriesData{}, err
 	}
 	tracer := query.GetTracer(ctx)
 	if tracer != nil {
 		span, _ := tracer.StartSpan(ctx, "seriesIndex.search")
 		span.Tagf("query", "%s", indexQuery.String())
 		defer func() {
-			span.Tagf("matched", "%d", len(sl))
-			if len(fields) > 0 {
-				span.Tagf("field_length", "%d", len(fields[0]))
+			span.Tagf("matched", "%d", len(data.SeriesList))
+			if len(data.Fields) > 0 {
+				span.Tagf("field_length", "%d", len(data.Fields[0]))
 			}
 			if err != nil {
 				span.Error(err)
@@ -107,15 +124,18 @@ func (s *seriesIndex) filter(ctx context.Context, series []*pbv1.Series,
 			span.Stop()
 		}()
 	}
-	ss, err := s.store.Search(ctx, projection, indexQuery)
+	ss, err := s.store.Search(ctx, projection, indexQuery, 0)
 	if err != nil {
-		return nil, nil, err
+		return SeriesData{}, err
 	}
-	sl, fields, err = convertIndexSeriesToSeriesList(ss, len(projection) > 0)
+	if len(ss) == 0 {
+		return SeriesData{}, nil
+	}
+	data.SeriesList, data.Fields, data.Timestamps, data.Versions, err = convertIndexSeriesToSeriesList(ss, len(projection) > 0)
 	if err != nil {
-		return nil, nil, errors.WithMessagef(err, "failed to convert index series to series list, matchers: %v, matched: %d", seriesMatchers, len(ss))
+		return SeriesData{}, errors.WithMessagef(err, "failed to convert index series to series list, matchers: %v, matched: %d", seriesMatchers, len(ss))
 	}
-	return sl, fields, nil
+	return data, nil
 }
 
 var emptySeriesMatcher = index.SeriesMatcher{}
@@ -173,31 +193,41 @@ func convertEntityValuesToSeriesMatcher(series *pbv1.Series) (index.SeriesMatche
 	}, nil
 }
 
-func convertIndexSeriesToSeriesList(indexSeries []index.SeriesDocument, hasFields bool) (pbv1.SeriesList, FieldResultList, error) {
+func convertIndexSeriesToSeriesList(indexSeries []index.SeriesDocument, hasFields bool) (pbv1.SeriesList, FieldResultList, []int64, []int64, error) {
 	seriesList := make(pbv1.SeriesList, 0, len(indexSeries))
 	var fields FieldResultList
 	if hasFields {
 		fields = make(FieldResultList, 0, len(indexSeries))
 	}
+	var timestamps, versions []int64
 	for _, s := range indexSeries {
 		var series pbv1.Series
-		series.ID = s.Key.ID
 		if err := series.Unmarshal(s.Key.EntityValues); err != nil {
-			return nil, nil, errors.WithMessagef(err, "failed to unmarshal series: %s", s.Key.EntityValues)
+			return nil, nil, nil, nil, errors.WithMessagef(err, "failed to unmarshal series: %s", s.Key.EntityValues)
 		}
 		seriesList = append(seriesList, &series)
 		if fields != nil {
 			fields = append(fields, s.Fields)
 		}
+		if s.Timestamp > 0 {
+			timestamps = append(timestamps, s.Timestamp)
+		}
+		if s.Version > 0 {
+			versions = append(versions, s.Version)
+		}
 	}
-	return seriesList, fields, nil
+	return seriesList, fields, timestamps, versions, nil
 }
 
-func (s *seriesIndex) Search(ctx context.Context, series []*pbv1.Series, opts IndexSearchOpts) (sl pbv1.SeriesList, frl FieldResultList, err error) {
+func (s *seriesIndex) Search(ctx context.Context, series []*pbv1.Series, opts IndexSearchOpts,
+) (sd SeriesData, sortedValues [][]byte, err error) {
 	tracer := query.GetTracer(ctx)
 	if tracer != nil {
 		var span *query.Span
 		span, ctx = tracer.StartSpan(ctx, "seriesIndex.Search")
+		if opts.Query != nil {
+			span.Tagf("secondary_query", "%s", opts.Query.String())
+		}
 		defer func() {
 			if err != nil {
 				span.Error(err)
@@ -207,21 +237,11 @@ func (s *seriesIndex) Search(ctx context.Context, series []*pbv1.Series, opts In
 	}
 
 	if opts.Order == nil || opts.Order.Index == nil {
-		var seriesList pbv1.SeriesList
-		var fieldResultList FieldResultList
-		if opts.Query != nil {
-			seriesList, fieldResultList, err = s.filter(ctx, series, opts.Projection, opts.Query)
-		} else {
-			seriesList, fieldResultList, err = s.filter(ctx, series, opts.Projection, nil)
-		}
+		sd, err = s.filter(ctx, series, opts.Projection, opts.Query, opts.TimeRange)
 		if err != nil {
-			return nil, nil, err
+			return sd, nil, err
 		}
-		return seriesList, fieldResultList, nil
-	}
-
-	fieldKey := index.FieldKey{
-		IndexRuleID: opts.Order.Index.GetMetadata().Id,
+		return sd, nil, nil
 	}
 	var span *query.Span
 	if tracer != nil {
@@ -238,43 +258,112 @@ func (s *seriesIndex) Search(ctx context.Context, series []*pbv1.Series, opts In
 	for i := range series {
 		seriesMatchers[i], err = convertEntityValuesToSeriesMatcher(series[i])
 		if err != nil {
-			return nil, nil, err
+			return sd, nil, err
 		}
 	}
-	query, err := s.store.BuildQuery(seriesMatchers, opts.Query)
+	query, err := s.store.BuildQuery(seriesMatchers, opts.Query, opts.TimeRange)
 	if err != nil {
-		return nil, nil, err
+		return sd, nil, err
 	}
-	iter, err := s.store.Iterator(ctx, fieldKey, rangeOpts,
-		opts.Order.Sort, opts.PreloadSize, query, opts.Projection)
+	iter, err := s.store.SeriesSort(ctx, query, opts.Order,
+		opts.PreloadSize, opts.Projection)
 	if err != nil {
-		return nil, nil, err
+		return sd, nil, err
 	}
 	defer func() {
 		err = multierr.Append(err, iter.Close())
 	}()
 
 	var r int
-	result := make([]index.SeriesDocument, 0, 10)
 	for iter.Next() {
 		r++
 		val := iter.Val()
-		var doc index.SeriesDocument
-		doc.Fields = maps.Clone(val.Values)
-		doc.Key.ID = common.SeriesID(val.DocID)
-		doc.Key.EntityValues = val.EntityValues
-		result = append(result, doc)
-	}
-	sortedSeriesList, sortedFieldResultList, err := convertIndexSeriesToSeriesList(result, len(opts.Projection) > 0)
-	if err != nil {
-		return nil, nil, errors.WithMessagef(err, "failed to convert index series to series list, matchers: %v, matched: %d", seriesMatchers, len(result))
+		var series pbv1.Series
+		if err = series.Unmarshal(val.EntityValues); err != nil {
+			return sd, nil, errors.WithMessagef(err, "failed to unmarshal series: %s", val.EntityValues)
+		}
+		sd.SeriesList = append(sd.SeriesList, &series)
+		sd.Timestamps = append(sd.Timestamps, val.Timestamp)
+		sd.Versions = append(sd.Versions, val.Version)
+		if len(opts.Projection) > 0 {
+			sd.Fields = append(sd.Fields, maps.Clone(val.Values))
+		}
+		sortedValues = append(sortedValues, val.SortedValue)
 	}
 	if span != nil {
 		span.Tagf("query", "%s", iter.Query().String())
 		span.Tagf("rounds", "%d", r)
-		span.Tagf("size", "%d", len(sortedSeriesList))
+		span.Tagf("size", "%d", len(sd.SeriesList))
 	}
-	return sortedSeriesList, sortedFieldResultList, err
+	return sd, sortedValues, err
+}
+
+func (s *seriesIndex) SearchWithoutSeries(ctx context.Context, opts IndexSearchOpts) (sd SeriesData, sortedValues [][]byte, err error) {
+	tracer := query.GetTracer(ctx)
+	if tracer != nil {
+		var span *query.Span
+		span, ctx = tracer.StartSpan(ctx, "seriesIndex.SearchWithoutSeries")
+		if opts.Query != nil {
+			span.Tagf("secondary_query", "%s", opts.Query.String())
+		}
+		defer func() {
+			if err != nil {
+				span.Error(err)
+			}
+			span.Stop()
+		}()
+	}
+
+	if opts.Order == nil || opts.Order.Index == nil {
+		sd, err = s.filter(ctx, nil, opts.Projection, opts.Query, opts.TimeRange)
+		if err != nil {
+			return sd, nil, err
+		}
+		return sd, nil, nil
+	}
+	var span *query.Span
+	if tracer != nil {
+		span, _ = tracer.StartSpan(ctx, "sort")
+		span.Tagf("preload", "%d", opts.PreloadSize)
+		defer func() {
+			if err != nil {
+				span.Error(err)
+			}
+			span.Stop()
+		}()
+	}
+
+	iter, err := s.store.SeriesSort(ctx, opts.Query, opts.Order,
+		opts.PreloadSize, opts.Projection)
+	if err != nil {
+		return sd, nil, err
+	}
+	defer func() {
+		err = multierr.Append(err, iter.Close())
+	}()
+
+	var r int
+	for iter.Next() {
+		r++
+		val := iter.Val()
+		var series pbv1.Series
+		if err = series.Unmarshal(val.EntityValues); err != nil {
+			return sd, nil, errors.WithMessagef(err, "failed to unmarshal series: %s", val.EntityValues)
+		}
+		sd.SeriesList = append(sd.SeriesList, &series)
+		sd.Timestamps = append(sd.Timestamps, val.Timestamp)
+		sd.Versions = append(sd.Versions, val.Version)
+		if len(opts.Projection) > 0 {
+			sd.Fields = append(sd.Fields, maps.Clone(val.Values))
+		}
+		sortedValues = append(sortedValues, val.SortedValue)
+	}
+	if span != nil {
+		span.Tagf("query", "%s", iter.Query().String())
+		span.Tagf("rounds", "%d", r)
+		span.Tagf("size", "%d", len(sd.SeriesList))
+	}
+	return sd, sortedValues, err
 }
 
 func (s *seriesIndex) Close() error {
