@@ -43,46 +43,58 @@ func (d *database[T, O]) Tick(ts int64) {
 }
 
 func (d *database[T, O]) startRotationTask() error {
-	rt := newRetentionTask(d, d.opts.TTL)
+	options := d.segmentController.getOptions()
+	var rt *retentionTask[T, O]
+	if !d.disableRetention {
+		rt = newRetentionTask(d, options.TTL)
+	}
 	go func(rt *retentionTask[T, O]) {
 		for ts := range d.tsEventCh {
 			func(ts int64) {
 				d.rotationProcessOn.Store(true)
 				defer d.rotationProcessOn.Store(false)
 				t := time.Unix(0, ts)
-				rt.run(t, d.logger)
-				shardsRef := d.sLst.Load()
-				if shardsRef == nil {
-					return
+				if rt != nil {
+					rt.run(t, d.logger)
 				}
-				for _, s := range *shardsRef {
-					func(s *shard[T, O]) {
-						ss := s.segmentController.segments()
-						if len(ss) == 0 {
-							return
+				func() {
+					ss := d.segmentController.segments()
+					if len(ss) == 0 {
+						return
+					}
+					defer func() {
+						for i := 0; i < len(ss); i++ {
+							ss[i].DecRef()
 						}
-						defer func() {
-							for i := 0; i < len(ss); i++ {
-								ss[i].DecRef()
-							}
-						}()
-						latest := ss[len(ss)-1]
-						gap := latest.End.UnixNano() - ts
-						// gap <=0 means the event is from the future
-						// the segment will be created by a written event directly
-						if gap <= 0 || gap > newSegmentTimeGap {
-							return
+					}()
+					for i := range ss {
+						if ss[i].End.UnixNano() < ts {
+							ss[i].index.store.Reset()
 						}
-						d.logger.Info().Time("segment_start", s.segmentController.segmentSize.nextTime(t)).Time("event_time", t).Msg("create new segment")
-						_, err := s.segmentController.create(s.segmentController.segmentSize.nextTime(t))
-						if err != nil {
-							d.logger.Error().Err(err).Msgf("failed to create new segment.")
-						}
-					}(s)
-				}
+					}
+					latest := ss[len(ss)-1]
+					gap := latest.End.UnixNano() - ts
+					// gap <=0 means the event is from the future
+					// the segment will be created by a written event directly
+					if gap <= 0 || gap > newSegmentTimeGap {
+						return
+					}
+					d.incTotalRotationStarted(1)
+					defer d.incTotalRotationFinished(1)
+					start := options.SegmentInterval.nextTime(t)
+					d.logger.Info().Time("segment_start", start).Time("event_time", t).Msg("create new segment")
+					_, err := d.segmentController.create(start)
+					if err != nil {
+						d.logger.Error().Err(err).Msgf("failed to create new segment.")
+						d.incTotalRotationErr(1)
+					}
+				}()
 			}(ts)
 		}
 	}(rt)
+	if rt == nil {
+		return nil
+	}
 	return d.scheduler.Register("retention", rt.option, rt.expr, rt.run)
 }
 
@@ -115,20 +127,18 @@ func (rc *retentionTask[T, O]) run(now time.Time, l *logger.Logger) bool {
 		<-rc.running
 	}()
 
-	shardList := rc.database.sLst.Load()
-	if shardList == nil {
-		return false
-	}
+	rc.database.incTotalRetentionStarted(1)
+	defer rc.database.incTotalRetentionFinished(1)
 	deadline := now.Add(-rc.duration)
-
-	for _, shard := range *shardList {
-		if err := shard.segmentController.remove(deadline); err != nil {
-			l.Error().Err(err)
-		}
+	start := time.Now()
+	hasData, err := rc.database.segmentController.remove(deadline)
+	if hasData {
+		rc.database.incTotalRetentionHasData(1)
+		rc.database.incTotalRetentionHasDataLatency(time.Since(start).Seconds())
 	}
-	stdDeadline := rc.database.opts.TTL.Unit.standard(deadline)
-	if err := rc.database.indexController.run(now, stdDeadline); err != nil {
+	if err != nil {
 		l.Error().Err(err)
+		rc.database.incTotalRetentionErr(1)
 	}
 	return true
 }
