@@ -30,8 +30,11 @@ import (
 	"time"
 
 	"github.com/pkg/errors"
+	"google.golang.org/protobuf/types/known/timestamppb"
 
 	"github.com/apache/skywalking-banyandb/api/common"
+	commonv1 "github.com/apache/skywalking-banyandb/api/proto/banyandb/common/v1"
+	modelv1 "github.com/apache/skywalking-banyandb/api/proto/banyandb/model/v1"
 	"github.com/apache/skywalking-banyandb/pkg/convert"
 	"github.com/apache/skywalking-banyandb/pkg/index/inverted"
 	"github.com/apache/skywalking-banyandb/pkg/logger"
@@ -43,6 +46,8 @@ var ErrExpiredData = errors.New("expired data")
 
 type segment[T TSTable, O any] struct {
 	metrics  any
+	option   O
+	creator  TSTableCreator[T, O]
 	l        *logger.Logger
 	index    *seriesIndex
 	sLst     atomic.Pointer[[]*shard[T]]
@@ -50,7 +55,6 @@ type segment[T TSTable, O any] struct {
 	timestamp.TimeRange
 	suffix        string
 	location      string
-	opts          TSDBOpts[T, O]
 	mu            sync.Mutex
 	refCount      int32
 	mustBeDeleted uint32
@@ -58,7 +62,6 @@ type segment[T TSTable, O any] struct {
 }
 
 func (sc *segmentController[T, O]) openSegment(ctx context.Context, startTime, endTime time.Time, path, suffix string,
-	opts TSDBOpts[T, O],
 ) (s *segment[T, O], err error) {
 	suffixInteger, err := strconv.Atoi(suffix)
 	if err != nil {
@@ -69,8 +72,9 @@ func (sc *segmentController[T, O]) openSegment(ctx context.Context, startTime, e
 	ctx = common.SetPosition(ctx, func(_ common.Position) common.Position {
 		return p
 	})
-	id := generateSegID(sc.opts.SegmentInterval.Unit, suffixInteger)
-	sir, err := newSeriesIndex(ctx, path, sc.opts.SeriesIndexFlushTimeoutSeconds, sc.indexMetrics)
+	options := sc.getOptions()
+	id := generateSegID(options.SegmentInterval.Unit, suffixInteger)
+	sir, err := newSeriesIndex(ctx, path, options.SeriesIndexFlushTimeoutSeconds, options.SeriesIndexCacheMaxBytes, sc.indexMetrics)
 	if err != nil {
 		return nil, errors.Wrap(errOpenDatabase, errors.WithMessage(err, "create series index controller failed").Error())
 	}
@@ -82,20 +86,21 @@ func (sc *segmentController[T, O]) openSegment(ctx context.Context, startTime, e
 		position:  p,
 		refCount:  1,
 		index:     sir,
-		opts:      opts,
 		metrics:   sc.metrics,
+		creator:   options.TSTableCreator,
+		option:    options.Option,
 	}
 	s.l = logger.Fetch(ctx, s.String())
-	return s, s.loadShards()
+	return s, s.loadShards(int(options.ShardNum))
 }
 
-func (s *segment[T, O]) loadShards() error {
+func (s *segment[T, O]) loadShards(shardNum int) error {
 	return walkDir(s.location, shardPathPrefix, func(suffix string) error {
 		shardID, err := strconv.Atoi(suffix)
 		if err != nil {
 			return err
 		}
-		if shardID >= int(s.opts.ShardNum) {
+		if shardID >= shardNum {
 			return nil
 		}
 		s.l.Info().Int("shard_id", shardID).Msg("loaded a existed shard")
@@ -201,39 +206,70 @@ func (s *segment[T, O]) String() string {
 }
 
 type segmentController[T TSTable, O any] struct {
-	clock        timestamp.Clock
-	metrics      Metrics
-	l            *logger.Logger
-	indexMetrics *inverted.Metrics
-	position     common.Position
-	location     string
-	lst          []*segment[T, O]
-	opts         TSDBOpts[T, O]
-	deadline     atomic.Int64
+	clock                   timestamp.Clock
+	metrics                 Metrics
+	opts                    *TSDBOpts[T, O]
+	l                       *logger.Logger
+	indexMetrics            *inverted.Metrics
+	segmentBoundaryUpdateFn SegmentBoundaryUpdateFn
+	position                common.Position
+	db                      string
+	stage                   string
+	location                string
+	lst                     []*segment[T, O]
+	deadline                atomic.Int64
+	optsMutex               sync.RWMutex
 	sync.RWMutex
 }
 
 func newSegmentController[T TSTable, O any](ctx context.Context, location string,
 	l *logger.Logger, opts TSDBOpts[T, O], indexMetrics *inverted.Metrics, metrics Metrics,
+	segmentsBoundaryUpdateFn SegmentBoundaryUpdateFn,
 ) *segmentController[T, O] {
 	clock, _ := timestamp.GetClock(ctx)
+	p := common.GetPosition(ctx)
 	return &segmentController[T, O]{
-		location:     location,
-		opts:         opts,
-		l:            l,
-		clock:        clock,
-		position:     common.GetPosition(ctx),
-		metrics:      metrics,
-		indexMetrics: indexMetrics,
+		location:                location,
+		opts:                    &opts,
+		l:                       l,
+		clock:                   clock,
+		position:                common.GetPosition(ctx),
+		metrics:                 metrics,
+		indexMetrics:            indexMetrics,
+		segmentBoundaryUpdateFn: segmentsBoundaryUpdateFn,
+		stage:                   p.Stage,
+		db:                      p.Database,
 	}
 }
 
+func (sc *segmentController[T, O]) getOptions() *TSDBOpts[T, O] {
+	sc.optsMutex.RLock()
+	defer sc.optsMutex.RUnlock()
+	return sc.opts
+}
+
+func (sc *segmentController[T, O]) updateOptions(resourceOpts *commonv1.ResourceOpts) {
+	sc.optsMutex.Lock()
+	defer sc.optsMutex.Unlock()
+	si := MustToIntervalRule(resourceOpts.SegmentInterval)
+	if sc.opts.SegmentInterval.Unit != si.Unit {
+		sc.l.Panic().Msg("segment interval unit cannot be changed")
+		return
+	}
+	sc.opts.SegmentInterval = si
+	sc.opts.TTL = MustToIntervalRule(resourceOpts.Ttl)
+	sc.opts.ShardNum = resourceOpts.ShardNum
+}
+
 func (sc *segmentController[T, O]) selectSegments(timeRange timestamp.TimeRange) (tt []Segment[T, O]) {
 	sc.RLock()
 	defer sc.RUnlock()
 	last := len(sc.lst) - 1
 	for i := range sc.lst {
 		s := sc.lst[last-i]
+		if s.GetTimeRange().End.Before(timeRange.Start) {
+			break
+		}
 		if s.Overlapping(timeRange) {
 			s.incRef()
 			tt = append(tt, s)
@@ -252,9 +288,30 @@ func (sc *segmentController[T, O]) createSegment(ts time.Time) (*segment[T, O],
 		return nil, err
 	}
 	s.incRef()
+	sc.notifySegmentBoundaryUpdate()
 	return s, nil
 }
 
+func (sc *segmentController[T, O]) notifySegmentBoundaryUpdate() {
+	if sc.segmentBoundaryUpdateFn == nil {
+		return
+	}
+	segs := sc.segments()
+	defer func() {
+		for i := range segs {
+			segs[i].DecRef()
+		}
+	}()
+	var tr *modelv1.TimeRange
+	if len(segs) > 0 {
+		tr = &modelv1.TimeRange{
+			Begin: timestamppb.New(segs[0].Start),
+			End:   timestamppb.New(segs[len(segs)-1].End),
+		}
+	}
+	sc.segmentBoundaryUpdateFn(sc.stage, sc.db, tr)
+}
+
 func (sc *segmentController[T, O]) segments() (ss []*segment[T, O]) {
 	sc.RLock()
 	defer sc.RUnlock()
@@ -267,7 +324,7 @@ func (sc *segmentController[T, O]) segments() (ss []*segment[T, O]) {
 }
 
 func (sc *segmentController[T, O]) format(tm time.Time) string {
-	switch sc.opts.SegmentInterval.Unit {
+	switch sc.getOptions().SegmentInterval.Unit {
 	case HOUR:
 		return tm.Format(hourFormat)
 	case DAY:
@@ -277,7 +334,7 @@ func (sc *segmentController[T, O]) format(tm time.Time) string {
 }
 
 func (sc *segmentController[T, O]) parse(value string) (time.Time, error) {
-	switch sc.opts.SegmentInterval.Unit {
+	switch sc.getOptions().SegmentInterval.Unit {
 	case HOUR:
 		return time.ParseInLocation(hourFormat, value, time.Local)
 	case DAY:
@@ -290,7 +347,7 @@ func (sc *segmentController[T, O]) open() error {
 	sc.Lock()
 	defer sc.Unlock()
 	emptySegments := make([]string, 0)
-	err := loadSegments(sc.location, segPathPrefix, sc, sc.opts.SegmentInterval, func(start, end time.Time) error {
+	err := loadSegments(sc.location, segPathPrefix, sc, sc.getOptions().SegmentInterval, func(start, end time.Time) error {
 		suffix := sc.format(start)
 		segmentPath := path.Join(sc.location, fmt.Sprintf(segTemplate, suffix))
 		metadataPath := path.Join(segmentPath, metadataFilename)
@@ -331,7 +388,8 @@ func (sc *segmentController[T, O]) create(start time.Time) (*segment[T, O], erro
 			return s, nil
 		}
 	}
-	start = sc.opts.SegmentInterval.Unit.standard(start)
+	options := sc.getOptions()
+	start = options.SegmentInterval.Unit.standard(start)
 	var next *segment[T, O]
 	for _, s := range sc.lst {
 		if s.Contains(start.UnixNano()) {
@@ -341,7 +399,7 @@ func (sc *segmentController[T, O]) create(start time.Time) (*segment[T, O], erro
 			next = s
 		}
 	}
-	stdEnd := sc.opts.SegmentInterval.nextTime(start)
+	stdEnd := options.SegmentInterval.nextTime(start)
 	var end time.Time
 	if next != nil && next.Start.Before(stdEnd) {
 		end = next.Start
@@ -349,10 +407,10 @@ func (sc *segmentController[T, O]) create(start time.Time) (*segment[T, O], erro
 		end = stdEnd
 	}
 	segPath := path.Join(sc.location, fmt.Sprintf(segTemplate, sc.format(start)))
-	lfs.MkdirPanicIfExist(segPath, dirPerm)
+	lfs.MkdirPanicIfExist(segPath, DirPerm)
 	data := []byte(currentVersion)
 	metadataPath := filepath.Join(segPath, metadataFilename)
-	lf, err := lfs.CreateLockFile(metadataPath, filePermission)
+	lf, err := lfs.CreateLockFile(metadataPath, FilePerm)
 	if err != nil {
 		logger.Panicf("cannot create lock file %s: %s", metadataPath, err)
 	}
@@ -378,7 +436,7 @@ func (sc *segmentController[T, O]) load(start, end time.Time, root string) (seg
 	ctx := common.SetPosition(context.WithValue(context.Background(), logger.ContextKey, sc.l), func(_ common.Position) common.Position {
 		return sc.position
 	})
-	seg, err = sc.openSegment(ctx, start, end, segPath, suffix, sc.opts)
+	seg, err = sc.openSegment(ctx, start, end, segPath, suffix)
 	if err != nil {
 		return nil, err
 	}
@@ -400,9 +458,45 @@ func (sc *segmentController[T, O]) remove(deadline time.Time) (hasSegment bool,
 		}
 		s.DecRef()
 	}
+	sc.notifySegmentBoundaryUpdate()
 	return hasSegment, err
 }
 
+func (sc *segmentController[T, O]) getExpiredSegmentsTimeRange() *timestamp.TimeRange {
+	deadline := time.Now().Local().Add(-sc.opts.TTL.estimatedDuration())
+	timeRange := &timestamp.TimeRange{
+		IncludeStart: true,
+		IncludeEnd:   false,
+	}
+	for _, s := range sc.segments() {
+		if s.Before(deadline) {
+			if timeRange.Start.IsZero() {
+				timeRange.Start = s.Start
+			}
+			timeRange.End = s.End
+		}
+		s.DecRef()
+	}
+	return timeRange
+}
+
+func (sc *segmentController[T, O]) deleteExpiredSegments(timeRange timestamp.TimeRange) int64 {
+	deadline := time.Now().Local().Add(-sc.opts.TTL.estimatedDuration())
+	var count int64
+	for _, s := range sc.segments() {
+		if s.Before(deadline) && s.Overlapping(timeRange) {
+			s.delete()
+			sc.Lock()
+			sc.removeSeg(s.id)
+			sc.Unlock()
+			count++
+		}
+		s.DecRef()
+	}
+	sc.notifySegmentBoundaryUpdate()
+	return count
+}
+
 func (sc *segmentController[T, O]) removeSeg(segID segmentID) {
 	for i, b := range sc.lst {
 		if b.id == segID {
