@@ -36,7 +36,6 @@ import (
 	"github.com/apache/skywalking-banyandb/pkg/index"
 	"github.com/apache/skywalking-banyandb/pkg/logger"
 	pbv1 "github.com/apache/skywalking-banyandb/pkg/pb/v1"
-	"github.com/apache/skywalking-banyandb/pkg/query/model"
 	"github.com/apache/skywalking-banyandb/pkg/timestamp"
 )
 
@@ -50,7 +49,14 @@ const (
 	hourFormat = "2006010215"
 	dayFormat  = "20060102"
 
-	dirPerm = 0o700
+	// DirPerm is the permission of the directory.
+	DirPerm = 0o700
+	// SnapshotsDir is the directory for snapshots.
+	SnapshotsDir = "snapshots"
+	// DataDir is the directory for data.
+	DataDir = "data"
+	// FilePerm is the permission of the file.
+	FilePerm = 0o600
 )
 
 var (
@@ -67,7 +73,8 @@ type SupplyTSDB[T TSTable] func() T
 // IndexSearchOpts is the options for searching index.
 type IndexSearchOpts struct {
 	Query       index.Query
-	Order       *model.OrderBy
+	Order       *index.OrderBy
+	TimeRange   *timestamp.TimeRange
 	Projection  []index.FieldKey
 	PreloadSize int
 }
@@ -78,10 +85,20 @@ type FieldResult map[string][]byte
 // FieldResultList is a list of FieldResult.
 type FieldResultList []FieldResult
 
+// SeriesData is the result of a series.
+type SeriesData struct {
+	SeriesList pbv1.SeriesList
+	Fields     FieldResultList
+	Timestamps []int64
+	Versions   []int64
+}
+
 // IndexDB is the interface of index database.
 type IndexDB interface {
-	Write(docs index.Documents) error
-	Search(ctx context.Context, series []*pbv1.Series, opts IndexSearchOpts) (pbv1.SeriesList, FieldResultList, error)
+	Insert(docs index.Documents) error
+	Update(docs index.Documents) error
+	Search(ctx context.Context, series []*pbv1.Series, opts IndexSearchOpts) (SeriesData, [][]byte, error)
+	SearchWithoutSeries(ctx context.Context, opts IndexSearchOpts) (sd SeriesData, sortedValues [][]byte, err error)
 }
 
 // TSDB allows listing and getting shard details.
@@ -90,6 +107,10 @@ type TSDB[T TSTable, O any] interface {
 	CreateSegmentIfNotExist(ts time.Time) (Segment[T, O], error)
 	SelectSegments(timeRange timestamp.TimeRange) []Segment[T, O]
 	Tick(ts int64)
+	UpdateOptions(opts *commonv1.ResourceOpts)
+	TakeFileSnapshot(dst string) error
+	GetExpiredSegmentsTimeRange() *timestamp.TimeRange
+	DeleteExpiredSegments(timeRange timestamp.TimeRange) int64
 }
 
 // Segment is a time range of data.
@@ -106,6 +127,7 @@ type Segment[T TSTable, O any] interface {
 type TSTable interface {
 	io.Closer
 	Collect(Metrics)
+	TakeFileSnapshot(dst string) error
 }
 
 // TSTableCreator creates a TSTable.
