@@ -18,6 +18,7 @@
 package grpc
 
 import (
+	"fmt"
 	"sync"
 
 	"github.com/pkg/errors"
@@ -25,6 +26,8 @@ import (
 	databasev1 "github.com/apache/skywalking-banyandb/api/proto/banyandb/database/v1"
 	"github.com/apache/skywalking-banyandb/banyand/metadata/schema"
 	"github.com/apache/skywalking-banyandb/banyand/queue"
+	"github.com/apache/skywalking-banyandb/pkg/bus"
+	"github.com/apache/skywalking-banyandb/pkg/logger"
 	"github.com/apache/skywalking-banyandb/pkg/node"
 )
 
@@ -37,29 +40,30 @@ var (
 // together with the shardID calculated from the incoming data.
 type NodeRegistry interface {
 	Locate(group, name string, shardID uint32) (string, error)
+	fmt.Stringer
 }
 
 type clusterNodeService struct {
 	schema.UnimplementedOnInitHandler
 	pipeline queue.Client
 	sel      node.Selector
+	l        *logger.Logger
+	topic    bus.Topic
 	sync.Once
 }
 
 // NewClusterNodeRegistry creates a cluster node registry.
-func NewClusterNodeRegistry(pipeline queue.Client, selector node.Selector) NodeRegistry {
+func NewClusterNodeRegistry(topic bus.Topic, pipeline queue.Client, selector node.Selector) NodeRegistry {
 	nr := &clusterNodeService{
 		pipeline: pipeline,
 		sel:      selector,
+		topic:    topic,
+		l:        logger.GetLogger("cluster-node-registry-" + topic.String()),
 	}
-	nr.init()
-	return nr
-}
-
-func (n *clusterNodeService) init() {
-	n.Do(func() {
-		n.pipeline.Register(n)
+	nr.Do(func() {
+		nr.pipeline.Register(nr.topic, nr)
 	})
+	return nr
 }
 
 func (n *clusterNodeService) Locate(group, name string, shardID uint32) (string, error) {
@@ -94,8 +98,16 @@ func (n *clusterNodeService) OnDelete(metadata schema.Metadata) {
 	}
 }
 
+func (n *clusterNodeService) String() string {
+	return n.sel.String()
+}
+
 type localNodeService struct{}
 
+func (l localNodeService) String() string {
+	return "local"
+}
+
 // NewLocalNodeRegistry creates a local(fake) node registry.
 func NewLocalNodeRegistry() NodeRegistry {
 	return localNodeService{}
