@@ -205,7 +205,7 @@ func (b *block) marshalTagFamily(tf columnFamily, bm *blockMetadata, ww *writers
 }
 
 func (b *block) unmarshalTagFamily(decoder *encoding.BytesBlockDecoder, tfIndex int, name string,
-	columnFamilyMetadataBlock *dataBlock, tagProjection []string, metaReader, valueReader fs.Reader,
+	columnFamilyMetadataBlock *dataBlock, tagProjection []string, metaReader, valueReader fs.Reader, count int,
 ) {
 	if len(tagProjection) < 1 {
 		return
@@ -222,13 +222,20 @@ func (b *block) unmarshalTagFamily(decoder *encoding.BytesBlockDecoder, tfIndex
 	bigValuePool.Release(bb)
 	b.tagFamilies[tfIndex].name = name
 	cc := b.tagFamilies[tfIndex].resizeColumns(len(tagProjection))
+NEXT:
 	for j := range tagProjection {
 		for i := range cfm.columnMetadata {
 			if tagProjection[j] == cfm.columnMetadata[i].name {
 				cc[j].mustReadValues(decoder, valueReader, cfm.columnMetadata[i], uint64(b.Len()))
-				break
+				continue NEXT
 			}
 		}
+		cc[j].name = tagProjection[j]
+		cc[j].valueType = pbv1.ValueTypeUnknown
+		cc[j].resizeValues(count)
+		for k := range cc[j].values {
+			cc[j].values[k] = nil
+		}
 	}
 }
 
@@ -303,11 +310,21 @@ func (b *block) mustReadFrom(decoder *encoding.BytesBlockDecoder, p *part, bm bl
 		name := bm.tagProjection[i].Family
 		block, ok := bm.tagFamilies[name]
 		if !ok {
+			b.tagFamilies[i].name = name
+			b.tagFamilies[i].resizeColumns(len(bm.tagProjection[i].Names))
+			for j := range bm.tagProjection[i].Names {
+				b.tagFamilies[i].columns[j].name = bm.tagProjection[i].Names[j]
+				b.tagFamilies[i].columns[j].valueType = pbv1.ValueTypeUnknown
+				b.tagFamilies[i].columns[j].resizeValues(int(bm.count))
+				for k := range bm.count {
+					b.tagFamilies[i].columns[j].values[k] = nil
+				}
+			}
 			continue
 		}
 		b.unmarshalTagFamily(decoder, i, name, block,
 			bm.tagProjection[i].Names, p.tagFamilyMetadata[name],
-			p.tagFamilies[name])
+			p.tagFamilies[name], int(bm.count))
 	}
 }
 
@@ -674,19 +691,25 @@ func (bc *blockCursor) replace(r *model.MeasureResult, storedIndexValue map[comm
 func (bc *blockCursor) loadData(tmpBlock *block) bool {
 	tmpBlock.reset()
 	cfm := make([]columnMetadata, 0, len(bc.fieldProjection))
-	for j := range bc.fieldProjection {
-		for i := range bc.bm.field.columnMetadata {
-			if bc.bm.field.columnMetadata[i].name == bc.fieldProjection[j] {
-				cfm = append(cfm, bc.bm.field.columnMetadata[i])
+NEXT_FIELD:
+	for _, fp := range bc.fieldProjection {
+		for _, cm := range bc.bm.field.columnMetadata {
+			if cm.name == fp {
+				cfm = append(cfm, cm)
+				continue NEXT_FIELD
 			}
 		}
+		cfm = append(cfm, columnMetadata{
+			name:      fp,
+			valueType: pbv1.ValueTypeUnknown,
+		})
 	}
 	bc.bm.field.columnMetadata = cfm
 	bc.bm.tagProjection = bc.tagProjection
 	var tf map[string]*dataBlock
-	for i := range bc.tagProjection {
+	for _, tp := range bc.tagProjection {
 		for tfName, block := range bc.bm.tagFamilies {
-			if bc.tagProjection[i].Family == tfName {
+			if tp.Family == tfName {
 				if tf == nil {
 					tf = make(map[string]*dataBlock, len(bc.tagProjection))
 				}
