@@ -21,12 +21,12 @@ import (
 	"errors"
 	"fmt"
 	"sort"
-	"sync"
 
 	"github.com/apache/skywalking-banyandb/api/common"
 	"github.com/apache/skywalking-banyandb/pkg/convert"
 	"github.com/apache/skywalking-banyandb/pkg/encoding"
-	pbv1 "github.com/apache/skywalking-banyandb/pkg/pb/v1"
+	"github.com/apache/skywalking-banyandb/pkg/pool"
+	"github.com/apache/skywalking-banyandb/pkg/query/model"
 )
 
 type dataBlock struct {
@@ -50,25 +50,19 @@ func (b *dataBlock) marshal(dst []byte) []byte {
 	return dst
 }
 
-func (b *dataBlock) unmarshal(src []byte) ([]byte, error) {
-	src, n, err := encoding.BytesToVarUint64(src)
-	if err != nil {
-		return nil, fmt.Errorf("cannot unmarshal offset: %w", err)
-	}
+func (b *dataBlock) unmarshal(src []byte) []byte {
+	src, n := encoding.BytesToVarUint64(src)
 	b.offset = n
 
-	src, n, err = encoding.BytesToVarUint64(src)
-	if err != nil {
-		return nil, fmt.Errorf("cannot unmarshal size: %w", err)
-	}
+	src, n = encoding.BytesToVarUint64(src)
 	b.size = n
-	return src, nil
+	return src
 }
 
 type blockMetadata struct {
 	tagFamilies           map[string]*dataBlock
 	field                 columnFamilyMetadata
-	tagProjection         []pbv1.TagProjection
+	tagProjection         []model.TagProjection
 	timestamps            timestampsMetadata
 	seriesID              common.SeriesID
 	uncompressedSizeBytes uint64
@@ -141,44 +135,30 @@ func (bm *blockMetadata) unmarshal(src []byte) ([]byte, error) {
 	}
 	bm.seriesID = common.SeriesID(encoding.BytesToUint64(src))
 	src = src[8:]
-	src, n, err := encoding.BytesToVarUint64(src)
-	if err != nil {
-		return nil, fmt.Errorf("cannot unmarshal uncompressedSizeBytes: %w", err)
-	}
+	src, n := encoding.BytesToVarUint64(src)
 	bm.uncompressedSizeBytes = n
 
-	src, n, err = encoding.BytesToVarUint64(src)
-	if err != nil {
-		return nil, fmt.Errorf("cannot unmarshal count: %w", err)
-	}
+	src, n = encoding.BytesToVarUint64(src)
 	bm.count = n
-	src, err = bm.timestamps.unmarshal(src)
-	if err != nil {
-		return nil, fmt.Errorf("cannot unmarshal timestampsMetadata: %w", err)
-	}
-	src, n, err = encoding.BytesToVarUint64(src)
-	if err != nil {
-		return nil, fmt.Errorf("cannot unmarshal tagFamilies count: %w", err)
-	}
+	src = bm.timestamps.unmarshal(src)
+	src, n = encoding.BytesToVarUint64(src)
 	if n > 0 {
 		if bm.tagFamilies == nil {
 			bm.tagFamilies = make(map[string]*dataBlock, n)
 		}
 		var nameBytes []byte
+		var err error
 		for i := uint64(0); i < n; i++ {
 			src, nameBytes, err = encoding.DecodeBytes(src)
 			if err != nil {
 				return nil, fmt.Errorf("cannot unmarshal tagFamily name: %w", err)
 			}
-			// TODO: cache dataBlock
 			tf := &dataBlock{}
-			src, err = tf.unmarshal(src)
-			if err != nil {
-				return nil, fmt.Errorf("cannot unmarshal tagFamily dataBlock: %w", err)
-			}
+			src = tf.unmarshal(src)
 			bm.tagFamilies[string(nameBytes)] = tf
 		}
 	}
+	var err error
 	src, err = bm.field.unmarshal(src)
 	if err != nil {
 		return nil, fmt.Errorf("cannot unmarshal columnFamilyMetadata: %w", err)
@@ -198,7 +178,7 @@ func generateBlockMetadata() *blockMetadata {
 	if v == nil {
 		return &blockMetadata{}
 	}
-	return v.(*blockMetadata)
+	return v
 }
 
 func releaseBlockMetadata(bm *blockMetadata) {
@@ -206,7 +186,7 @@ func releaseBlockMetadata(bm *blockMetadata) {
 	blockMetadataPool.Put(bm)
 }
 
-var blockMetadataPool sync.Pool
+var blockMetadataPool = pool.Register[*blockMetadata]("measure-blockMetadata")
 
 type blockMetadataArray struct {
 	arr []blockMetadata
@@ -219,14 +199,14 @@ func (bma *blockMetadataArray) reset() {
 	bma.arr = bma.arr[:0]
 }
 
-var blockMetadataArrayPool sync.Pool
+var blockMetadataArrayPool = pool.Register[*blockMetadataArray]("measure-blockMetadataArray")
 
 func generateBlockMetadataArray() *blockMetadataArray {
 	v := blockMetadataArrayPool.Get()
 	if v == nil {
 		return &blockMetadataArray{}
 	}
-	return v.(*blockMetadataArray)
+	return v
 }
 
 func releaseBlockMetadataArray(bma *blockMetadataArray) {
@@ -236,9 +216,12 @@ func releaseBlockMetadataArray(bma *blockMetadataArray) {
 
 type timestampsMetadata struct {
 	dataBlock
-	min        int64
-	max        int64
-	encodeType encoding.EncodeType
+	min               int64
+	max               int64
+	versionOffset     uint64
+	versionFirst      int64
+	encodeType        encoding.EncodeType
+	versionEncodeType encoding.EncodeType
 }
 
 func (tm *timestampsMetadata) reset() {
@@ -246,6 +229,9 @@ func (tm *timestampsMetadata) reset() {
 	tm.min = 0
 	tm.max = 0
 	tm.encodeType = 0
+	tm.versionOffset = 0
+	tm.versionFirst = 0
+	tm.versionEncodeType = 0
 }
 
 func (tm *timestampsMetadata) copyFrom(src *timestampsMetadata) {
@@ -253,6 +239,9 @@ func (tm *timestampsMetadata) copyFrom(src *timestampsMetadata) {
 	tm.min = src.min
 	tm.max = src.max
 	tm.encodeType = src.encodeType
+	tm.versionOffset = src.versionOffset
+	tm.versionFirst = src.versionFirst
+	tm.versionEncodeType = src.versionEncodeType
 }
 
 func (tm *timestampsMetadata) marshal(dst []byte) []byte {
@@ -260,20 +249,26 @@ func (tm *timestampsMetadata) marshal(dst []byte) []byte {
 	dst = encoding.Uint64ToBytes(dst, uint64(tm.min))
 	dst = encoding.Uint64ToBytes(dst, uint64(tm.max))
 	dst = append(dst, byte(tm.encodeType))
+	dst = encoding.VarUint64ToBytes(dst, tm.versionOffset)
+	dst = encoding.Uint64ToBytes(dst, uint64(tm.versionFirst))
+	dst = append(dst, byte(tm.versionEncodeType))
 	return dst
 }
 
-func (tm *timestampsMetadata) unmarshal(src []byte) ([]byte, error) {
-	src, err := tm.dataBlock.unmarshal(src)
-	if err != nil {
-		return nil, fmt.Errorf("cannot unmarshal dataBlock: %w", err)
-	}
+func (tm *timestampsMetadata) unmarshal(src []byte) []byte {
+	src = tm.dataBlock.unmarshal(src)
 	tm.min = int64(encoding.BytesToUint64(src))
 	src = src[8:]
 	tm.max = int64(encoding.BytesToUint64(src))
 	src = src[8:]
 	tm.encodeType = encoding.EncodeType(src[0])
-	return src[1:], nil
+	src = src[1:]
+	src, n := encoding.BytesToVarUint64(src)
+	tm.versionOffset = n
+	tm.versionFirst = int64(encoding.BytesToUint64(src))
+	src = src[8:]
+	tm.versionEncodeType = encoding.EncodeType(src[0])
+	return src[1:]
 }
 
 func unmarshalBlockMetadata(dst []blockMetadata, src []byte) ([]blockMetadata, error) {
