@@ -6,24 +6,7 @@
 // not use this file except in compliance with the License.
 // You may obtain a copy of the License at
 //
-//     http://www.apache.org/licenses/LICENSE-2.0
-//
-// Unless required by applicable law or agreed to in writing,
-// software distributed under the License is distributed on an
-// "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
-// KIND, either express or implied.  See the License for the
-// specific language governing permissions and limitations
-// under the License.
-
-// Licensed to Apacme Software Foundation (ASF) under one or more contributor
-// license agreements. See the NOTICE file distributed with
-// this work for additional information regarding copyright
-// ownership. Apacme Software Foundation (ASF) licenses this file to you under
-// the Apacme License, Version 2.0 (the "License"); you may
-// not use this file except in compliance with the License.
-// You may obtain a copy of the License at
-//
-//	http://www.apacme.org/licenses/LICENSE-2.0
+//	http://www.apache.org/licenses/LICENSE-2.0
 //
 // Unless required by applicable law or agreed to in writing,
 // software distributed under the License is distributed on an
@@ -79,10 +62,7 @@ func (cm *columnMetadata) unmarshal(src []byte) ([]byte, error) {
 	}
 	cm.valueType = pbv1.ValueType(src[0])
 	src = src[1:]
-	src, err = cm.dataBlock.unmarshal(src)
-	if err != nil {
-		return nil, fmt.Errorf("cannot unmarshal columnMetadata.dataBlock: %w", err)
-	}
+	src = cm.dataBlock.unmarshal(src)
 	return src, nil
 }
 
@@ -126,14 +106,12 @@ func (cfm *columnFamilyMetadata) marshal(dst []byte) []byte {
 }
 
 func (cfm *columnFamilyMetadata) unmarshal(src []byte) ([]byte, error) {
-	src, columnMetadataLen, err := encoding.BytesToVarUint64(src)
-	if err != nil {
-		return nil, fmt.Errorf("cannot unmarshal columnMetadataLen: %w", err)
-	}
+	src, columnMetadataLen := encoding.BytesToVarUint64(src)
 	if columnMetadataLen < 1 {
 		return src, nil
 	}
 	cms := cfm.resizeColumnMetadata(int(columnMetadataLen))
+	var err error
 	for i := range cms {
 		src, err = cms[i].unmarshal(src)
 		if err != nil {
