@@ -26,6 +26,7 @@ import (
 	"github.com/apache/skywalking-banyandb/banyand/internal/storage"
 	"github.com/apache/skywalking-banyandb/pkg/index"
 	pbv1 "github.com/apache/skywalking-banyandb/pkg/pb/v1"
+	"github.com/apache/skywalking-banyandb/pkg/pool"
 	"github.com/apache/skywalking-banyandb/pkg/timestamp"
 )
 
@@ -36,6 +37,27 @@ type nameValue struct {
 	valueType pbv1.ValueType
 }
 
+func (n *nameValue) reset() {
+	n.name = ""
+	n.value = nil
+	n.valueArr = nil
+}
+
+func generateNameValue() *nameValue {
+	v := nameValuePool.Get()
+	if v == nil {
+		return &nameValue{}
+	}
+	return v
+}
+
+func releaseNameValue(v *nameValue) {
+	v.reset()
+	nameValuePool.Put(v)
+}
+
+var nameValuePool = pool.Register[*nameValue]("measure-nameValue")
+
 func (n *nameValue) size() int {
 	s := len(n.name)
 	if n.value != nil {
@@ -115,13 +137,51 @@ type nameValues struct {
 	values []*nameValue
 }
 
+func (n *nameValues) reset() {
+	n.name = ""
+	for i := range n.values {
+		releaseNameValue(n.values[i])
+	}
+	n.values = n.values[:0]
+}
+
 type dataPoints struct {
 	seriesIDs   []common.SeriesID
 	timestamps  []int64
+	versions    []int64
 	tagFamilies [][]nameValues
 	fields      []nameValues
 }
 
+func (d *dataPoints) reset() {
+	d.seriesIDs = d.seriesIDs[:0]
+	d.timestamps = d.timestamps[:0]
+	d.versions = d.versions[:0]
+	for i := range d.tagFamilies {
+		for j := range d.tagFamilies[i] {
+			d.tagFamilies[i][j].reset()
+		}
+	}
+	d.tagFamilies = d.tagFamilies[:0]
+	for i := range d.fields {
+		d.fields[i].reset()
+	}
+	d.fields = d.fields[:0]
+}
+
+func (d *dataPoints) skip(i int) {
+	if len(d.timestamps) <= i {
+		return
+	}
+	d.seriesIDs = append(d.seriesIDs[:i], d.seriesIDs[i+1:]...)
+	d.timestamps = append(d.timestamps[:i], d.timestamps[i+1:]...)
+	d.versions = append(d.versions[:i], d.versions[i+1:]...)
+	d.tagFamilies = append(d.tagFamilies[:i], d.tagFamilies[i+1:]...)
+	if len(d.fields) > 0 {
+		d.fields = append(d.fields[:i], d.fields[i+1:]...)
+	}
+}
+
 func (d *dataPoints) Len() int {
 	return len(d.seriesIDs)
 }
@@ -130,27 +190,48 @@ func (d *dataPoints) Less(i, j int) bool {
 	if d.seriesIDs[i] != d.seriesIDs[j] {
 		return d.seriesIDs[i] < d.seriesIDs[j]
 	}
-	return d.timestamps[i] < d.timestamps[j]
+	if d.timestamps[i] != d.timestamps[j] {
+		return d.timestamps[i] < d.timestamps[j]
+	}
+	return d.versions[i] > d.versions[j]
 }
 
 func (d *dataPoints) Swap(i, j int) {
 	d.seriesIDs[i], d.seriesIDs[j] = d.seriesIDs[j], d.seriesIDs[i]
 	d.timestamps[i], d.timestamps[j] = d.timestamps[j], d.timestamps[i]
+	d.versions[i], d.versions[j] = d.versions[j], d.versions[i]
 	d.tagFamilies[i], d.tagFamilies[j] = d.tagFamilies[j], d.tagFamilies[i]
 	d.fields[i], d.fields[j] = d.fields[j], d.fields[i]
 }
 
-type dataPointsInTable struct {
-	timeRange timestamp.TimeRange
-	tsTable   storage.TSTableWrapper[*tsTable]
+func generateDataPoints() *dataPoints {
+	v := dataPointsPool.Get()
+	if v == nil {
+		return &dataPoints{}
+	}
+	return v
+}
 
-	dataPoints dataPoints
+func releaseDataPoints(v *dataPoints) {
+	v.reset()
+	dataPointsPool.Put(v)
+}
+
+var dataPointsPool = pool.Register[*dataPoints]("measure-dataPoints")
+
+type dataPointsInTable struct {
+	tsTable    *tsTable
+	dataPoints *dataPoints
+	timeRange  timestamp.TimeRange
 }
 
 type dataPointsInGroup struct {
-	tsdb storage.TSDB[*tsTable, option]
-
-	docs     index.Documents
-	tables   []*dataPointsInTable
-	latestTS int64
+	tsdb            storage.TSDB[*tsTable, option]
+	metadataDocs    index.Documents
+	indexModeDocs   index.Documents
+	metadataDocMap  map[uint64]int
+	indexModeDocMap map[uint64]int
+	tables          []*dataPointsInTable
+	segments        []storage.Segment[*tsTable, option]
+	latestTS        int64
 }
