@@ -22,14 +22,14 @@ import (
 	"path"
 	"path/filepath"
 	"sort"
-	"sync"
 	"sync/atomic"
-	"time"
 
 	"github.com/apache/skywalking-banyandb/api/common"
+	"github.com/apache/skywalking-banyandb/banyand/internal/storage"
 	"github.com/apache/skywalking-banyandb/pkg/bytes"
 	"github.com/apache/skywalking-banyandb/pkg/fs"
 	"github.com/apache/skywalking-banyandb/pkg/logger"
+	"github.com/apache/skywalking-banyandb/pkg/pool"
 )
 
 const (
@@ -150,38 +150,49 @@ func (mp *memPart) mustInitFromDataPoints(dps *dataPoints) {
 	var sidPrev common.SeriesID
 	uncompressedBlockSizeBytes := uint64(0)
 	var indexPrev int
-	for i := range dps.timestamps {
+	var tsPrev int64
+	for i := 0; i < len(dps.timestamps); i++ {
 		sid := dps.seriesIDs[i]
 		if sidPrev == 0 {
 			sidPrev = sid
 		}
 
+		if sid == sidPrev {
+			if tsPrev == dps.timestamps[i] {
+				dps.skip(i)
+				i--
+				continue
+			}
+			tsPrev = dps.timestamps[i]
+		}
+
 		if uncompressedBlockSizeBytes >= maxUncompressedBlockSize ||
 			(i-indexPrev) > maxBlockLength || sid != sidPrev {
-			bsw.MustWriteDataPoints(sidPrev, dps.timestamps[indexPrev:i], dps.tagFamilies[indexPrev:i], dps.fields[indexPrev:i])
+			bsw.MustWriteDataPoints(sidPrev, dps.timestamps[indexPrev:i], dps.versions[indexPrev:i], dps.tagFamilies[indexPrev:i], dps.fields[indexPrev:i])
 			sidPrev = sid
 			indexPrev = i
+			tsPrev = dps.timestamps[indexPrev]
 			uncompressedBlockSizeBytes = 0
 		}
 		uncompressedBlockSizeBytes += uncompressedDataPointSizeBytes(i, dps)
 	}
-	bsw.MustWriteDataPoints(sidPrev, dps.timestamps[indexPrev:], dps.tagFamilies[indexPrev:], dps.fields[indexPrev:])
+	bsw.MustWriteDataPoints(sidPrev, dps.timestamps[indexPrev:], dps.versions[indexPrev:], dps.tagFamilies[indexPrev:], dps.fields[indexPrev:])
 	bsw.Flush(&mp.partMetadata)
 	releaseBlockWriter(bsw)
 }
 
 func (mp *memPart) mustFlush(fileSystem fs.FileSystem, path string) {
-	fileSystem.MkdirPanicIfExist(path, dirPermission)
+	fileSystem.MkdirPanicIfExist(path, storage.DirPerm)
 
-	fs.MustFlush(fileSystem, mp.meta.Buf, filepath.Join(path, metaFilename), filePermission)
-	fs.MustFlush(fileSystem, mp.primary.Buf, filepath.Join(path, primaryFilename), filePermission)
-	fs.MustFlush(fileSystem, mp.timestamps.Buf, filepath.Join(path, timestampsFilename), filePermission)
-	fs.MustFlush(fileSystem, mp.fieldValues.Buf, filepath.Join(path, fieldValuesFilename), filePermission)
+	fs.MustFlush(fileSystem, mp.meta.Buf, filepath.Join(path, metaFilename), storage.FilePerm)
+	fs.MustFlush(fileSystem, mp.primary.Buf, filepath.Join(path, primaryFilename), storage.FilePerm)
+	fs.MustFlush(fileSystem, mp.timestamps.Buf, filepath.Join(path, timestampsFilename), storage.FilePerm)
+	fs.MustFlush(fileSystem, mp.fieldValues.Buf, filepath.Join(path, fieldValuesFilename), storage.FilePerm)
 	for name, tf := range mp.tagFamilies {
-		fs.MustFlush(fileSystem, tf.Buf, filepath.Join(path, name+tagFamiliesFilenameExt), filePermission)
+		fs.MustFlush(fileSystem, tf.Buf, filepath.Join(path, name+tagFamiliesFilenameExt), storage.FilePerm)
 	}
 	for name, tfh := range mp.tagFamilyMetadata {
-		fs.MustFlush(fileSystem, tfh.Buf, filepath.Join(path, name+tagFamiliesMetadataFilenameExt), filePermission)
+		fs.MustFlush(fileSystem, tfh.Buf, filepath.Join(path, name+tagFamiliesMetadataFilenameExt), storage.FilePerm)
 	}
 
 	mp.partMetadata.mustWriteMetadata(fileSystem, path)
@@ -190,7 +201,9 @@ func (mp *memPart) mustFlush(fileSystem fs.FileSystem, path string) {
 }
 
 func uncompressedDataPointSizeBytes(index int, dps *dataPoints) uint64 {
-	n := uint64(len(time.RFC3339Nano))
+	// 8 bytes for timestamp
+	// 8 bytes for version
+	n := uint64(8 + 8)
 	n += uint64(len(dps.fields[index].name))
 	for i := range dps.fields[index].values {
 		n += uint64(dps.fields[index].values[i].size())
@@ -209,7 +222,7 @@ func generateMemPart() *memPart {
 	if v == nil {
 		return &memPart{}
 	}
-	return v.(*memPart)
+	return v
 }
 
 func releaseMemPart(mp *memPart) {
@@ -217,7 +230,7 @@ func releaseMemPart(mp *memPart) {
 	memPartPool.Put(mp)
 }
 
-var memPartPool sync.Pool
+var memPartPool = pool.Register[*memPart]("measure-memPart")
 
 type partWrapper struct {
 	mp        *memPart
@@ -239,6 +252,12 @@ func (pw *partWrapper) decRef() {
 	if n > 0 {
 		return
 	}
+	if pw.mp != nil {
+		releaseMemPart(pw.mp)
+		pw.mp = nil
+		pw.p = nil
+		return
+	}
 	pw.p.close()
 	if pw.removable.Load() && pw.p.fileSystem != nil {
 		go func(pw *partWrapper) {
@@ -251,6 +270,13 @@ func (pw *partWrapper) ID() uint64 {
 	return pw.p.partMetadata.ID
 }
 
+func (pw *partWrapper) String() string {
+	if pw.mp != nil {
+		return fmt.Sprintf("mem part %v", pw.mp.partMetadata)
+	}
+	return fmt.Sprintf("part %v", pw.p.partMetadata)
+}
+
 func mustOpenFilePart(id uint64, root string, fileSystem fs.FileSystem) *part {
 	var p part
 	partPath := partPath(root, id)
