@@ -18,10 +18,10 @@
 package measure
 
 import (
+	"bytes"
 	"container/heap"
 	"context"
 	"fmt"
-	"io"
 	"sort"
 
 	"github.com/pkg/errors"
@@ -38,6 +38,7 @@ import (
 	pbv1 "github.com/apache/skywalking-banyandb/pkg/pb/v1"
 	"github.com/apache/skywalking-banyandb/pkg/query/model"
 	resourceSchema "github.com/apache/skywalking-banyandb/pkg/schema"
+	"github.com/apache/skywalking-banyandb/pkg/timestamp"
 )
 
 const (
@@ -45,15 +46,17 @@ const (
 	checkDoneEvery = 128
 )
 
+var nilResult = model.MeasureQueryResult(nil)
+
 // Query allow to retrieve measure data points.
 type Query interface {
 	LoadGroup(name string) (resourceSchema.Group, bool)
 	Measure(measure *commonv1.Metadata) (Measure, error)
+	GetRemovalSegmentsTimeRange(group string) *timestamp.TimeRange
 }
 
 // Measure allows inspecting measure data points' details.
 type Measure interface {
-	io.Closer
 	Query(ctx context.Context, opts model.MeasureQueryOptions) (model.MeasureQueryResult, error)
 	GetSchema() *databasev1.Measure
 	GetIndexRules() []*databasev1.IndexRule
@@ -68,15 +71,35 @@ type queryOptions struct {
 }
 
 func (s *measure) Query(ctx context.Context, mqo model.MeasureQueryOptions) (mqr model.MeasureQueryResult, err error) {
-	if mqo.TimeRange == nil || len(mqo.Entities) < 1 {
-		return nil, errors.New("invalid query options: timeRange and series are required")
+	if mqo.TimeRange == nil {
+		return nil, errors.New("invalid query options: timeRange are required")
 	}
 	if len(mqo.TagProjection) == 0 && len(mqo.FieldProjection) == 0 {
 		return nil, errors.New("invalid query options: tagProjection or fieldProjection is required")
 	}
-	db := s.databaseSupplier.SupplyTSDB()
+	var tsdb storage.TSDB[*tsTable, option]
+	db := s.tsdb.Load()
 	if db == nil {
-		return mqr, nil
+		tsdb, err = s.schemaRepo.loadTSDB(s.group)
+		if err != nil {
+			return nil, err
+		}
+		s.tsdb.Store(tsdb)
+	} else {
+		tsdb = db.(storage.TSDB[*tsTable, option])
+	}
+
+	segments := tsdb.SelectSegments(*mqo.TimeRange)
+	if len(segments) < 1 {
+		return nilResult, nil
+	}
+
+	if s.schema.IndexMode {
+		return s.buildIndexQueryResult(ctx, mqo, segments)
+	}
+
+	if len(mqo.Entities) < 1 {
+		return nil, errors.New("invalid query options: series is required")
 	}
 
 	series := make([]*pbv1.Series, len(mqo.Entities))
@@ -86,29 +109,29 @@ func (s *measure) Query(ctx context.Context, mqo model.MeasureQueryOptions) (mqr
 			EntityValues: mqo.Entities[i],
 		}
 	}
-	var result queryResult
-	result.ctx = ctx
-	tsdb := db.(storage.TSDB[*tsTable, option])
-	result.segments = tsdb.SelectSegments(*mqo.TimeRange)
-	if len(result.segments) < 1 {
-		return &result, nil
+
+	sids, tables, storedIndexValue, newTagProjection, err := s.searchSeriesList(ctx, series, mqo, segments)
+	if err != nil {
+		return nil, err
+	}
+	if len(sids) < 1 {
+		for i := range segments {
+			segments[i].DecRef()
+		}
+		return nilResult, nil
+	}
+	result := queryResult{
+		ctx:              ctx,
+		segments:         segments,
+		tagProjection:    mqo.TagProjection,
+		storedIndexValue: storedIndexValue,
 	}
 	defer func() {
 		if err != nil {
 			result.Release()
 		}
 	}()
-
-	sids, tables, storedIndexValue, newTagProjection, err := s.searchSeriesList(ctx, series, mqo, result.segments)
-	if err != nil {
-		return nil, err
-	}
-	if len(sids) < 1 {
-		return &result, nil
-	}
-	result.tagProjection = mqo.TagProjection
 	mqo.TagProjection = newTagProjection
-	result.storedIndexValue = storedIndexValue
 	var parts []*part
 	qo := queryOptions{
 		MeasureQueryOptions: mqo,
@@ -135,23 +158,27 @@ func (s *measure) Query(ctx context.Context, mqo model.MeasureQueryOptions) (mqr
 
 	if mqo.Order == nil {
 		result.ascTS = true
-	} else if mqo.Order.Sort == modelv1.Sort_SORT_ASC || mqo.Order.Sort == modelv1.Sort_SORT_UNSPECIFIED {
-		result.ascTS = true
-	}
-	switch mqo.OrderByType {
-	case model.OrderByTypeTime:
 		result.orderByTS = true
-	case model.OrderByTypeIndex:
-		result.orderByTS = false
-	case model.OrderByTypeSeries:
-		result.orderByTS = false
+	} else {
+		if mqo.Order.Sort == modelv1.Sort_SORT_ASC || mqo.Order.Sort == modelv1.Sort_SORT_UNSPECIFIED {
+			result.ascTS = true
+		}
+		switch mqo.Order.Type {
+		case index.OrderByTypeTime:
+			result.orderByTS = true
+		case index.OrderByTypeIndex:
+			result.orderByTS = false
+		case index.OrderByTypeSeries:
+			result.orderByTS = false
+		}
 	}
+
 	return &result, nil
 }
 
 type tagNameWithType struct {
-	name string
-	typ  pbv1.ValueType
+	fieldName string
+	typ       pbv1.ValueType
 }
 
 func (s *measure) searchSeriesList(ctx context.Context, series []*pbv1.Series, mqo model.MeasureQueryOptions,
@@ -163,6 +190,7 @@ func (s *measure) searchSeriesList(ctx context.Context, series []*pbv1.Series, m
 	fieldToValueType := make(map[string]tagNameWithType)
 	var projectedEntityOffsets map[string]int
 	newTagProjection = make([]model.TagProjection, 0)
+	is := s.indexSchema.Load().(indexSchema)
 	for _, tp := range mqo.TagProjection {
 		var tagProjection model.TagProjection
 	TAG:
@@ -176,14 +204,16 @@ func (s *measure) searchSeriesList(ctx context.Context, series []*pbv1.Series, m
 					continue TAG
 				}
 			}
-			if fields, ok := s.fieldIndexLocation[tp.Family]; ok {
-				if field, ok := fields[n]; ok {
-					indexProjection = append(indexProjection, field.Key)
-					fieldToValueType[field.Key.Marshal()] = tagNameWithType{
-						name: n,
-						typ:  field.Type,
+			if is.fieldIndexLocation != nil {
+				if fields, ok := is.fieldIndexLocation[tp.Family]; ok {
+					if field, ok := fields[n]; ok {
+						indexProjection = append(indexProjection, field.Key)
+						fieldToValueType[field.Key.Marshal()] = tagNameWithType{
+							fieldName: n,
+							typ:       field.Type,
+						}
+						continue TAG
 					}
-					continue TAG
 				}
 			}
 			tagProjection.Family = tp.Family
@@ -195,7 +225,7 @@ func (s *measure) searchSeriesList(ctx context.Context, series []*pbv1.Series, m
 	}
 	seriesFilter := roaring.NewPostingList()
 	for i := range segments {
-		sll, fieldResultList, err := segments[i].IndexDB().Search(ctx, series, storage.IndexSearchOpts{
+		sd, _, err := segments[i].IndexDB().Search(ctx, series, storage.IndexSearchOpts{
 			Query:       mqo.Query,
 			Order:       mqo.Order,
 			PreloadSize: preloadSize,
@@ -204,32 +234,37 @@ func (s *measure) searchSeriesList(ctx context.Context, series []*pbv1.Series, m
 		if err != nil {
 			return nil, nil, nil, nil, err
 		}
-		if len(sll) > 0 {
+		if len(sd.SeriesList) > 0 {
 			tables = append(tables, segments[i].Tables()...)
 		}
-		for j := range sll {
-			if seriesFilter.Contains(uint64(sll[j].ID)) {
+		for j := range sd.SeriesList {
+			if seriesFilter.Contains(uint64(sd.SeriesList[j].ID)) {
 				continue
 			}
-			seriesFilter.Insert(uint64(sll[j].ID))
-			sl = append(sl, sll[j].ID)
-			if projectedEntityOffsets == nil && fieldResultList == nil {
+			seriesFilter.Insert(uint64(sd.SeriesList[j].ID))
+			sl = append(sl, sd.SeriesList[j].ID)
+			if projectedEntityOffsets == nil && sd.Fields == nil {
 				continue
 			}
 			if storedIndexValue == nil {
 				storedIndexValue = make(map[common.SeriesID]map[string]*modelv1.TagValue)
 			}
 			tagValues := make(map[string]*modelv1.TagValue)
-			storedIndexValue[sll[j].ID] = tagValues
+			storedIndexValue[sd.SeriesList[j].ID] = tagValues
 			for name, offset := range projectedEntityOffsets {
-				tagValues[name] = sll[j].EntityValues[offset]
+				if offset < 0 || offset >= len(sd.SeriesList[j].EntityValues) {
+					logger.Warningf("offset %d for tag %s is out of range for series ID %v", offset, name, sd.SeriesList[j].ID)
+					tagValues[name] = pbv1.NullTagValue
+					continue
+				}
+				tagValues[name] = sd.SeriesList[j].EntityValues[offset]
 			}
-			if fieldResultList == nil {
+			if sd.Fields == nil {
 				continue
 			}
-			for f, v := range fieldResultList[j] {
+			for f, v := range sd.Fields[j] {
 				if tnt, ok := fieldToValueType[f]; ok {
-					tagValues[tnt.name] = mustDecodeTagValue(tnt.typ, v)
+					tagValues[tnt.fieldName] = mustDecodeTagValue(tnt.typ, v)
 				} else {
 					logger.Panicf("unknown field %s not found in fieldToValueType", f)
 				}
@@ -239,6 +274,87 @@ func (s *measure) searchSeriesList(ctx context.Context, series []*pbv1.Series, m
 	return sl, tables, storedIndexValue, newTagProjection, nil
 }
 
+func (s *measure) buildIndexQueryResult(ctx context.Context, mqo model.MeasureQueryOptions,
+	segments []storage.Segment[*tsTable, option],
+) (model.MeasureQueryResult, error) {
+	defer func() {
+		for i := range segments {
+			segments[i].DecRef()
+		}
+	}()
+	is := s.indexSchema.Load().(indexSchema)
+	r := &indexSortResult{}
+	var indexProjection []index.FieldKey
+	for _, tp := range mqo.TagProjection {
+		tagFamilyLocation := tagFamilyLocation{
+			name:                   tp.Family,
+			fieldToValueType:       make(map[string]tagNameWithType),
+			projectedEntityOffsets: make(map[string]int),
+		}
+	TAG:
+		for _, n := range tp.Names {
+			tagFamilyLocation.tagNames = append(tagFamilyLocation.tagNames, n)
+			for i := range s.schema.GetEntity().GetTagNames() {
+				if n == s.schema.GetEntity().GetTagNames()[i] {
+					tagFamilyLocation.projectedEntityOffsets[n] = i
+					continue TAG
+				}
+			}
+			if is.fieldIndexLocation != nil {
+				if fields, ok := is.fieldIndexLocation[tp.Family]; ok {
+					if field, ok := fields[n]; ok {
+						indexProjection = append(indexProjection, field.Key)
+						tagFamilyLocation.fieldToValueType[n] = tagNameWithType{
+							fieldName: field.Key.Marshal(),
+							typ:       field.Type,
+						}
+						continue TAG
+					}
+				}
+			}
+			return nil, fmt.Errorf("tag %s not found in schema", n)
+		}
+		r.tfl = append(r.tfl, tagFamilyLocation)
+	}
+	var err error
+	opts := storage.IndexSearchOpts{
+		Query:       mqo.Query,
+		Order:       mqo.Order,
+		PreloadSize: preloadSize,
+		Projection:  indexProjection,
+	}
+	seriesFilter := roaring.NewPostingList()
+	for i := range segments {
+		if mqo.TimeRange.Include(segments[i].GetTimeRange()) {
+			opts.TimeRange = nil
+		} else {
+			opts.TimeRange = mqo.TimeRange
+		}
+		sr := &segResult{}
+		sr.SeriesData, sr.sortedValues, err = segments[i].IndexDB().SearchWithoutSeries(ctx, opts)
+		if err != nil {
+			return nil, err
+		}
+		for j := 0; j < len(sr.SeriesList); j++ {
+			if seriesFilter.Contains(uint64(sr.SeriesList[j].ID)) {
+				sr.remove(j)
+				j--
+				continue
+			}
+			seriesFilter.Insert(uint64(sr.SeriesList[j].ID))
+		}
+		if len(sr.SeriesList) < 1 {
+			continue
+		}
+		r.segResults = append(r.segResults, sr)
+	}
+	if len(r.segResults) < 1 {
+		return nilResult, nil
+	}
+	heap.Init(&r.segResults)
+	return r, nil
+}
+
 func (s *measure) searchBlocks(ctx context.Context, result *queryResult, sids []common.SeriesID, parts []*part, qo queryOptions) error {
 	bma := generateBlockMetadataArray()
 	defer releaseBlockMetadataArray(bma)
@@ -254,6 +370,7 @@ func (s *measure) searchBlocks(ctx context.Context, result *queryResult, sids []
 		return fmt.Errorf("cannot init tstIter: %w", tstIter.Error())
 	}
 	var hit int
+	var totalBlockBytes uint64
 	for tstIter.nextBlock() {
 		if hit%checkDoneEvery == 0 {
 			select {
@@ -267,10 +384,14 @@ func (s *measure) searchBlocks(ctx context.Context, result *queryResult, sids []
 		p := tstIter.piHeap[0]
 		bc.init(p.p, p.curBlock, qo)
 		result.data = append(result.data, bc)
+		totalBlockBytes += bc.bm.uncompressedSizeBytes
 	}
 	if tstIter.Error() != nil {
 		return fmt.Errorf("cannot iterate tstIter: %w", tstIter.Error())
 	}
+	if err := s.pm.AcquireResource(ctx, totalBlockBytes); err != nil {
+		return err
+	}
 	result.sidToIndex = make(map[common.SeriesID]int)
 	for i, si := range originalSids {
 		result.sidToIndex[si] = i
@@ -634,3 +755,124 @@ func (qr *queryResult) merge(storedIndexValue map[common.SeriesID]map[string]*mo
 
 	return result
 }
+
+type indexSortResult struct {
+	tfl        []tagFamilyLocation
+	segResults segResultHeap
+}
+
+// Pull implements model.MeasureQueryResult.
+func (iqr *indexSortResult) Pull() *model.MeasureResult {
+	if len(iqr.segResults) < 1 {
+		return nil
+	}
+	if len(iqr.segResults) == 1 {
+		if iqr.segResults[0].i >= len(iqr.segResults[0].SeriesList) {
+			return nil
+		}
+		sr := iqr.segResults[0]
+		r := iqr.copyTo(sr)
+		sr.i++
+		if sr.i >= len(sr.SeriesList) {
+			iqr.segResults = iqr.segResults[:0]
+		}
+		return r
+	}
+	top := heap.Pop(&iqr.segResults)
+	sr := top.(*segResult)
+	r := iqr.copyTo(sr)
+	sr.i++
+	if sr.i < len(sr.SeriesList) {
+		heap.Push(&iqr.segResults, sr)
+	}
+	return r
+}
+
+func (iqr *indexSortResult) Release() {}
+
+func (iqr *indexSortResult) copyTo(src *segResult) (dest *model.MeasureResult) {
+	index := src.i
+	dest = &model.MeasureResult{
+		SID:        src.SeriesList[index].ID,
+		Timestamps: []int64{src.Timestamps[index]},
+		Versions:   []int64{src.Versions[index]},
+	}
+	for i := range iqr.tfl {
+		tagFamily := model.TagFamily{Name: iqr.tfl[i].name}
+		peo := iqr.tfl[i].projectedEntityOffsets
+		var fr storage.FieldResult
+		if src.Fields != nil {
+			fr = src.Fields[index]
+		}
+		for _, n := range iqr.tfl[i].tagNames {
+			if offset, ok := peo[n]; ok {
+				tagFamily.Tags = append(tagFamily.Tags, model.Tag{
+					Name:   n,
+					Values: []*modelv1.TagValue{src.SeriesList[index].EntityValues[offset]},
+				})
+				continue
+			}
+			if fr == nil {
+				continue
+			}
+			if tnt, ok := iqr.tfl[i].fieldToValueType[n]; ok {
+				tagFamily.Tags = append(tagFamily.Tags, model.Tag{
+					Name:   n,
+					Values: []*modelv1.TagValue{mustDecodeTagValue(tnt.typ, fr[tnt.fieldName])},
+				})
+			} else {
+				logger.Panicf("unknown field %s not found in fieldToValueType", n)
+			}
+		}
+		dest.TagFamilies = append(dest.TagFamilies, tagFamily)
+	}
+	return dest
+}
+
+type tagFamilyLocation struct {
+	fieldToValueType       map[string]tagNameWithType
+	projectedEntityOffsets map[string]int
+	name                   string
+	tagNames               []string
+}
+
+type segResult struct {
+	storage.SeriesData
+	sortedValues [][]byte
+	i            int
+}
+
+func (sr *segResult) remove(i int) {
+	sr.SeriesList = append(sr.SeriesList[:i], sr.SeriesList[i+1:]...)
+	if sr.Fields != nil {
+		sr.Fields = append(sr.Fields[:i], sr.Fields[i+1:]...)
+	}
+	sr.Timestamps = append(sr.Timestamps[:i], sr.Timestamps[i+1:]...)
+	sr.Versions = append(sr.Versions[:i], sr.Versions[i+1:]...)
+	if sr.sortedValues != nil {
+		sr.sortedValues = append(sr.sortedValues[:i], sr.sortedValues[i+1:]...)
+	}
+}
+
+type segResultHeap []*segResult
+
+func (h segResultHeap) Len() int { return len(h) }
+func (h segResultHeap) Less(i, j int) bool {
+	if h[i].sortedValues == nil {
+		return h[i].SeriesList[h[i].i].ID < h[j].SeriesList[h[j].i].ID
+	}
+	return bytes.Compare(h[i].sortedValues[h[i].i], h[j].sortedValues[h[j].i]) < 0
+}
+func (h segResultHeap) Swap(i, j int) { h[i], h[j] = h[j], h[i] }
+
+func (h *segResultHeap) Push(x interface{}) {
+	*h = append(*h, x.(*segResult))
+}
+
+func (h *segResultHeap) Pop() interface{} {
+	old := *h
+	n := len(old)
+	x := old[n-1]
+	*h = old[0 : n-1]
+	return x
+}
