@@ -30,11 +30,10 @@ import (
 	"github.com/apache/skywalking-banyandb/api/common"
 	"github.com/apache/skywalking-banyandb/pkg/convert"
 	"github.com/apache/skywalking-banyandb/pkg/fs"
-	"github.com/apache/skywalking-banyandb/pkg/logger"
 	pbv1 "github.com/apache/skywalking-banyandb/pkg/pb/v1"
+	"github.com/apache/skywalking-banyandb/pkg/query/model"
 	"github.com/apache/skywalking-banyandb/pkg/run"
 	"github.com/apache/skywalking-banyandb/pkg/test"
-	"github.com/apache/skywalking-banyandb/pkg/timestamp"
 	"github.com/apache/skywalking-banyandb/pkg/watcher"
 )
 
@@ -49,6 +48,7 @@ func Test_tsTable_mustAddDataPoints(t *testing.T) {
 			dpsList: []*dataPoints{
 				{
 					timestamps:  []int64{},
+					versions:    []int64{},
 					seriesIDs:   []common.SeriesID{},
 					tagFamilies: make([][]nameValues, 0),
 					fields:      make([]nameValues, 0),
@@ -61,6 +61,7 @@ func Test_tsTable_mustAddDataPoints(t *testing.T) {
 			dpsList: []*dataPoints{
 				{
 					timestamps: []int64{1},
+					versions:   []int64{1},
 					seriesIDs:  []common.SeriesID{1},
 					tagFamilies: [][]nameValues{
 						{
@@ -191,9 +192,9 @@ func Test_tstIter(t *testing.T) {
 				minTimestamp: 1,
 				maxTimestamp: 1,
 				want: []blockMetadata{
-					{seriesID: 1, count: 1, uncompressedSizeBytes: 1676},
-					{seriesID: 2, count: 1, uncompressedSizeBytes: 55},
-					{seriesID: 3, count: 1, uncompressedSizeBytes: 24},
+					{seriesID: 1, count: 1, uncompressedSizeBytes: 1684},
+					{seriesID: 2, count: 1, uncompressedSizeBytes: 63},
+					{seriesID: 3, count: 1, uncompressedSizeBytes: 32},
 				},
 			},
 			{
@@ -203,12 +204,24 @@ func Test_tstIter(t *testing.T) {
 				minTimestamp: 1,
 				maxTimestamp: 2,
 				want: []blockMetadata{
-					{seriesID: 1, count: 1, uncompressedSizeBytes: 1676},
-					{seriesID: 1, count: 1, uncompressedSizeBytes: 1676},
-					{seriesID: 2, count: 1, uncompressedSizeBytes: 55},
-					{seriesID: 2, count: 1, uncompressedSizeBytes: 55},
-					{seriesID: 3, count: 1, uncompressedSizeBytes: 24},
-					{seriesID: 3, count: 1, uncompressedSizeBytes: 24},
+					{seriesID: 1, count: 1, uncompressedSizeBytes: 1684},
+					{seriesID: 1, count: 1, uncompressedSizeBytes: 1684},
+					{seriesID: 2, count: 1, uncompressedSizeBytes: 63},
+					{seriesID: 2, count: 1, uncompressedSizeBytes: 63},
+					{seriesID: 3, count: 1, uncompressedSizeBytes: 32},
+					{seriesID: 3, count: 1, uncompressedSizeBytes: 32},
+				},
+			},
+			{
+				name:         "Test with a single part with same ts",
+				dpsList:      []*dataPoints{duplicatedDps1},
+				sids:         []common.SeriesID{1, 2, 3},
+				minTimestamp: 1,
+				maxTimestamp: 1,
+				want: []blockMetadata{
+					{seriesID: 1, count: 1, uncompressedSizeBytes: 16},
+					{seriesID: 2, count: 1, uncompressedSizeBytes: 16},
+					{seriesID: 3, count: 1, uncompressedSizeBytes: 16},
 				},
 			},
 			{
@@ -218,12 +231,12 @@ func Test_tstIter(t *testing.T) {
 				minTimestamp: 1,
 				maxTimestamp: 2,
 				want: []blockMetadata{
-					{seriesID: 1, count: 1, uncompressedSizeBytes: 1676},
-					{seriesID: 1, count: 1, uncompressedSizeBytes: 1676},
-					{seriesID: 2, count: 1, uncompressedSizeBytes: 55},
-					{seriesID: 2, count: 1, uncompressedSizeBytes: 55},
-					{seriesID: 3, count: 1, uncompressedSizeBytes: 24},
-					{seriesID: 3, count: 1, uncompressedSizeBytes: 24},
+					{seriesID: 1, count: 1, uncompressedSizeBytes: 1684},
+					{seriesID: 1, count: 1, uncompressedSizeBytes: 1684},
+					{seriesID: 2, count: 1, uncompressedSizeBytes: 63},
+					{seriesID: 2, count: 1, uncompressedSizeBytes: 63},
+					{seriesID: 3, count: 1, uncompressedSizeBytes: 32},
+					{seriesID: 3, count: 1, uncompressedSizeBytes: 32},
 				},
 			},
 		}
@@ -250,144 +263,9 @@ func Test_tstIter(t *testing.T) {
 			})
 		}
 	})
-
-	t.Run("file snapshot", func(t *testing.T) {
-		tests := []testCtx{
-			{
-				name:         "Test with no data points",
-				dpsList:      []*dataPoints{},
-				sids:         []common.SeriesID{1, 2, 3},
-				minTimestamp: 1,
-				maxTimestamp: 1,
-			},
-			{
-				name:         "Test with single part",
-				dpsList:      []*dataPoints{dpsTS1},
-				sids:         []common.SeriesID{1, 2, 3},
-				minTimestamp: 1,
-				maxTimestamp: 1,
-				want: []blockMetadata{
-					{seriesID: 1, count: 1, uncompressedSizeBytes: 1676},
-					{seriesID: 2, count: 1, uncompressedSizeBytes: 55},
-					{seriesID: 3, count: 1, uncompressedSizeBytes: 24},
-				},
-			},
-			{
-				name:         "Test with multiple parts with different ts, the block will be merged",
-				dpsList:      []*dataPoints{dpsTS1, dpsTS2, dpsTS2},
-				sids:         []common.SeriesID{1, 2, 3},
-				minTimestamp: 1,
-				maxTimestamp: 2,
-				want: []blockMetadata{
-					{seriesID: 1, count: 2, uncompressedSizeBytes: 3352},
-					{seriesID: 2, count: 2, uncompressedSizeBytes: 110},
-					{seriesID: 3, count: 2, uncompressedSizeBytes: 48},
-				},
-			},
-			{
-				name:         "Test with multiple parts with same ts, duplicated blocks will be merged",
-				dpsList:      []*dataPoints{dpsTS1, dpsTS1},
-				sids:         []common.SeriesID{1, 2, 3},
-				minTimestamp: 1,
-				maxTimestamp: 2,
-				want: []blockMetadata{
-					{seriesID: 1, count: 1, uncompressedSizeBytes: 1676},
-					{seriesID: 2, count: 1, uncompressedSizeBytes: 55},
-					{seriesID: 3, count: 1, uncompressedSizeBytes: 24},
-				},
-			},
-		}
-		for _, tt := range tests {
-			t.Run(tt.name, func(t *testing.T) {
-				t.Run("merging on the fly", func(t *testing.T) {
-					tmpPath, defFn := test.Space(require.New(t))
-					fileSystem := fs.NewLocalFileSystem()
-					defer defFn()
-
-					tst, err := newTSTable(fileSystem, tmpPath, common.Position{},
-						logger.GetLogger("test"), timestamp.TimeRange{}, option{flushTimeout: 0, mergePolicy: newDefaultMergePolicyForTesting()})
-					require.NoError(t, err)
-					for i, dps := range tt.dpsList {
-						tst.mustAddDataPoints(dps)
-						for {
-							snp := tst.currentSnapshot()
-							if snp == nil {
-								t.Logf("waiting for snapshot %d to be introduced", i)
-								time.Sleep(100 * time.Millisecond)
-								continue
-							}
-							if snp.creator != snapshotCreatorMemPart {
-								snp.decRef()
-								break
-							}
-							t.Logf("waiting for snapshot %d to be flushed or merged: current creator:%d, parts: %+v",
-								i, snp.creator, snp.parts)
-							snp.decRef()
-							time.Sleep(100 * time.Millisecond)
-						}
-					}
-					// wait until some parts are merged
-					if len(tt.dpsList) > 0 {
-						for {
-							snp := tst.currentSnapshot()
-							if snp == nil {
-								time.Sleep(100 * time.Millisecond)
-								continue
-							}
-							if len(snp.parts) == 1 || len(snp.parts) < len(tt.dpsList) {
-								snp.decRef()
-								break
-							}
-							t.Logf("waiting for snapshot to be merged: current creator:%d, parts: %+v", snp.creator, snp.parts)
-							snp.decRef()
-							time.Sleep(100 * time.Millisecond)
-						}
-					}
-					verify(t, tt, tst)
-				})
-
-				t.Run("merging on close", func(t *testing.T) {
-					t.Skip("the test is flaky due to unpredictable merge loop schedule.")
-					tmpPath, defFn := test.Space(require.New(t))
-					fileSystem := fs.NewLocalFileSystem()
-					defer defFn()
-
-					tst, err := newTSTable(fileSystem, tmpPath, common.Position{},
-						logger.GetLogger("test"), timestamp.TimeRange{}, option{flushTimeout: defaultFlushTimeout, mergePolicy: newDefaultMergePolicyForTesting()})
-					require.NoError(t, err)
-					for _, dps := range tt.dpsList {
-						tst.mustAddDataPoints(dps)
-						time.Sleep(100 * time.Millisecond)
-					}
-					// wait until the introducer is done
-					if len(tt.dpsList) > 0 {
-						for {
-							snp := tst.currentSnapshot()
-							if snp == nil {
-								time.Sleep(100 * time.Millisecond)
-								continue
-							}
-							if len(snp.parts) == len(tt.dpsList) {
-								snp.decRef()
-								tst.Close()
-								break
-							}
-							snp.decRef()
-							time.Sleep(100 * time.Millisecond)
-						}
-					}
-					// reopen the table
-					tst, err = newTSTable(fileSystem, tmpPath, common.Position{},
-						logger.GetLogger("test"), timestamp.TimeRange{}, option{flushTimeout: defaultFlushTimeout, mergePolicy: newDefaultMergePolicyForTesting()})
-					require.NoError(t, err)
-					verify(t, tt, tst)
-				})
-			})
-		}
-	})
 }
 
-var tagProjections = map[int][]pbv1.TagProjection{
+var tagProjections = map[int][]model.TagProjection{
 	1: {
 		{Family: "arrTag", Names: []string{"strArrTag", "intArrTag"}},
 		{Family: "binaryTag", Names: []string{"binaryTag"}},
@@ -398,7 +276,7 @@ var tagProjections = map[int][]pbv1.TagProjection{
 	},
 }
 
-var allTagProjections = []pbv1.TagProjection{
+var allTagProjections = []model.TagProjection{
 	{Family: "arrTag", Names: []string{"strArrTag", "intArrTag"}},
 	{Family: "binaryTag", Names: []string{"binaryTag"}},
 	{Family: "singleTag", Names: []string{"strTag", "intTag", "strTag1", "strTag2"}},
@@ -412,6 +290,7 @@ var fieldProjections = map[int][]string{
 var dpsTS1 = &dataPoints{
 	seriesIDs:  []common.SeriesID{1, 2, 3},
 	timestamps: []int64{1, 1, 1},
+	versions:   []int64{1, 2, 3},
 	tagFamilies: [][]nameValues{
 		{
 			{
@@ -460,9 +339,10 @@ var dpsTS1 = &dataPoints{
 	},
 }
 
-var dpsTS2 = &dataPoints{
+var dpsTS11 = &dataPoints{
 	seriesIDs:  []common.SeriesID{1, 2, 3},
-	timestamps: []int64{2, 2, 2},
+	timestamps: []int64{1, 1, 1},
+	versions:   []int64{0, 1, 2},
 	tagFamilies: [][]nameValues{
 		{
 			{
@@ -511,16 +391,151 @@ var dpsTS2 = &dataPoints{
 	},
 }
 
+var dpsTS2 = &dataPoints{
+	seriesIDs:  []common.SeriesID{1, 2, 3},
+	timestamps: []int64{2, 2, 2},
+	versions:   []int64{4, 5, 6},
+	tagFamilies: [][]nameValues{
+		{
+			{
+				name: "arrTag", values: []*nameValue{
+					{name: "strArrTag", valueType: pbv1.ValueTypeStrArr, value: nil, valueArr: [][]byte{[]byte("value5"), []byte("value6")}},
+					{name: "intArrTag", valueType: pbv1.ValueTypeInt64Arr, value: nil, valueArr: [][]byte{convert.Int64ToBytes(35), convert.Int64ToBytes(40)}},
+				},
+			},
+			{
+				name: "binaryTag", values: []*nameValue{
+					{name: "binaryTag", valueType: pbv1.ValueTypeBinaryData, value: longText, valueArr: nil},
+				},
+			},
+			{
+				name: "singleTag", values: []*nameValue{
+					{name: "strTag", valueType: pbv1.ValueTypeStr, value: []byte("value3"), valueArr: nil},
+					{name: "intTag", valueType: pbv1.ValueTypeInt64, value: convert.Int64ToBytes(30), valueArr: nil},
+				},
+			},
+		},
+		{
+			{
+				name: "singleTag", values: []*nameValue{
+					{name: "strTag1", valueType: pbv1.ValueTypeStr, value: []byte("tag3"), valueArr: nil},
+					{name: "strTag2", valueType: pbv1.ValueTypeStr, value: []byte("tag4"), valueArr: nil},
+				},
+			},
+		},
+		{}, // empty tagFamilies for seriesID 6
+	},
+	fields: []nameValues{
+		{
+			name: "skipped", values: []*nameValue{
+				{name: "strField", valueType: pbv1.ValueTypeStr, value: []byte("field3"), valueArr: nil},
+				{name: "intField", valueType: pbv1.ValueTypeInt64, value: convert.Int64ToBytes(3330), valueArr: nil},
+				{name: "floatField", valueType: pbv1.ValueTypeFloat64, value: convert.Float64ToBytes(3663699.029), valueArr: nil},
+				{name: "binaryField", valueType: pbv1.ValueTypeBinaryData, value: longText, valueArr: nil},
+			},
+		},
+		{}, // empty fields for seriesID 5
+		{
+			name: "onlyFields", values: []*nameValue{
+				{name: "intField", valueType: pbv1.ValueTypeInt64, value: convert.Int64ToBytes(4440), valueArr: nil},
+			},
+		},
+	},
+}
+
+var duplicatedDps = &dataPoints{
+	seriesIDs:  []common.SeriesID{1, 1, 1},
+	timestamps: []int64{1, 1, 1},
+	versions:   []int64{1, 2, 3},
+	tagFamilies: [][]nameValues{
+		{
+			{
+				name: "arrTag", values: []*nameValue{
+					{name: "strArrTag", valueType: pbv1.ValueTypeStrArr, value: nil, valueArr: [][]byte{[]byte("value1"), []byte("value2")}},
+					{name: "intArrTag", valueType: pbv1.ValueTypeInt64Arr, value: nil, valueArr: [][]byte{convert.Int64ToBytes(25), convert.Int64ToBytes(30)}},
+				},
+			},
+			{
+				name: "binaryTag", values: []*nameValue{
+					{name: "binaryTag", valueType: pbv1.ValueTypeBinaryData, value: longText, valueArr: nil},
+				},
+			},
+			{
+				name: "singleTag", values: []*nameValue{
+					{name: "strTag", valueType: pbv1.ValueTypeStr, value: []byte("value1"), valueArr: nil},
+					{name: "intTag", valueType: pbv1.ValueTypeInt64, value: convert.Int64ToBytes(10), valueArr: nil},
+				},
+			},
+		},
+		{
+			{
+				name: "singleTag", values: []*nameValue{
+					{name: "strTag1", valueType: pbv1.ValueTypeStr, value: []byte("tag1"), valueArr: nil},
+					{name: "strTag2", valueType: pbv1.ValueTypeStr, value: []byte("tag2"), valueArr: nil},
+				},
+			},
+		},
+		{}, // empty tagFamilies for seriesID 3
+	},
+	fields: []nameValues{
+		{
+			name: "skipped", values: []*nameValue{
+				{name: "strField", valueType: pbv1.ValueTypeStr, value: []byte("field1"), valueArr: nil},
+				{name: "intField", valueType: pbv1.ValueTypeInt64, value: convert.Int64ToBytes(1110), valueArr: nil},
+				{name: "floatField", valueType: pbv1.ValueTypeFloat64, value: convert.Float64ToBytes(1221233.343), valueArr: nil},
+				{name: "binaryField", valueType: pbv1.ValueTypeBinaryData, value: longText, valueArr: nil},
+			},
+		},
+		{}, // empty fields for seriesID 2
+		{
+			name: "onlyFields", values: []*nameValue{
+				{name: "intField", valueType: pbv1.ValueTypeInt64, value: convert.Int64ToBytes(1110), valueArr: nil},
+			},
+		},
+	},
+}
+
+var duplicatedDps1 = &dataPoints{
+	seriesIDs:  []common.SeriesID{2, 2, 2, 1, 1, 1, 3, 3, 3},
+	timestamps: []int64{1, 1, 1, 1, 1, 1, 1, 1, 1},
+	versions:   []int64{1, 2, 3, 3, 2, 1, 2, 1, 3},
+	tagFamilies: [][]nameValues{
+		{},
+		{},
+		{},
+		{},
+		{},
+		{},
+		{},
+		{},
+		{},
+	},
+	fields: []nameValues{
+		{},
+		{},
+		{},
+		{},
+		{},
+		{},
+		{},
+		{},
+		{},
+	},
+}
+
 func generateHugeDps(startTimestamp, endTimestamp, timestamp int64) *dataPoints {
 	hugeDps := &dataPoints{
 		seriesIDs:   []common.SeriesID{},
 		timestamps:  []int64{},
+		versions:    []int64{},
 		tagFamilies: [][]nameValues{},
 		fields:      []nameValues{},
 	}
+	now := time.Now().UnixNano()
 	for i := startTimestamp; i <= endTimestamp; i++ {
 		hugeDps.seriesIDs = append(hugeDps.seriesIDs, 1)
 		hugeDps.timestamps = append(hugeDps.timestamps, i)
+		hugeDps.versions = append(hugeDps.versions, now+i)
 		hugeDps.tagFamilies = append(hugeDps.tagFamilies, []nameValues{
 			{
 				name: "arrTag", values: []*nameValue{
@@ -551,6 +566,7 @@ func generateHugeDps(startTimestamp, endTimestamp, timestamp int64) *dataPoints
 	}
 	hugeDps.seriesIDs = append(hugeDps.seriesIDs, []common.SeriesID{2, 3}...)
 	hugeDps.timestamps = append(hugeDps.timestamps, []int64{timestamp, timestamp}...)
+	hugeDps.versions = append(hugeDps.versions, []int64{now + timestamp, now + timestamp}...)
 	hugeDps.tagFamilies = append(hugeDps.tagFamilies, [][]nameValues{{
 		{
 			name: "singleTag", values: []*nameValue{
@@ -566,3 +582,35 @@ func generateHugeDps(startTimestamp, endTimestamp, timestamp int64) *dataPoints
 	}}...)
 	return hugeDps
 }
+
+func generateHugeSmallDps(startTimestamp, endTimestamp, timestamp int64) *dataPoints {
+	hugeDps := &dataPoints{
+		seriesIDs:   []common.SeriesID{},
+		timestamps:  []int64{},
+		versions:    []int64{},
+		tagFamilies: [][]nameValues{},
+		fields:      []nameValues{},
+	}
+	now := time.Now().UnixNano()
+	for i := startTimestamp; i <= endTimestamp; i++ {
+		hugeDps.seriesIDs = append(hugeDps.seriesIDs, 1)
+		hugeDps.timestamps = append(hugeDps.timestamps, i)
+		hugeDps.versions = append(hugeDps.versions, now+i)
+		hugeDps.tagFamilies = append(hugeDps.tagFamilies, []nameValues{})
+		hugeDps.fields = append(hugeDps.fields, nameValues{
+			name: "skipped", values: []*nameValue{
+				{name: "intField", valueType: pbv1.ValueTypeInt64, value: convert.Int64ToBytes(3330), valueArr: nil},
+			},
+		})
+	}
+	hugeDps.seriesIDs = append(hugeDps.seriesIDs, []common.SeriesID{2, 3}...)
+	hugeDps.timestamps = append(hugeDps.timestamps, []int64{timestamp, timestamp}...)
+	hugeDps.versions = append(hugeDps.versions, []int64{now + timestamp, now + timestamp}...)
+	hugeDps.tagFamilies = append(hugeDps.tagFamilies, [][]nameValues{{}, {}}...)
+	hugeDps.fields = append(hugeDps.fields, []nameValues{{}, {
+		name: "onlyFields", values: []*nameValue{
+			{name: "intField", valueType: pbv1.ValueTypeInt64, value: convert.Int64ToBytes(4440), valueArr: nil},
+		},
+	}}...)
+	return hugeDps
+}
