@@ -21,6 +21,8 @@ import (
 	"context"
 	"os"
 	"os/signal"
+	"strings"
+	"sync"
 	"syscall"
 	"time"
 
@@ -31,6 +33,7 @@ import (
 	"github.com/apache/skywalking-banyandb/api/common"
 	commonv1 "github.com/apache/skywalking-banyandb/api/proto/banyandb/common/v1"
 	databasev1 "github.com/apache/skywalking-banyandb/api/proto/banyandb/database/v1"
+	modelv1 "github.com/apache/skywalking-banyandb/api/proto/banyandb/model/v1"
 	"github.com/apache/skywalking-banyandb/banyand/metadata/schema"
 	"github.com/apache/skywalking-banyandb/pkg/logger"
 	"github.com/apache/skywalking-banyandb/pkg/run"
@@ -54,25 +57,30 @@ const flagEtcdTLSCertFile = "etcd-tls-cert-file"
 const flagEtcdTLSKeyFile = "etcd-tls-key-file"
 
 // NewClient returns a new metadata client.
-func NewClient(forceRegisterNode bool) (Service, error) {
+func NewClient(toRegisterNode, forceRegisterNode bool) (Service, error) {
 	return &clientService{
 		closer:            run.NewCloser(1),
 		forceRegisterNode: forceRegisterNode,
+		toRegisterNode:    toRegisterNode,
 	}, nil
 }
 
 type clientService struct {
-	schemaRegistry    schema.Registry
-	closer            *run.Closer
-	namespace         string
-	etcdUsername      string
-	etcdPassword      string
-	etcdTLSCAFile     string
-	etcdTLSCertFile   string
-	etcdTLSKeyFile    string
-	endpoints         []string
-	registryTimeout   time.Duration
-	forceRegisterNode bool
+	schemaRegistry       schema.Registry
+	closer               *run.Closer
+	nodeInfo             *databasev1.Node
+	etcdTLSCertFile      string
+	etcdPassword         string
+	etcdTLSCAFile        string
+	etcdUsername         string
+	etcdTLSKeyFile       string
+	namespace            string
+	endpoints            []string
+	registryTimeout      time.Duration
+	etcdFullSyncInterval time.Duration
+	nodeInfoMux          sync.Mutex
+	forceRegisterNode    bool
+	toRegisterNode       bool
 }
 
 func (s *clientService) SchemaRegistry() schema.Registry {
@@ -89,6 +97,7 @@ func (s *clientService) FlagSet() *run.FlagSet {
 	fs.StringVar(&s.etcdTLSCertFile, flagEtcdTLSCertFile, "", "Etcd client certificate")
 	fs.StringVar(&s.etcdTLSKeyFile, flagEtcdTLSKeyFile, "", "Private key for the etcd client certificate.")
 	fs.DurationVar(&s.registryTimeout, "node-registry-timeout", 2*time.Minute, "The timeout for the node registry")
+	fs.DurationVar(&s.etcdFullSyncInterval, "etcd-full-sync-interval", 30*time.Minute, "The interval for full sync etcd")
 	return fs
 }
 
@@ -123,8 +132,9 @@ func (s *clientService) PreRun(ctx context.Context) error {
 			schema.ConfigureEtcdUser(s.etcdUsername, s.etcdPassword),
 			schema.ConfigureEtcdTLSCAFile(s.etcdTLSCAFile),
 			schema.ConfigureEtcdTLSCertAndKey(s.etcdTLSCertFile, s.etcdTLSKeyFile),
+			schema.ConfigureWatchCheckInterval(s.etcdFullSyncInterval),
 		)
-		if errors.Is(err, context.DeadlineExceeded) {
+		if errors.Is(err, context.DeadlineExceeded) || errors.Is(err, context.Canceled) {
 			select {
 			case <-stopCh:
 				return errors.New("pre-run interrupted")
@@ -143,6 +153,9 @@ func (s *clientService) PreRun(ctx context.Context) error {
 		}
 		return err
 	}
+	if !s.toRegisterNode {
+		return nil
+	}
 
 	val := ctx.Value(common.ContextNodeKey)
 	if val == nil {
@@ -161,16 +174,18 @@ func (s *clientService) PreRun(ctx context.Context) error {
 		GrpcAddress: node.GrpcAddress,
 		HttpAddress: node.HTTPAddress,
 		Roles:       nodeRoles,
+		Labels:      node.Labels,
 		CreatedAt:   timestamppb.Now(),
 	}
-	var cancel context.CancelFunc
 	for {
-		ctx, cancel = context.WithTimeout(ctx, time.Second*10)
-		err := s.schemaRegistry.RegisterNode(ctx, nodeInfo, s.forceRegisterNode)
+		ctxCancelable, cancel := context.WithTimeout(ctx, time.Second*10)
+		err := s.schemaRegistry.RegisterNode(ctxCancelable, nodeInfo, s.forceRegisterNode)
 		cancel()
-		if errors.Is(err, schema.ErrGRPCAlreadyExists) {
-			return errors.Wrapf(err, "node[%s] already exists in etcd", node.NodeID)
-		} else if errors.Is(err, context.DeadlineExceeded) {
+		if errors.Is(err, schema.ErrGRPCAlreadyExists) ||
+			errors.Is(err, context.DeadlineExceeded) || errors.Is(err, context.Canceled) {
+			// Log the specific error
+			l.Warn().Err(err).Strs("etcd-endpoints", s.endpoints).Msg("register node error")
+
 			select {
 			case <-stopCh:
 				return errors.New("register node interrupted")
@@ -179,19 +194,24 @@ func (s *clientService) PreRun(ctx context.Context) error {
 			case <-s.closer.CloseNotify():
 				return errors.New("register node interrupted")
 			default:
-				l.Warn().Strs("etcd-endpoints", s.endpoints).Msg("register node timeout, retrying...")
 				time.Sleep(time.Second)
 				continue
 			}
 		}
 		if err == nil {
 			l.Info().Stringer("info", nodeInfo).Msg("register node successfully")
+			s.nodeInfoMux.Lock()
+			s.nodeInfo = nodeInfo
+			s.nodeInfoMux.Unlock()
 		}
 		return err
 	}
 }
 
 func (s *clientService) Serve() run.StopNotify {
+	if s.schemaRegistry != nil {
+		s.schemaRegistry.StartWatcher()
+	}
 	return s.closer.CloseNotify()
 }
 
@@ -233,6 +253,10 @@ func (s *clientService) TopNAggregationRegistry() schema.TopNAggregation {
 	return s.schemaRegistry
 }
 
+func (s *clientService) NodeRegistry() schema.Node {
+	return s.schemaRegistry
+}
+
 func (s *clientService) PropertyRegistry() schema.Property {
 	return s.schemaRegistry
 }
@@ -337,3 +361,50 @@ func contains(s []string, e string) bool {
 	}
 	return false
 }
+
+func (s *clientService) UpdateSegmentsBoundary(stage, group string, boundary *modelv1.TimeRange) {
+	s.nodeInfoMux.Lock()
+	defer s.nodeInfoMux.Unlock()
+	nodeInfo := s.nodeInfo
+	b := nodeInfo.DataSegmentsBoundary
+	if b == nil {
+		b = make(map[string]*modelv1.TimeRange)
+	}
+	key := getNodeBoundaryKey(group, stage)
+	if boundary == nil {
+		delete(b, key)
+	} else {
+		b[key] = boundary
+	}
+	nodeInfo.DataSegmentsBoundary = b
+	s.nodeInfo = nodeInfo
+	if err := s.schemaRegistry.UpdateNode(context.Background(), nodeInfo); err != nil {
+		logger.GetLogger(s.Name()).Error().Err(err).Msg("failed to update node")
+	}
+}
+
+func getNodeBoundaryKey(items ...string) string {
+	return strings.Join(items, "|")
+}
+
+// FindSegmentsBoundary finds the segments boundary for the given group.
+func FindSegmentsBoundary(nodeInfo *databasev1.Node, group string) *modelv1.TimeRange {
+	b := nodeInfo.DataSegmentsBoundary
+	if len(b) == 0 {
+		return nil
+	}
+	result := &modelv1.TimeRange{}
+	for g, tr := range b {
+		segments := strings.Split(g, "|")
+		if segments[0] != group {
+			continue
+		}
+		if result.Begin == nil || result.Begin.AsTime().After(tr.Begin.AsTime()) {
+			result.Begin = tr.Begin
+		}
+		if result.End == nil || result.End.AsTime().Before(tr.End.AsTime()) {
+			result.End = tr.End
+		}
+	}
+	return result
+}
