@@ -81,6 +81,7 @@ func (s *server) PreRun(ctx context.Context) error {
 }
 
 func (s *server) Serve() run.StopNotify {
+	_ = s.Service.Serve()
 	return s.metaServer.StoppingNotify()
 }
 
@@ -96,7 +97,7 @@ func (s *server) GracefulStop() {
 func NewService(_ context.Context) (metadata.Service, error) {
 	s := &server{}
 	var err error
-	s.Service, err = metadata.NewClient(true)
+	s.Service, err = metadata.NewClient(true, true)
 	if err != nil {
 		return nil, err
 	}
