@@ -27,7 +27,6 @@ import (
 
 	commonv1 "github.com/apache/skywalking-banyandb/api/proto/banyandb/common/v1"
 	databasev1 "github.com/apache/skywalking-banyandb/api/proto/banyandb/database/v1"
-	propertyv1 "github.com/apache/skywalking-banyandb/api/proto/banyandb/property/v1"
 )
 
 var errUnsupportedEntityType = errors.New("unsupported entity type")
@@ -52,6 +51,9 @@ type ListOpt struct {
 	Group string
 }
 
+// WatcherOption is a placeholder for watcher configuration.
+type WatcherOption func(*watcherConfig)
+
 // Registry allowing depositing resources.
 type Registry interface {
 	io.Closer
@@ -61,11 +63,13 @@ type Registry interface {
 	Measure
 	Group
 	TopNAggregation
-	Property
 	Node
+	Property
 	RegisterHandler(string, Kind, EventHandler)
-	NewWatcher(string, Kind, watchEventHandler) *watcher
+	NewWatcher(string, Kind, ...WatcherOption) *watcher
 	Register(context.Context, Metadata, bool) error
+	Compact(context.Context, int64) error
+	StartWatcher()
 }
 
 // TypeMeta defines the identity and type of an Event.
@@ -109,11 +113,7 @@ func (m Metadata) key() (string, error) {
 			Group: m.Group,
 			Name:  m.Name,
 		}), nil
-	case KindProperty:
-		return formatPropertyKey(&commonv1.Metadata{
-			Group: m.Group,
-			Name:  m.Name,
-		}), nil
+
 	case KindTopNAggregation:
 		return formatTopNAggregationKey(&commonv1.Metadata{
 			Group: m.Group,
@@ -121,6 +121,11 @@ func (m Metadata) key() (string, error) {
 		}), nil
 	case KindNode:
 		return formatNodeKey(m.Name), nil
+	case KindProperty:
+		return formatPropertyKey(&commonv1.Metadata{
+			Group: m.Group,
+			Name:  m.Name,
+		}), nil
 	default:
 		return "", errUnsupportedEntityType
 	}
@@ -194,18 +199,19 @@ type TopNAggregation interface {
 	DeleteTopNAggregation(ctx context.Context, metadata *commonv1.Metadata) (bool, error)
 }
 
-// Property allows CRUD properties or tags in group.
-type Property interface {
-	GetProperty(ctx context.Context, metadata *propertyv1.Metadata, tags []string) (*propertyv1.Property, error)
-	ListProperty(ctx context.Context, container *commonv1.Metadata, ids []string, tags []string) ([]*propertyv1.Property, error)
-	ApplyProperty(ctx context.Context, property *propertyv1.Property, strategy propertyv1.ApplyRequest_Strategy) (bool, uint32, int64, error)
-	DeleteProperty(ctx context.Context, metadata *propertyv1.Metadata, tags []string) (bool, uint32, error)
-	KeepAlive(ctx context.Context, leaseID int64) error
-}
-
 // Node allows CRUD node schemas in a group.
 type Node interface {
 	ListNode(ctx context.Context, role databasev1.Role) ([]*databasev1.Node, error)
 	RegisterNode(ctx context.Context, node *databasev1.Node, forced bool) error
 	GetNode(ctx context.Context, node string) (*databasev1.Node, error)
+	UpdateNode(ctx context.Context, node *databasev1.Node) error
+}
+
+// Property allows CRUD property schemas in a group.
+type Property interface {
+	GetProperty(ctx context.Context, metadata *commonv1.Metadata) (*databasev1.Property, error)
+	ListProperty(ctx context.Context, opt ListOpt) ([]*databasev1.Property, error)
+	CreateProperty(ctx context.Context, property *databasev1.Property) error
+	UpdateProperty(ctx context.Context, property *databasev1.Property) error
+	DeleteProperty(ctx context.Context, metadata *commonv1.Metadata) (bool, error)
 }
