@@ -20,11 +20,11 @@ package schema
 import (
 	"context"
 
-	"google.golang.org/protobuf/proto"
 	"google.golang.org/protobuf/types/known/timestamppb"
 
 	commonv1 "github.com/apache/skywalking-banyandb/api/proto/banyandb/common/v1"
 	databasev1 "github.com/apache/skywalking-banyandb/api/proto/banyandb/database/v1"
+	"github.com/apache/skywalking-banyandb/api/validate"
 )
 
 var topNAggregationKeyPrefix = "/topnagg/"
@@ -41,9 +41,7 @@ func (e *etcdSchemaRegistry) ListTopNAggregation(ctx context.Context, opt ListOp
 	if opt.Group == "" {
 		return nil, BadRequest("group", "group should not be empty")
 	}
-	messages, err := e.listWithPrefix(ctx, listPrefixesForEntity(opt.Group, topNAggregationKeyPrefix), func() proto.Message {
-		return &databasev1.TopNAggregation{}
-	})
+	messages, err := e.listWithPrefix(ctx, listPrefixesForEntity(opt.Group, topNAggregationKeyPrefix), KindTopNAggregation)
 	if err != nil {
 		return nil, err
 	}
@@ -58,7 +56,10 @@ func (e *etcdSchemaRegistry) CreateTopNAggregation(ctx context.Context, topNAggr
 	if topNAggregation.UpdatedAt != nil {
 		topNAggregation.UpdatedAt = timestamppb.Now()
 	}
-	return e.create(ctx, Metadata{
+	if err := validate.TopNAggregation(topNAggregation); err != nil {
+		return err
+	}
+	_, err := e.create(ctx, Metadata{
 		TypeMeta: TypeMeta{
 			Kind:  KindTopNAggregation,
 			Group: topNAggregation.GetMetadata().GetGroup(),
@@ -66,10 +67,17 @@ func (e *etcdSchemaRegistry) CreateTopNAggregation(ctx context.Context, topNAggr
 		},
 		Spec: topNAggregation,
 	})
+	return err
 }
 
 func (e *etcdSchemaRegistry) UpdateTopNAggregation(ctx context.Context, topNAggregation *databasev1.TopNAggregation) error {
-	return e.update(ctx, Metadata{
+	if topNAggregation.UpdatedAt != nil {
+		topNAggregation.UpdatedAt = timestamppb.Now()
+	}
+	if err := validate.TopNAggregation(topNAggregation); err != nil {
+		return err
+	}
+	_, err := e.update(ctx, Metadata{
 		TypeMeta: TypeMeta{
 			Kind:  KindTopNAggregation,
 			Group: topNAggregation.GetMetadata().GetGroup(),
@@ -77,6 +85,7 @@ func (e *etcdSchemaRegistry) UpdateTopNAggregation(ctx context.Context, topNAggr
 		},
 		Spec: topNAggregation,
 	})
+	return err
 }
 
 func (e *etcdSchemaRegistry) DeleteTopNAggregation(ctx context.Context, metadata *commonv1.Metadata) (bool, error) {
