@@ -20,9 +20,11 @@ package v1
 import (
 	"bytes"
 	"fmt"
+	"strconv"
 
 	"github.com/pkg/errors"
 
+	databasev1 "github.com/apache/skywalking-banyandb/api/proto/banyandb/database/v1"
 	modelv1 "github.com/apache/skywalking-banyandb/api/proto/banyandb/model/v1"
 	"github.com/apache/skywalking-banyandb/pkg/convert"
 	"github.com/apache/skywalking-banyandb/pkg/encoding"
@@ -63,13 +65,72 @@ func MustTagValueToValueType(tag *modelv1.TagValue) ValueType {
 	}
 }
 
+// MustTagValueSpecToValueType converts databasev1.TagType to ValueType.
+func MustTagValueSpecToValueType(tag databasev1.TagType) ValueType {
+	switch tag {
+	case databasev1.TagType_TAG_TYPE_STRING:
+		return ValueTypeStr
+	case databasev1.TagType_TAG_TYPE_INT:
+		return ValueTypeInt64
+	case databasev1.TagType_TAG_TYPE_DATA_BINARY:
+		return ValueTypeBinaryData
+	case databasev1.TagType_TAG_TYPE_STRING_ARRAY:
+		return ValueTypeStrArr
+	case databasev1.TagType_TAG_TYPE_INT_ARRAY:
+		return ValueTypeInt64Arr
+	default:
+		panic("unknown tag value type")
+	}
+}
+
+// MustTagValueToStr converts modelv1.TagValue to string.
+func MustTagValueToStr(tag *modelv1.TagValue) string {
+	switch tag.Value.(type) {
+	case *modelv1.TagValue_Str:
+		return `"` + tag.GetStr().Value + `"`
+	case *modelv1.TagValue_Int:
+		return strconv.FormatInt(tag.GetInt().Value, 10)
+	case *modelv1.TagValue_BinaryData:
+		return fmt.Sprintf("%x", tag.GetBinaryData())
+	default:
+		panic("unknown tag value type")
+	}
+}
+
+// MarshalTagValues marshals tag values.
+func MarshalTagValues(dest []byte, tags []*modelv1.TagValue) ([]byte, error) {
+	var err error
+	for _, tag := range tags {
+		dest, err = marshalTagValue(dest, tag)
+		if err != nil {
+			return nil, err
+		}
+	}
+	return dest, nil
+}
+
+// UnmarshalTagValues unmarshals tag values.
+func UnmarshalTagValues(dest []byte, destTags []*modelv1.TagValue, src []byte) ([]byte, []*modelv1.TagValue, error) {
+	var err error
+	var tag *modelv1.TagValue
+	for len(src) > 0 {
+		dest = dest[:0]
+		dest, src, tag, err = unmarshalTagValue(dest, src)
+		if err != nil {
+			return nil, nil, err
+		}
+		destTags = append(destTags, tag)
+	}
+	return dest, destTags, nil
+}
+
 func marshalTagValue(dest []byte, tv *modelv1.TagValue) ([]byte, error) {
 	dest = append(dest, byte(MustTagValueToValueType(tv)))
 	switch tv.Value.(type) {
 	case *modelv1.TagValue_Null:
 		dest = marshalEntityValue(dest, nil)
 	case *modelv1.TagValue_Str:
-		dest = marshalEntityValue(dest, convert.StringToBytes(tv.GetStr().Value))
+		dest = marshalEntityValue(dest, []byte(tv.GetStr().Value))
 	case *modelv1.TagValue_Int:
 		dest = marshalEntityValue(dest, encoding.Int64ToBytes(nil, tv.GetInt().Value))
 	case *modelv1.TagValue_BinaryData:
@@ -102,6 +163,9 @@ func unmarshalTagValue(dest []byte, src []byte) ([]byte, []byte, *modelv1.TagVal
 		if dest, src, err = unmarshalEntityValue(dest, src[1:]); err != nil {
 			return nil, nil, nil, errors.WithMessage(err, "unmarshal string tag value")
 		}
+		if len(dest) == 0 {
+			return dest, src, NullTagValue, nil
+		}
 		return dest, src, &modelv1.TagValue{
 			Value: &modelv1.TagValue_Str{
 				Str: &modelv1.Str{
@@ -124,6 +188,9 @@ func unmarshalTagValue(dest []byte, src []byte) ([]byte, []byte, *modelv1.TagVal
 		if dest, src, err = unmarshalEntityValue(dest, src[1:]); err != nil {
 			return nil, nil, nil, errors.WithMessage(err, "unmarshal binary tag value")
 		}
+		if len(dest) == 0 {
+			return dest, src, NullTagValue, nil
+		}
 		data := make([]byte, len(dest))
 		copy(data, dest)
 		return dest, src, &modelv1.TagValue{
