@@ -80,6 +80,28 @@ func (uis *unresolvedIndexScan) Analyze(s logical.Schema) (logical.Plan, error)
 		}
 	}
 
+	tr := timestamp.NewInclusiveTimeRange(uis.startTime, uis.endTime)
+	ms := s.(*schema)
+	if ms.measure.IndexMode {
+		query, err := inverted.BuildIndexModeQuery(uis.metadata.Name, uis.criteria, s)
+		if err != nil {
+			return nil, err
+		}
+		return &localIndexScan{
+			timeRange:            tr,
+			schema:               s,
+			projectionTags:       projTags,
+			projectionFields:     projField,
+			projectionTagsRefs:   projTagsRefs,
+			projectionFieldsRefs: projFieldRefs,
+			metadata:             uis.metadata,
+			query:                query,
+			groupByEntity:        uis.groupByEntity,
+			uis:                  uis,
+			l:                    logger.GetLogger("query", "measure", uis.metadata.Group, uis.metadata.Name, "local-index"),
+		}, nil
+	}
+
 	entityList := s.EntityList()
 	entityMap := make(map[string]int)
 	entity := make([]*modelv1.TagValue, len(entityList))
@@ -88,13 +110,13 @@ func (uis *unresolvedIndexScan) Analyze(s logical.Schema) (logical.Plan, error)
 		// fill AnyEntry by default
 		entity[idx] = pbv1.AnyTagValue
 	}
-	query, entities, _, err := inverted.BuildLocalQuery(uis.criteria, s, entityMap, entity)
+	query, entities, _, err := inverted.BuildQuery(uis.criteria, s, entityMap, entity)
 	if err != nil {
 		return nil, err
 	}
 
 	return &localIndexScan{
-		timeRange:            timestamp.NewInclusiveTimeRange(uis.startTime, uis.endTime),
+		timeRange:            tr,
 		schema:               s,
 		projectionTags:       projTags,
 		projectionFields:     projField,
@@ -135,29 +157,33 @@ func (i *localIndexScan) Sort(order *logical.OrderBy) {
 }
 
 func (i *localIndexScan) Execute(ctx context.Context) (mit executor.MIterator, err error) {
-	var orderBy *model.OrderBy
-	orderByType := model.OrderByTypeTime
+	var orderBy *index.OrderBy
+
 	if i.order != nil {
-		if i.order.Index != nil {
-			orderByType = model.OrderByTypeIndex
-		}
-		orderBy = &model.OrderBy{
-			Index: i.order.Index,
+		orderBy = &index.OrderBy{
 			Sort:  i.order.Sort,
+			Index: i.order.Index,
+		}
+		if orderBy.Index == nil {
+			orderBy.Type = index.OrderByTypeTime
+		} else {
+			orderBy.Type = index.OrderByTypeIndex
 		}
 	}
 	if i.groupByEntity {
-		orderByType = model.OrderByTypeSeries
+		if orderBy == nil {
+			orderBy = &index.OrderBy{}
+		}
+		orderBy.Type = index.OrderByTypeSeries
 	}
 	ec := executor.FromMeasureExecutionContext(ctx)
-	ctx, stop := i.startSpan(ctx, query.GetTracer(ctx), orderByType, orderBy)
+	ctx, stop := i.startSpan(ctx, query.GetTracer(ctx), orderBy)
 	defer stop(err)
 	result, err := ec.Query(ctx, model.MeasureQueryOptions{
 		Name:            i.metadata.GetName(),
 		TimeRange:       &i.timeRange,
 		Entities:        i.entities,
 		Query:           i.query,
-		OrderByType:     orderByType,
 		Order:           orderBy,
 		TagProjection:   i.projectionTags,
 		FieldProjection: i.projectionFields,
@@ -269,20 +295,24 @@ func (ei *resultMIterator) Close() error {
 	return ei.err
 }
 
-func (i *localIndexScan) startSpan(ctx context.Context, tracer *query.Tracer, orderType model.OrderByType, orderBy *model.OrderBy) (context.Context, func(error)) {
+func (i *localIndexScan) startSpan(ctx context.Context, tracer *query.Tracer, orderBy *index.OrderBy) (context.Context, func(error)) {
 	if tracer == nil {
 		return ctx, func(error) {}
 	}
 
 	span, ctx := tracer.StartSpan(ctx, "indexScan-%s", i.metadata)
-	sortName := modelv1.Sort_name[int32(orderBy.Sort)]
-	switch orderType {
-	case model.OrderByTypeTime:
-		span.Tag("orderBy", "time "+sortName)
-	case model.OrderByTypeIndex:
-		span.Tag("orderBy", fmt.Sprintf("indexRule:%s", orderBy.Index.Metadata.Name))
-	case model.OrderByTypeSeries:
-		span.Tag("orderBy", "series")
+	if orderBy != nil {
+		sortName := modelv1.Sort_name[int32(orderBy.Sort)]
+		switch orderBy.Type {
+		case index.OrderByTypeTime:
+			span.Tag("orderBy", "time-"+sortName)
+		case index.OrderByTypeIndex:
+			span.Tag("orderBy", fmt.Sprintf("indexRule:%s-%s", orderBy.Index.Metadata.Name, sortName))
+		case index.OrderByTypeSeries:
+			span.Tag("orderBy", "series")
+		}
+	} else {
+		span.Tag("orderBy", "time-asc(default)")
 	}
 	span.Tag("details", i.String())
 
