@@ -45,6 +45,7 @@ import (
 type topNQueryProcessor struct {
 	measureService measure.Service
 	*queryService
+	*bus.UnImplementedHealthyListener
 }
 
 func (t *topNQueryProcessor) Rev(ctx context.Context, message bus.Message) (resp bus.Message) {
@@ -94,21 +95,7 @@ func (t *topNQueryProcessor) Rev(ctx context.Context, message bus.Message) (resp
 		return
 	}
 
-	topNResultMeasure, err := t.measureService.Measure(topNMetadata)
-	if err != nil {
-		t.log.Error().Err(err).
-			Str("topN", topNMetadata.GetName()).
-			Msg("fail to find topN result measure")
-		return
-	}
-
-	s, err := logical_measure.BuildTopNSchema(topNResultMeasure.GetSchema())
-	if err != nil {
-		t.log.Error().Err(err).
-			Str("topN", topNMetadata.GetName()).
-			Msg("fail to build schema")
-	}
-	plan, err := logical_measure.TopNAnalyze(ctx, request, topNResultMeasure.GetSchema(), sourceMeasure.GetSchema(), s)
+	plan, err := logical_measure.TopNAnalyze(request, sourceMeasure.GetSchema(), topNSchema)
 	if err != nil {
 		resp = bus.NewMessage(bus.MessageID(now), common.NewError("fail to analyze the query request for topn %s: %v", topNMetadata.GetName(), err))
 		return
@@ -117,6 +104,11 @@ func (t *topNQueryProcessor) Rev(ctx context.Context, message bus.Message) (resp
 	if e := ml.Debug(); e.Enabled() {
 		e.Str("plan", plan.String()).Msg("topn plan")
 	}
+	topNResultMeasure, err := t.measureService.Measure(measure.GetTopNSchemaMetadata(topNMetadata.Group))
+	if err != nil {
+		ml.Error().Err(err).Str("topN", topNMetadata.GetName()).Msg("fail to find topn result measure")
+		return
+	}
 	var tracer *query.Tracer
 	var span *query.Span
 	if request.Trace {
@@ -128,8 +120,8 @@ func (t *topNQueryProcessor) Rev(ctx context.Context, message bus.Message) (resp
 			switch d := data.(type) {
 			case *measurev1.TopNResponse:
 				d.Trace = tracer.ToProto()
-			case common.Error:
-				span.Error(errors.New(d.Msg()))
+			case *common.Error:
+				span.Error(errors.New(d.Error()))
 				resp = bus.NewMessage(bus.MessageID(now), &measurev1.QueryResponse{Trace: tracer.ToProto()})
 			default:
 				panic("unexpected data type")
