@@ -26,9 +26,12 @@ import (
 	"google.golang.org/grpc/credentials/insecure"
 	"google.golang.org/grpc/health/grpc_health_v1"
 
+	"github.com/apache/skywalking-banyandb/api/common"
 	clusterv1 "github.com/apache/skywalking-banyandb/api/proto/banyandb/cluster/v1"
 	databasev1 "github.com/apache/skywalking-banyandb/api/proto/banyandb/database/v1"
+	modelv1 "github.com/apache/skywalking-banyandb/api/proto/banyandb/model/v1"
 	"github.com/apache/skywalking-banyandb/banyand/metadata/schema"
+	"github.com/apache/skywalking-banyandb/pkg/bus"
 	"github.com/apache/skywalking-banyandb/pkg/grpchelper"
 	"github.com/apache/skywalking-banyandb/pkg/logger"
 )
@@ -97,34 +100,57 @@ func (p *pub) OnAddOrUpdate(md schema.Metadata) {
 	p.mu.Lock()
 	defer p.mu.Unlock()
 
-	p.registered[name] = struct{}{}
+	p.registerNode(node)
 
-	// If the client already exists, just return
 	if _, ok := p.active[name]; ok {
 		return
 	}
 	if _, ok := p.evictable[name]; ok {
 		return
 	}
-	conn, err := grpc.Dial(address, grpc.WithTransportCredentials(insecure.NewCredentials()), grpc.WithDefaultServiceConfig(retryPolicy))
+	conn, err := grpc.NewClient(address, grpc.WithTransportCredentials(insecure.NewCredentials()), grpc.WithDefaultServiceConfig(retryPolicy))
 	if err != nil {
 		p.log.Error().Err(err).Msg("failed to connect to grpc server")
 		return
 	}
 
-	if !p.checkClient(conn, md) {
-		p.log.Info().Str("status", p.dump()).Stringer("node", node).Msg("node is unhealthy, move it to evict queue")
+	if !p.checkClientHealthAndReconnect(conn, md) {
+		p.log.Info().Str("status", p.dump()).Stringer("node", node).Msg("node is unhealthy in the register flow, move it to evict queue")
 		return
 	}
 
 	c := clusterv1.NewServiceClient(conn)
 	p.active[name] = &client{conn: conn, client: c, md: md}
-	if p.handler != nil {
-		p.handler.OnAddOrUpdate(md)
-	}
+	p.addClient(md)
 	p.log.Info().Str("status", p.dump()).Stringer("node", node).Msg("new node is healthy, add it to active queue")
 }
 
+func (p *pub) registerNode(node *databasev1.Node) {
+	name := node.Metadata.GetName()
+	defer func() {
+		p.registered[name] = node
+	}()
+
+	n, ok := p.registered[name]
+	if !ok {
+		return
+	}
+	if n.GrpcAddress == node.GrpcAddress {
+		return
+	}
+	if en, ok := p.evictable[name]; ok {
+		close(en.c)
+		delete(p.evictable, name)
+		p.log.Info().Str("node", name).Str("status", p.dump()).Msg("node is removed from evict queue by the new gRPC address updated event")
+	}
+	if client, ok := p.active[name]; ok {
+		_ = client.conn.Close()
+		delete(p.active, name)
+		p.deleteClient(client.md)
+		p.log.Info().Str("status", p.dump()).Str("node", name).Msg("node is removed from active queue by the new gRPC address updated event")
+	}
+}
+
 func (p *pub) OnDelete(md schema.Metadata) {
 	if md.Kind != schema.KindNode {
 		return
@@ -149,23 +175,68 @@ func (p *pub) OnDelete(md schema.Metadata) {
 		return
 	}
 
-	if client, ok := p.active[name]; ok && !p.healthCheck(node, client.conn) {
-		_ = client.conn.Close()
-		delete(p.active, name)
-		if p.handler != nil {
-			p.handler.OnDelete(md)
+	if client, ok := p.active[name]; ok {
+		if p.removeNodeIfUnhealthy(md, node, client) {
+			p.log.Info().Str("status", p.dump()).Stringer("node", node).Msg("remove node from active queue by delete event")
+			return
 		}
-		p.log.Info().Str("status", p.dump()).Stringer("node", node).Msg("node is removed from active queue by delete event")
+		if !p.closer.AddRunning() {
+			return
+		}
+		go func() {
+			defer p.closer.Done()
+			backoff := initBackoff
+			var elapsed time.Duration
+			for {
+				select {
+				case <-time.After(backoff):
+					if func() bool {
+						elapsed += backoff
+						p.mu.Lock()
+						defer p.mu.Unlock()
+						if _, ok := p.registered[name]; ok {
+							// The client has been added back to registered clients map, just return
+							return true
+						}
+						if p.removeNodeIfUnhealthy(md, node, client) {
+							p.log.Info().Str("status", p.dump()).Stringer("node", node).Dur("after", elapsed).Msg("remove node from active queue by delete event")
+							return true
+						}
+						return false
+					}() {
+						return
+					}
+				case <-p.closer.CloseNotify():
+					return
+				}
+				if backoff < maxBackoff {
+					backoff *= time.Duration(backoffMultiplier)
+				} else {
+					backoff = maxBackoff
+				}
+			}
+		}()
 	}
 }
 
-func (p *pub) checkClient(conn *grpc.ClientConn, md schema.Metadata) bool {
+func (p *pub) removeNodeIfUnhealthy(md schema.Metadata, node *databasev1.Node, client *client) bool {
+	if p.healthCheck(node.String(), client.conn) {
+		return false
+	}
+	_ = client.conn.Close()
+	name := node.Metadata.GetName()
+	delete(p.active, name)
+	p.deleteClient(md)
+	return true
+}
+
+func (p *pub) checkClientHealthAndReconnect(conn *grpc.ClientConn, md schema.Metadata) bool {
 	node, ok := md.Spec.(*databasev1.Node)
 	if !ok {
 		logger.Panicf("failed to cast node spec")
 		return false
 	}
-	if p.healthCheck(node, conn) {
+	if p.healthCheck(node.String(), conn) {
 		return true
 	}
 	_ = conn.Close()
@@ -174,30 +245,28 @@ func (p *pub) checkClient(conn *grpc.ClientConn, md schema.Metadata) bool {
 	}
 	name := node.Metadata.Name
 	p.evictable[name] = evictNode{n: node, c: make(chan struct{})}
-	if p.handler != nil {
-		p.handler.OnDelete(md)
-	}
+	p.deleteClient(md)
 	go func(p *pub, name string, en evictNode, md schema.Metadata) {
 		defer p.closer.Done()
 		backoff := initBackoff
 		for {
 			select {
 			case <-time.After(backoff):
-				connEvict, errEvict := grpc.Dial(node.GrpcAddress, grpc.WithTransportCredentials(insecure.NewCredentials()), grpc.WithDefaultServiceConfig(retryPolicy))
-				if errEvict == nil && p.healthCheck(en.n, connEvict) {
-					p.mu.Lock()
-					defer p.mu.Unlock()
-					if _, ok := p.evictable[name]; !ok {
-						// The client has been removed from evict clients map, just return
-						return
-					}
-					c := clusterv1.NewServiceClient(connEvict)
-					p.active[name] = &client{conn: connEvict, client: c, md: md}
-					if p.handler != nil {
-						p.handler.OnAddOrUpdate(md)
-					}
-					delete(p.evictable, name)
-					p.log.Info().Stringer("node", en.n).Msg("node is healthy, move it back to active queue")
+				connEvict, errEvict := grpc.NewClient(node.GrpcAddress, grpc.WithTransportCredentials(insecure.NewCredentials()), grpc.WithDefaultServiceConfig(retryPolicy))
+				if errEvict == nil && p.healthCheck(en.n.String(), connEvict) {
+					func() {
+						p.mu.Lock()
+						defer p.mu.Unlock()
+						if _, ok := p.evictable[name]; !ok {
+							// The client has been removed from evict clients map, just return
+							return
+						}
+						c := clusterv1.NewServiceClient(connEvict)
+						p.active[name] = &client{conn: connEvict, client: c, md: md}
+						p.addClient(md)
+						delete(p.evictable, name)
+						p.log.Info().Str("status", p.dump()).Stringer("node", en.n).Msg("node is healthy, move it back to active queue")
+					}()
 					return
 				}
 				if errEvict != nil {
@@ -209,6 +278,8 @@ func (p *pub) checkClient(conn *grpc.ClientConn, md schema.Metadata) bool {
 				p.log.Error().Err(errEvict).Msgf("failed to re-connect to grpc server after waiting for %s", backoff)
 			case <-en.c:
 				return
+			case <-p.closer.CloseNotify():
+				return
 			}
 			if backoff < maxBackoff {
 				backoff *= time.Duration(backoffMultiplier)
@@ -220,7 +291,7 @@ func (p *pub) checkClient(conn *grpc.ClientConn, md schema.Metadata) bool {
 	return false
 }
 
-func (p *pub) healthCheck(node fmt.Stringer, conn *grpc.ClientConn) bool {
+func (p *pub) healthCheck(node string, conn *grpc.ClientConn) bool {
 	var resp *grpc_health_v1.HealthCheckResponse
 	if err := grpchelper.Request(context.Background(), rpcTimeout, func(rpcCtx context.Context) (err error) {
 		resp, err = grpc_health_v1.NewHealthClient(conn).Check(rpcCtx,
@@ -230,7 +301,7 @@ func (p *pub) healthCheck(node fmt.Stringer, conn *grpc.ClientConn) bool {
 		return err
 	}); err != nil {
 		if e := p.log.Debug(); e.Enabled() {
-			e.Err(err).Stringer("node", node).Msg("service unhealthy")
+			e.Err(err).Str("node", node).Msg("service unhealthy")
 		}
 		return false
 	}
@@ -240,23 +311,108 @@ func (p *pub) healthCheck(node fmt.Stringer, conn *grpc.ClientConn) bool {
 	return false
 }
 
-func (p *pub) failover(node string) {
+func (p *pub) checkServiceHealth(svc string, conn *grpc.ClientConn) *common.Error {
+	client := clusterv1.NewServiceClient(conn)
+	ctx, cancel := context.WithTimeout(context.Background(), rpcTimeout)
+	defer cancel()
+	resp, err := client.HealthCheck(ctx, &clusterv1.HealthCheckRequest{
+		ServiceName: svc,
+	})
+	if err != nil {
+		return common.NewErrorWithStatus(modelv1.Status_STATUS_INTERNAL_ERROR, err.Error())
+	}
+	if resp.Status == modelv1.Status_STATUS_SUCCEED {
+		return nil
+	}
+	return common.NewErrorWithStatus(resp.Status, resp.Error)
+}
+
+func (p *pub) failover(node string, ce *common.Error, topic bus.Topic) {
 	p.mu.Lock()
 	defer p.mu.Unlock()
-	if en, ok := p.evictable[node]; ok {
-		close(en.c)
-		delete(p.evictable, node)
-		p.log.Info().Str("node", node).Str("status", p.dump()).Msg("node is removed from evict queue by wire event")
+	if ce.Status() != modelv1.Status_STATUS_INTERNAL_ERROR {
+		_, _ = p.checkWritable(node, topic)
+		return
+	}
+	if en, evictable := p.evictable[node]; evictable {
+		if _, registered := p.registered[node]; !registered {
+			close(en.c)
+			delete(p.evictable, node)
+			p.log.Info().Str("node", node).Str("status", p.dump()).Msg("node is removed from evict queue by wire event")
+		}
 		return
 	}
 
-	if client, ok := p.active[node]; ok && !p.checkClient(client.conn, client.md) {
+	if client, ok := p.active[node]; ok && !p.checkClientHealthAndReconnect(client.conn, client.md) {
 		_ = client.conn.Close()
 		delete(p.active, node)
-		if p.handler != nil {
-			p.handler.OnDelete(client.md)
+		p.deleteClient(client.md)
+		p.log.Info().Str("status", p.dump()).Str("node", node).Msg("node is unhealthy in the failover flow, move it to evict queue")
+	}
+}
+
+func (p *pub) checkWritable(n string, topic bus.Topic) (bool, *common.Error) {
+	h, ok := p.handlers[topic]
+	if !ok {
+		return false, nil
+	}
+	node, ok := p.active[n]
+	if !ok {
+		return false, nil
+	}
+	err := p.checkServiceHealth(topic.String(), node.conn)
+	if err == nil {
+		return true, nil
+	}
+	h.OnDelete(node.md)
+	if !p.closer.AddRunning() {
+		return false, err
+	}
+	go func() {
+		defer p.closer.Done()
+		backoff := initBackoff
+		for {
+			select {
+			case <-time.After(backoff):
+				if errInternal := p.checkServiceHealth(topic.String(), node.conn); errInternal != nil {
+					func() {
+						p.mu.Lock()
+						defer p.mu.Unlock()
+						node, ok := p.active[n]
+						if !ok {
+							return
+						}
+						h.OnAddOrUpdate(node.md)
+					}()
+					return
+				}
+				p.log.Warn().Str("topic", topic.String()).Err(err).Str("node", n).Dur("backoff", backoff).Msg("data node can not ingest data")
+			case <-p.closer.CloseNotify():
+				return
+			}
+			if backoff < maxBackoff {
+				backoff *= time.Duration(backoffMultiplier)
+			} else {
+				backoff = maxBackoff
+			}
+		}
+	}()
+	return false, err
+}
+
+func (p *pub) deleteClient(md schema.Metadata) {
+	if len(p.handlers) > 0 {
+		for _, h := range p.handlers {
+			h.OnDelete(md)
+		}
+	}
+}
+
+func (p *pub) addClient(md schema.Metadata) {
+	if len(p.handlers) > 0 {
+		for _, h := range p.handlers {
+			h.OnAddOrUpdate(md)
 		}
-		p.log.Info().Str("status", p.dump()).Str("node", node).Msg("node is unhealthy, move it to evict queue")
 	}
 }
 
