@@ -32,15 +32,18 @@ import (
 	"google.golang.org/protobuf/proto"
 	"google.golang.org/protobuf/types/known/anypb"
 
+	"github.com/apache/skywalking-banyandb/api/common"
 	"github.com/apache/skywalking-banyandb/api/data"
 	clusterv1 "github.com/apache/skywalking-banyandb/api/proto/banyandb/cluster/v1"
 	databasev1 "github.com/apache/skywalking-banyandb/api/proto/banyandb/database/v1"
+	modelv1 "github.com/apache/skywalking-banyandb/api/proto/banyandb/model/v1"
 	"github.com/apache/skywalking-banyandb/banyand/metadata"
 	"github.com/apache/skywalking-banyandb/banyand/metadata/schema"
 	"github.com/apache/skywalking-banyandb/banyand/queue"
 	"github.com/apache/skywalking-banyandb/pkg/bus"
 	"github.com/apache/skywalking-banyandb/pkg/logger"
 	"github.com/apache/skywalking-banyandb/pkg/run"
+	"github.com/apache/skywalking-banyandb/pkg/timestamp"
 )
 
 var (
@@ -51,7 +54,7 @@ var (
 type pub struct {
 	schema.UnimplementedOnInitHandler
 	metadata   metadata.Repo
-	handler    schema.EventHandler
+	handlers   map[bus.Topic]schema.EventHandler
 	log        *logger.Logger
 	registered map[string]*databasev1.Node
 	active     map[string]*client
@@ -60,8 +63,8 @@ type pub struct {
 	mu         sync.RWMutex
 }
 
-func (p *pub) Register(handler schema.EventHandler) {
-	p.handler = handler
+func (p *pub) Register(topic bus.Topic, handler schema.EventHandler) {
+	p.handlers[topic] = handler
 }
 
 func (p *pub) GracefulStop() {
@@ -84,16 +87,54 @@ func (p *pub) Serve() run.StopNotify {
 	return p.closer.CloseNotify()
 }
 
+func bypassMatches(_ map[string]string) bool { return true }
+
 func (p *pub) Broadcast(timeout time.Duration, topic bus.Topic, messages bus.Message) ([]bus.Future, error) {
-	var names []string
+	var nodes []*databasev1.Node
 	p.mu.RLock()
 	for k := range p.active {
-		names = append(names, k)
+		nodes = append(nodes, p.registered[k])
 	}
 	p.mu.RUnlock()
+	if len(nodes) == 0 {
+		return nil, errors.New("no active nodes")
+	}
+	names := make(map[string]struct{})
+	if len(messages.NodeSelectors()) == 0 {
+		for _, n := range nodes {
+			names[n.Metadata.Name] = struct{}{}
+		}
+	} else {
+		for g, sel := range messages.NodeSelectors() {
+			var matches func(map[string]string) bool
+			if sel == "" {
+				matches = bypassMatches
+			} else {
+				selector, err := ParseLabelSelector(sel)
+				if err != nil {
+					return nil, fmt.Errorf("failed to parse node selector: %w", err)
+				}
+				matches = selector.Matches
+			}
+			for _, n := range nodes {
+				tr := metadata.FindSegmentsBoundary(n, g)
+				if tr == nil {
+					continue
+				}
+				if matches(n.Labels) && timestamp.PbHasOverlap(messages.TimeRange(), tr) {
+					names[n.Metadata.Name] = struct{}{}
+				}
+			}
+		}
+	}
+
+	if l := p.log.Debug(); l.Enabled() {
+		l.Msgf("broadcasting message to %s nodes", names)
+	}
+
 	futureCh := make(chan publishResult, len(names))
 	var wg sync.WaitGroup
-	for _, n := range names {
+	for n := range names {
 		wg.Add(1)
 		go func(n string) {
 			defer wg.Done()
@@ -114,7 +155,7 @@ func (p *pub) Broadcast(timeout time.Duration, topic bus.Topic, messages bus.Mes
 				if p.closer.AddRunning() {
 					go func() {
 						defer p.closer.Done()
-						p.failover(f.n)
+						p.failover(f.n, common.NewErrorWithStatus(modelv1.Status_STATUS_INTERNAL_ERROR, f.e.Error()), topic)
 					}()
 				}
 			}
@@ -170,18 +211,9 @@ func (p *pub) publish(timeout time.Duration, topic bus.Topic, messages ...bus.Me
 	return f, err
 }
 
-func (p *pub) Publish(_ context.Context, _ bus.Topic, _ ...bus.Message) (bus.Future, error) {
-	panic("should not be called")
-}
-
-// NewBatchPublisher returns a new batch publisher.
-func (p *pub) NewBatchPublisher(timeout time.Duration) queue.BatchPublisher {
-	return &batchPublisher{
-		pub:     p,
-		streams: make(map[string]writeStream),
-		timeout: timeout,
-		f:       batchFuture{errNodes: make(map[string]struct{}), l: p.log},
-	}
+func (p *pub) Publish(_ context.Context, topic bus.Topic, messages ...bus.Message) (bus.Future, error) {
+	// nolint: contextcheck
+	return p.publish(15*time.Second, topic, messages...)
 }
 
 // New returns a new queue client.
@@ -191,139 +223,30 @@ func New(metadata metadata.Repo) queue.Client {
 		active:     make(map[string]*client),
 		evictable:  make(map[string]evictNode),
 		registered: make(map[string]*databasev1.Node),
+		handlers:   make(map[bus.Topic]schema.EventHandler),
 		closer:     run.NewCloser(1),
 	}
 }
 
+// NewWithoutMetadata returns a new queue client without metadata.
+func NewWithoutMetadata() queue.Client {
+	p := New(nil)
+	p.(*pub).log = logger.GetLogger("queue-client")
+	return p
+}
+
 func (*pub) Name() string {
 	return "queue-client"
 }
 
 func (p *pub) PreRun(context.Context) error {
+	if p.metadata != nil {
+		p.metadata.RegisterHandler("queue-client", schema.KindNode, p)
+	}
 	p.log = logger.GetLogger("server-queue-pub")
-	p.metadata.RegisterHandler("queue-client", schema.KindNode, p)
 	return nil
 }
 
-type writeStream struct {
-	client    clusterv1.Service_SendClient
-	ctxDoneCh <-chan struct{}
-}
-
-type batchPublisher struct {
-	pub     *pub
-	streams map[string]writeStream
-	f       batchFuture
-	timeout time.Duration
-}
-
-func (bp *batchPublisher) Close() (err error) {
-	for i := range bp.streams {
-		err = multierr.Append(err, bp.streams[i].client.CloseSend())
-	}
-	for i := range bp.streams {
-		<-bp.streams[i].ctxDoneCh
-	}
-	if bp.pub.closer.AddRunning() {
-		go func(f *batchFuture) {
-			defer bp.pub.closer.Done()
-			for _, n := range f.get() {
-				bp.pub.failover(n)
-			}
-		}(&bp.f)
-	}
-	return err
-}
-
-func (bp *batchPublisher) Publish(ctx context.Context, topic bus.Topic, messages ...bus.Message) (bus.Future, error) {
-	var err error
-	for _, m := range messages {
-		r, errM2R := messageToRequest(topic, m)
-		if errM2R != nil {
-			err = multierr.Append(err, fmt.Errorf("failed to marshal message %T: %w", m, errM2R))
-			continue
-		}
-		node := m.Node()
-		sendData := func() (success bool) {
-			if stream, ok := bp.streams[node]; ok {
-				defer func() {
-					if !success {
-						delete(bp.streams, node)
-					}
-				}()
-				select {
-				case <-ctx.Done():
-					return false
-				case <-stream.client.Context().Done():
-					return false
-				default:
-				}
-				errSend := stream.client.Send(r)
-				if errSend != nil {
-					err = multierr.Append(err, fmt.Errorf("failed to send message to node %s: %w", node, errSend))
-					return false
-				}
-				return errSend == nil
-			}
-			return false
-		}
-		if sendData() {
-			continue
-		}
-
-		select {
-		case <-ctx.Done():
-			return nil, ctx.Err()
-		default:
-		}
-
-		bp.pub.mu.RLock()
-		client, ok := bp.pub.active[node]
-		bp.pub.mu.RUnlock()
-		if !ok {
-			err = multierr.Append(err, fmt.Errorf("failed to get client for node %s", node))
-			continue
-		}
-		streamCtx, cancel := context.WithTimeout(ctx, bp.timeout)
-		// this assignment is for getting around the go vet lint
-		deferFn := cancel
-		stream, errCreateStream := client.client.Send(streamCtx)
-		if errCreateStream != nil {
-			err = multierr.Append(err, fmt.Errorf("failed to get stream for node %s: %w", node, errCreateStream))
-			continue
-		}
-		bp.streams[node] = writeStream{
-			client:    stream,
-			ctxDoneCh: streamCtx.Done(),
-		}
-		bp.f.events = append(bp.f.events, make(chan batchEvent))
-		_ = sendData()
-		go func(s clusterv1.Service_SendClient, deferFn func(), bc chan batchEvent) {
-			defer func() {
-				close(bc)
-				deferFn()
-			}()
-			for {
-				select {
-				case <-ctx.Done():
-					return
-				default:
-				}
-				_, errRecv := s.Recv()
-				if errRecv == nil {
-					continue
-				}
-				if errors.Is(errRecv, io.EOF) {
-					return
-				}
-				bc <- batchEvent{n: node, e: errRecv}
-				return
-			}
-		}(stream, deferFn, bp.f.events[len(bp.f.events)-1])
-	}
-	return nil, err
-}
-
 func messageToRequest(topic bus.Topic, m bus.Message) (*clusterv1.SendRequest, error) {
 	r := &clusterv1.SendRequest{
 		Topic:     topic.String(),
@@ -400,49 +323,6 @@ func (l *future) GetAll() ([]bus.Message, error) {
 	}
 }
 
-type batchEvent struct {
-	e error
-	n string
-}
-
-type batchFuture struct {
-	errNodes map[string]struct{}
-	l        *logger.Logger
-	events   []chan batchEvent
-}
-
-func (b *batchFuture) get() []string {
-	var wg sync.WaitGroup
-	var mux sync.Mutex
-	wg.Add(len(b.events))
-	for _, e := range b.events {
-		go func(e chan batchEvent, mux *sync.Mutex) {
-			defer wg.Done()
-			for evt := range e {
-				func() {
-					mux.Lock()
-					defer mux.Unlock()
-					// only log the error once for each node
-					if _, ok := b.errNodes[evt.n]; !ok && isFailoverError(evt.e) {
-						b.l.Error().Err(evt.e).Msgf("failed to send message to node %s", evt.n)
-						b.errNodes[evt.n] = struct{}{}
-						return
-					}
-					b.l.Error().Err(evt.e).Msgf("failed to send message to node %s", evt.n)
-				}()
-			}
-		}(e, &mux)
-	}
-	wg.Wait()
-	mux.Lock()
-	defer mux.Unlock()
-	var result []string
-	for n := range b.errNodes {
-		result = append(result, n)
-	}
-	return result
-}
-
 func isFailoverError(err error) bool {
 	s, ok := status.FromError(err)
 	if !ok {
