@@ -18,9 +18,9 @@
 package queue
 
 import (
-	"io"
 	"time"
 
+	"github.com/apache/skywalking-banyandb/api/common"
 	"github.com/apache/skywalking-banyandb/banyand/metadata/schema"
 	"github.com/apache/skywalking-banyandb/pkg/bus"
 	"github.com/apache/skywalking-banyandb/pkg/run"
@@ -43,7 +43,9 @@ type Client interface {
 	bus.Publisher
 	bus.Broadcaster
 	NewBatchPublisher(timeout time.Duration) BatchPublisher
-	Register(schema.EventHandler)
+	Register(bus.Topic, schema.EventHandler)
+	OnAddOrUpdate(md schema.Metadata)
+	GracefulStop()
 }
 
 // Server is the interface for receiving data from the queue.
@@ -54,7 +56,9 @@ type Server interface {
 }
 
 // BatchPublisher is the interface for publishing data in batch.
+//
+//go:generate mockgen -destination=./batch_publisher_mock.go -package=queue . BatchPublisher
 type BatchPublisher interface {
 	bus.Publisher
-	io.Closer
+	Close() (map[string]*common.Error, error)
 }
