@@ -24,8 +24,6 @@ import (
 	"time"
 
 	"github.com/pkg/errors"
-	"google.golang.org/grpc/codes"
-	"google.golang.org/grpc/status"
 	"google.golang.org/protobuf/proto"
 	"google.golang.org/protobuf/types/known/anypb"
 
@@ -33,20 +31,10 @@ import (
 	"github.com/apache/skywalking-banyandb/api/data"
 	clusterv1 "github.com/apache/skywalking-banyandb/api/proto/banyandb/cluster/v1"
 	"github.com/apache/skywalking-banyandb/pkg/bus"
+	"github.com/apache/skywalking-banyandb/pkg/logger"
 )
 
 func (s *server) Send(stream clusterv1.Service_SendServer) error {
-	reply := func(writeEntity *clusterv1.SendRequest, err error, message string) {
-		s.log.Error().Stringer("request", writeEntity).Err(err).Msg(message)
-		s.metrics.totalMsgReceivedErr.Inc(1, writeEntity.Topic)
-		if errResp := stream.Send(&clusterv1.SendResponse{
-			MessageId: writeEntity.MessageId,
-			Error:     message,
-		}); errResp != nil {
-			s.log.Error().Err(errResp).AnErr("original", err).Stringer("request", writeEntity).Msg("failed to send error response")
-			s.metrics.totalMsgSentErr.Inc(1, writeEntity.Topic)
-		}
-	}
 	ctx := stream.Context()
 	var topic *bus.Topic
 	var m bus.Message
@@ -66,74 +54,51 @@ func (s *server) Send(stream clusterv1.Service_SendServer) error {
 		}
 		writeEntity, err := stream.Recv()
 		if errors.Is(err, io.EOF) {
-			if len(dataCollection) < 1 {
-				return nil
-			}
-			listener := s.getListeners(*topic)
-			if listener == nil {
-				reply(writeEntity, err, "no listener found")
-				return nil
-			}
-			_ = listener.Rev(ctx, bus.NewMessage(bus.MessageID(0), dataCollection))
+			s.handleEOF(stream, topic, dataCollection, writeEntity)
 			return nil
 		}
 		if err != nil {
-			// If the context is canceled or the deadline is exceeded, the stream will be closed.
-			// In this case, we should return nil to avoid logging the error.
-			// Deadline exceeded will be raised when other data nodes are not available, and the client timeout context is triggered.
-			if status.Code(err) == codes.Canceled || status.Code(err) == codes.DeadlineExceeded {
-				return nil
-			}
-			s.log.Error().Err(err).Msg("failed to receive message")
-			return err
+			return s.handleRecvError(err)
 		}
 		s.metrics.totalMsgReceived.Inc(1, writeEntity.Topic)
 		if writeEntity.Topic != "" && topic == nil {
 			t, ok := data.TopicMap[writeEntity.Topic]
 			if !ok {
-				reply(writeEntity, err, "invalid topic")
+				s.reply(stream, writeEntity, err, "invalid topic")
 				continue
 			}
 			topic = &t
 		}
 		if topic == nil {
-			reply(writeEntity, err, "topic is empty")
+			s.reply(stream, writeEntity, err, "topic is empty")
 			continue
 		}
 
 		if reqSupplier, ok := data.TopicRequestMap[*topic]; ok {
 			req := reqSupplier()
 			if errUnmarshal := writeEntity.Body.UnmarshalTo(req); errUnmarshal != nil {
-				reply(writeEntity, errUnmarshal, "failed to unmarshal message")
+				s.reply(stream, writeEntity, errUnmarshal, "failed to unmarshal message")
 				continue
 			}
 			m = bus.NewMessage(bus.MessageID(writeEntity.MessageId), req)
 		} else {
-			reply(writeEntity, err, "unknown topic")
+			s.reply(stream, writeEntity, err, "unknown topic")
 			continue
 		}
 		if writeEntity.BatchMod {
-			if len(dataCollection) == 0 {
-				s.metrics.totalStarted.Inc(1, writeEntity.Topic)
-				start = time.Now()
-			}
-			dataCollection = append(dataCollection, writeEntity.Body)
-			if errSend := stream.Send(&clusterv1.SendResponse{
-				MessageId: writeEntity.MessageId,
-			}); errSend != nil {
-				s.log.Error().Stringer("written", writeEntity).Err(errSend).Msg("failed to send write response")
-				s.metrics.totalMsgSentErr.Inc(1, writeEntity.Topic)
-				continue
-			}
-			s.metrics.totalMsgSent.Inc(1, writeEntity.Topic)
+			s.handleBatch(&dataCollection, writeEntity, &start)
 			continue
 		}
 		s.metrics.totalStarted.Inc(1, writeEntity.Topic)
-		listener := s.getListeners(*topic)
-		if listener == nil {
-			reply(writeEntity, err, "no listener found")
+		listeners := s.getListeners(*topic)
+		if len(listeners) == 0 {
+			s.reply(stream, writeEntity, err, "no listener found")
 			continue
 		}
+		if len(listeners) > 1 {
+			logger.Panicf("multiple listeners found for topic %s", *topic)
+		}
+		listener := listeners[0]
 
 		m = listener.Rev(ctx, m)
 		if m.Data() == nil {
@@ -151,22 +116,22 @@ func (s *server) Send(stream clusterv1.Service_SendServer) error {
 		switch d := m.Data().(type) {
 		case proto.Message:
 			message = d
-		case common.Error:
+		case *common.Error:
 			select {
 			case <-ctx.Done():
 				s.metrics.totalMsgReceivedErr.Inc(1, writeEntity.Topic)
 				return ctx.Err()
 			default:
 			}
-			reply(writeEntity, nil, d.Msg())
+			s.reply(stream, writeEntity, nil, d.Error())
 			continue
 		default:
-			reply(writeEntity, nil, fmt.Sprintf("invalid response: %T", d))
+			s.reply(stream, writeEntity, nil, fmt.Sprintf("invalid response: %T", d))
 			continue
 		}
 		anyMessage, err := anypb.New(message)
 		if err != nil {
-			reply(writeEntity, err, "failed to marshal message")
+			s.reply(stream, writeEntity, err, "failed to marshal message")
 			continue
 		}
 		if err := stream.Send(&clusterv1.SendResponse{
@@ -184,15 +149,44 @@ func (s *server) Send(stream clusterv1.Service_SendServer) error {
 func (s *server) Subscribe(topic bus.Topic, listener bus.MessageListener) error {
 	s.listenersLock.Lock()
 	defer s.listenersLock.Unlock()
-	if _, ok := s.listeners[topic]; !ok {
-		s.listeners[topic] = listener
+	listeners, ok := s.listeners[topic]
+	if ok {
+		listeners = append(listeners, listener)
+		s.listeners[topic] = listeners
 		return nil
 	}
-	return errors.New("topic already exists")
+	listeners = make([]bus.MessageListener, 0)
+	listeners = append(listeners, listener)
+	s.listeners[topic] = listeners
+	s.topicMap[topic.String()] = topic
+	return nil
 }
 
-func (s *server) getListeners(topic bus.Topic) bus.MessageListener {
+func (s *server) getListeners(topic bus.Topic) []bus.MessageListener {
 	s.listenersLock.RLock()
 	defer s.listenersLock.RUnlock()
 	return s.listeners[topic]
 }
+
+func (s *server) reply(stream clusterv1.Service_SendServer, writeEntity *clusterv1.SendRequest, err error, message string) {
+	s.log.Error().Stringer("request", writeEntity).Err(err).Msg(message)
+	s.metrics.totalMsgReceivedErr.Inc(1, writeEntity.Topic)
+	resp := &clusterv1.SendResponse{
+		MessageId: writeEntity.MessageId,
+	}
+
+	var ce *common.Error
+	if errors.As(err, &ce) {
+		resp.Error = ce.Error()
+		resp.Status = ce.Status()
+	} else {
+		resp.Error = message
+	}
+	if errResp := stream.Send(&clusterv1.SendResponse{
+		MessageId: writeEntity.MessageId,
+		Error:     message,
+	}); errResp != nil {
+		s.log.Error().Err(errResp).AnErr("original", err).Stringer("request", writeEntity).Msg("failed to send error response")
+		s.metrics.totalMsgSentErr.Inc(1, writeEntity.Topic)
+	}
+}
