@@ -90,8 +90,8 @@ func (mf mockFilter) String() string {
 	return "filter"
 }
 
-func (mf mockFilter) Execute(_ index.GetSearcher, seriesID common.SeriesID) (posting.List, error) {
-	return mf.index[mf.value][seriesID], nil
+func (mf mockFilter) Execute(_ index.GetSearcher, seriesID common.SeriesID, _ *index.RangeOpts) (posting.List, posting.List, error) {
+	return mf.index[mf.value][seriesID], roaring.DummyPostingList, nil
 }
 
 type databaseSupplier struct {
@@ -148,13 +148,11 @@ func generateData(p parameter) ([]*elements, []index.Documents, mockIndex) {
 				es.tagFamilies = append(es.tagFamilies, tfs)
 				idx.insert(value, common.SeriesID(k), int(unixTimestamp))
 				var fields []index.Field
-				fields = append(fields, index.Field{
-					Key: index.FieldKey{
-						IndexRuleID: 1,
-						SeriesID:    common.SeriesID(k),
-					},
-					Term: []byte(value),
-				})
+
+				fields = append(fields, index.NewBytesField(index.FieldKey{
+					IndexRuleID: 1,
+					SeriesID:    common.SeriesID(k),
+				}, []byte(value)))
 				docs = append(docs, index.Document{
 					DocID:  uint64(unixTimestamp),
 					Fields: fields,
@@ -220,7 +218,7 @@ func write(b *testing.B, p parameter, esList []*elements, docsList []index.Docum
 	}
 	seg, err := db.CreateSegmentIfNotExist(time.Unix(0, esList[0].timestamps[0]))
 	require.NoError(b, err)
-	seg.IndexDB().Write(docs)
+	seg.IndexDB().Insert(docs)
 
 	tst, err := seg.CreateTSTableIfNotExist(common.ShardID(0))
 	require.NoError(b, err)
@@ -258,13 +256,14 @@ func generateStream(db storage.TSDB[*tsTable, option]) *stream {
 		Entity:      entity,
 		TagFamilies: []*databasev1.TagFamilySpec{tagFamily},
 	}
-	return &stream{
-		databaseSupplier: dbSupplier,
-		schema:           schema,
+	s := &stream{
+		schema: schema,
 	}
+	s.tsdb.Store(db)
+	return s
 }
 
-func generateStreamQueryOptions(p parameter, index mockIndex) model.StreamQueryOptions {
+func generateStreamQueryOptions(p parameter, midx mockIndex) model.StreamQueryOptions {
 	timeRange := timestamp.TimeRange{
 		Start:        time.Unix(int64(p.startTimestamp), 0),
 		End:          time.Unix(int64(p.endTimestamp), 0),
@@ -287,7 +286,7 @@ func generateStreamQueryOptions(p parameter, index mockIndex) model.StreamQueryO
 	num := generateRandomNumber(int64(p.tagCardinality))
 	value := filterTagValuePrefix + strconv.Itoa(num)
 	filter := mockFilter{
-		index: index,
+		index: midx,
 		value: value,
 	}
 	indexRule := &databasev1.IndexRule{
@@ -297,7 +296,7 @@ func generateStreamQueryOptions(p parameter, index mockIndex) model.StreamQueryO
 		Tags: []string{"filter-tag"},
 		Type: databasev1.IndexRule_TYPE_INVERTED,
 	}
-	order := &model.OrderBy{
+	order := &index.OrderBy{
 		Index: indexRule,
 		Sort:  modelv1.Sort_SORT_ASC,
 	}
