@@ -26,6 +26,7 @@ import (
 	"github.com/apache/skywalking-banyandb/banyand/internal/storage"
 	"github.com/apache/skywalking-banyandb/pkg/index"
 	pbv1 "github.com/apache/skywalking-banyandb/pkg/pb/v1"
+	"github.com/apache/skywalking-banyandb/pkg/pool"
 	"github.com/apache/skywalking-banyandb/pkg/timestamp"
 )
 
@@ -36,6 +37,12 @@ type tagValue struct {
 	valueType pbv1.ValueType
 }
 
+func (t *tagValue) reset() {
+	t.tag = ""
+	t.value = nil
+	t.valueArr = nil
+}
+
 func (t *tagValue) size() int {
 	s := len(t.tag)
 	if t.value != nil {
@@ -64,6 +71,21 @@ func (t *tagValue) marshal() []byte {
 	return t.value
 }
 
+func generateTagValue() *tagValue {
+	v := tagValuePool.Get()
+	if v == nil {
+		return &tagValue{}
+	}
+	return v
+}
+
+func releaseTagValue(v *tagValue) {
+	v.reset()
+	tagValuePool.Put(v)
+}
+
+var tagValuePool = pool.Register[*tagValue]("stream-tagValue")
+
 const (
 	entityDelimiter = '|'
 	escape          = '\\'
@@ -115,13 +137,33 @@ type tagValues struct {
 	values []*tagValue
 }
 
+func (t *tagValues) reset() {
+	t.tag = ""
+	for i := range t.values {
+		releaseTagValue(t.values[i])
+	}
+	t.values = t.values[:0]
+}
+
 type elements struct {
 	seriesIDs   []common.SeriesID
 	timestamps  []int64
-	elementIDs  []string
+	elementIDs  []uint64
 	tagFamilies [][]tagValues
 }
 
+func (e *elements) reset() {
+	e.seriesIDs = e.seriesIDs[:0]
+	e.timestamps = e.timestamps[:0]
+	e.elementIDs = e.elementIDs[:0]
+	for i := range e.tagFamilies {
+		for j := range e.tagFamilies[i] {
+			e.tagFamilies[i][j].reset()
+		}
+	}
+	e.tagFamilies = e.tagFamilies[:0]
+}
+
 func (e *elements) Len() int {
 	return len(e.seriesIDs)
 }
@@ -140,17 +182,35 @@ func (e *elements) Swap(i, j int) {
 	e.tagFamilies[i], e.tagFamilies[j] = e.tagFamilies[j], e.tagFamilies[i]
 }
 
+func generateElements() *elements {
+	v := elementsPool.Get()
+	if v == nil {
+		return &elements{}
+	}
+	return v
+}
+
+func releaseElements(e *elements) {
+	e.reset()
+	elementsPool.Put(e)
+}
+
+var elementsPool = pool.Register[*elements]("stream-elements")
+
 type elementsInTable struct {
 	timeRange timestamp.TimeRange
-	tsTable   storage.TSTableWrapper[*tsTable]
+	tsTable   *tsTable
 
-	elements elements
-	docs     index.Documents
+	elements *elements
+
+	docs index.Documents
 }
 
 type elementsInGroup struct {
-	tsdb     storage.TSDB[*tsTable, option]
-	docs     index.Documents
-	tables   []*elementsInTable
-	latestTS int64
+	tsdb        storage.TSDB[*tsTable, option]
+	docIDsAdded map[uint64]struct{}
+	docs        index.Documents
+	tables      []*elementsInTable
+	segments    []storage.Segment[*tsTable, option]
+	latestTS    int64
 }
