@@ -30,22 +30,23 @@ import (
 	"github.com/apache/skywalking-banyandb/pkg/index/inverted"
 	"github.com/apache/skywalking-banyandb/pkg/index/posting"
 	"github.com/apache/skywalking-banyandb/pkg/logger"
-	pbv1 "github.com/apache/skywalking-banyandb/pkg/pb/v1"
 	"github.com/apache/skywalking-banyandb/pkg/timestamp"
 )
 
 type elementIndex struct {
-	store index.Store
-	l     *logger.Logger
+	store    index.Store
+	l        *logger.Logger
+	location string
 }
 
 func newElementIndex(ctx context.Context, root string, flushTimeoutSeconds int64, metrics *inverted.Metrics) (*elementIndex, error) {
 	ei := &elementIndex{
-		l: logger.Fetch(ctx, "element_index"),
+		l:        logger.Fetch(ctx, "element_index"),
+		location: path.Join(root, elementIndexFilename),
 	}
 	var err error
 	if ei.store, err = inverted.NewStore(inverted.StoreOpts{
-		Path:         path.Join(root, elementIndexFilename),
+		Path:         ei.location,
 		Logger:       ei.l,
 		BatchWaitSec: flushTimeoutSeconds,
 		Metrics:      metrics,
@@ -71,19 +72,19 @@ func (e *elementIndex) Write(docs index.Documents) error {
 	})
 }
 
-func (e *elementIndex) Search(ctx context.Context, seriesList pbv1.SeriesList, filter index.Filter) (posting.List, error) {
-	var result posting.List
-	for i, series := range seriesList {
+func (e *elementIndex) Search(ctx context.Context, seriesList []uint64, filter index.Filter, tr *index.RangeOpts) (posting.List, posting.List, error) {
+	var result, resultTS posting.List
+	for i, id := range seriesList {
 		select {
 		case <-ctx.Done():
-			return nil, errors.WithMessagef(ctx.Err(), "search series %d/%d", i, len(seriesList))
+			return nil, nil, errors.WithMessagef(ctx.Err(), "search series %d/%d", i, len(seriesList))
 		default:
 		}
-		pl, err := filter.Execute(func(_ databasev1.IndexRule_Type) (index.Searcher, error) {
+		pl, plTS, err := filter.Execute(func(_ databasev1.IndexRule_Type) (index.Searcher, error) {
 			return e.store, nil
-		}, series.ID)
+		}, common.SeriesID(id), tr)
 		if err != nil {
-			return nil, err
+			return nil, nil, err
 		}
 		if pl == nil || pl.IsEmpty() {
 			continue
@@ -92,11 +93,18 @@ func (e *elementIndex) Search(ctx context.Context, seriesList pbv1.SeriesList, f
 			result = pl
 		} else {
 			if err := result.Union(pl); err != nil {
-				return nil, err
+				return nil, nil, err
+			}
+		}
+		if resultTS == nil {
+			resultTS = plTS
+		} else {
+			if err := resultTS.Union(plTS); err != nil {
+				return nil, nil, err
 			}
 		}
 	}
-	return result, nil
+	return result, resultTS, nil
 }
 
 func (e *elementIndex) Close() error {
