@@ -18,11 +18,21 @@
 package stream
 
 import (
+	"path/filepath"
+	"runtime"
 	"testing"
+	"time"
 
 	"github.com/stretchr/testify/assert"
+	"github.com/stretchr/testify/require"
 
+	"github.com/apache/skywalking-banyandb/api/common"
 	"github.com/apache/skywalking-banyandb/pkg/bytes"
+	"github.com/apache/skywalking-banyandb/pkg/fs"
+	"github.com/apache/skywalking-banyandb/pkg/logger"
+	"github.com/apache/skywalking-banyandb/pkg/test"
+	"github.com/apache/skywalking-banyandb/pkg/test/flags"
+	"github.com/apache/skywalking-banyandb/pkg/timestamp"
 )
 
 func TestSnapshotGetParts(t *testing.T) {
@@ -390,7 +400,6 @@ func TestSnapshotRemove(t *testing.T) {
 					{p: &part{
 						partMetadata: partMetadata{ID: 1},
 						timestamps:   &bytes.Buffer{},
-						elementIDs:   &bytes.Buffer{},
 						primary:      &bytes.Buffer{},
 					}, ref: 1},
 					{p: &part{partMetadata: partMetadata{ID: 2}}, ref: 2},
@@ -421,3 +430,130 @@ func TestSnapshotRemove(t *testing.T) {
 		})
 	}
 }
+
+func TestSnapshotFunctionality(t *testing.T) {
+	fileSystem := fs.NewLocalFileSystem()
+
+	tmpPath, deferFn := test.Space(require.New(t))
+	defer deferFn()
+
+	tabDir := filepath.Join(tmpPath, "tab")
+	fileSystem.MkdirPanicIfExist(tabDir, 0o755)
+
+	tst, err := newTSTable(
+		fileSystem,
+		tabDir,
+		common.Position{},
+		logger.GetLogger("test"),
+		timestamp.TimeRange{},
+		option{
+			flushTimeout: 0,
+			mergePolicy:  newDefaultMergePolicy(),
+		},
+		nil,
+	)
+	if err != nil {
+		t.Fatalf("failed to create newTSTable: %v", err)
+	}
+	defer tst.Close()
+
+	tst.mustAddElements(esTS1)
+	tst.mustAddElements(esTS2)
+	time.Sleep(100 * time.Millisecond) // allow time for flushing
+
+	require.Eventually(t, func() bool {
+		dd := fileSystem.ReadDir(tabDir)
+		partNum := 0
+		for _, d := range dd {
+			if d.IsDir() {
+				partNum++
+			}
+		}
+		return partNum >= 1
+	}, flags.EventuallyTimeout, time.Millisecond, "wait for file parts to be created")
+
+	snapshotPath := filepath.Join(tmpPath, "snapshot")
+	fileSystem.MkdirIfNotExist(snapshotPath, 0o755)
+
+	if err := tst.TakeFileSnapshot(snapshotPath); err != nil {
+		t.Fatalf("TakeFileSnapshot failed: %v", err)
+	}
+
+	entries := fileSystem.ReadDir(snapshotPath)
+
+	if len(entries) < 1 {
+		t.Fatalf("expected 1 directory for flushed part, got %d", len(entries))
+	}
+	hasIndex := false
+	var partDir string
+	for _, entry := range entries {
+		if !entry.IsDir() {
+			continue
+		}
+		if entry.Name() == elementIndexFilename {
+			hasIndex = true
+		} else {
+			partDir = entry.Name()
+		}
+	}
+	if !hasIndex {
+		t.Error("expected index file in snapshot, but none found")
+	}
+	if partDir == "" {
+		t.Error("expected part directory in snapshot, but none found")
+	}
+
+	partFiles := fileSystem.ReadDir(filepath.Join(snapshotPath, partDir))
+
+	// Check "primary.bin" and "meta.bin" existence
+	hasPrimary := false
+	hasMeta := false
+	for _, pf := range partFiles {
+		switch pf.Name() {
+		case "primary.bin":
+			hasPrimary = true
+		case "meta.bin":
+			hasMeta = true
+		}
+	}
+	if !hasPrimary {
+		t.Error("expected primary.bin in snapshot, but none found")
+	}
+	if !hasMeta {
+		t.Error("expected meta.bin in snapshot, but none found")
+	}
+
+	// Verify hard links (Unix-only)
+	if runtime.GOOS != "windows" {
+		srcFile := filepath.Join(tabDir, partDir, "primary.bin")
+		destFile := filepath.Join(snapshotPath, partDir, "primary.bin")
+
+		err := fs.CompareINode(srcFile, destFile)
+		if err != nil {
+			t.Fatalf("expected hard linked files to share inode: %v", err)
+		}
+	}
+}
+
+func TestGetDisjointParts(t *testing.T) {
+	p1 := &part{partMetadata: partMetadata{ID: 1, MinTimestamp: 1, MaxTimestamp: 3}}
+	p2 := &part{partMetadata: partMetadata{ID: 2, MinTimestamp: 2, MaxTimestamp: 4}}
+	p3 := &part{partMetadata: partMetadata{ID: 3, MinTimestamp: 5, MaxTimestamp: 7}}
+	p4 := &part{partMetadata: partMetadata{ID: 4, MinTimestamp: 6, MaxTimestamp: 8}}
+
+	parts := []*part{p1, p2, p3, p4}
+
+	groupsAsc := getDisjointParts(parts, true)
+	require.Equal(t, 2, len(groupsAsc), "expected 2 groups in ascending order")
+	require.Equal(t, 2, len(groupsAsc[0]), "first group should have 2 parts")
+	require.Equal(t, 2, len(groupsAsc[1]), "second group should have 2 parts")
+	require.Equal(t, p1, groupsAsc[0][0])
+	require.Equal(t, p2, groupsAsc[0][1])
+	require.Equal(t, p3, groupsAsc[1][0])
+	require.Equal(t, p4, groupsAsc[1][1])
+
+	groupsDesc := getDisjointParts(parts, false)
+	require.Equal(t, 2, len(groupsDesc), "expected 2 groups in descending order")
+	require.Equal(t, groupsAsc[1], groupsDesc[0], "first group in descending order should match second group in ascending order")
+	require.Equal(t, groupsAsc[0], groupsDesc[1], "second group in descending order should match first group in ascending order")
+}
