@@ -23,10 +23,12 @@ import (
 
 	g "github.com/onsi/ginkgo/v2"
 	"github.com/onsi/gomega"
-	"go.uber.org/mock/gomock"
 
 	"github.com/apache/skywalking-banyandb/banyand/metadata"
 	"github.com/apache/skywalking-banyandb/banyand/metadata/embeddedserver"
+	"github.com/apache/skywalking-banyandb/banyand/observability"
+	"github.com/apache/skywalking-banyandb/banyand/protector"
+	"github.com/apache/skywalking-banyandb/banyand/query"
 	"github.com/apache/skywalking-banyandb/banyand/queue"
 	"github.com/apache/skywalking-banyandb/banyand/stream"
 	"github.com/apache/skywalking-banyandb/pkg/logger"
@@ -62,11 +64,10 @@ func (p *preloadStreamService) PreRun(ctx context.Context) error {
 type services struct {
 	stream          stream.Service
 	metadataService metadata.Service
+	pipeline        queue.Queue
 }
 
 func setUp() (*services, func()) {
-	ctrl := gomock.NewController(g.GinkgoT())
-	gomega.Expect(ctrl).ShouldNot(gomega.BeNil())
 	// Init Pipeline
 	pipeline := queue.Local()
 
@@ -75,10 +76,14 @@ func setUp() (*services, func()) {
 	gomega.Expect(err).NotTo(gomega.HaveOccurred())
 	gomega.Expect(err).NotTo(gomega.HaveOccurred())
 
+	metricSvc := observability.NewMetricService(metadataService, pipeline, "test", nil)
+	pm := protector.NewMemory(metricSvc)
 	// Init Stream Service
-	streamService, err := stream.NewService(context.TODO(), metadataService, pipeline)
+	streamService, err := stream.NewService(metadataService, pipeline, metricSvc, pm)
 	gomega.Expect(err).NotTo(gomega.HaveOccurred())
 	preloadStreamSvc := &preloadStreamService{metaSvc: metadataService}
+	querySvc, err := query.NewService(context.TODO(), streamService, nil, metadataService, pipeline)
+	gomega.Expect(err).NotTo(gomega.HaveOccurred())
 	var flags []string
 	metaPath, metaDeferFunc, err := test.NewSpace()
 	gomega.Expect(err).NotTo(gomega.HaveOccurred())
@@ -95,10 +100,12 @@ func setUp() (*services, func()) {
 		metadataService,
 		preloadStreamSvc,
 		streamService,
+		querySvc,
 	)
 	return &services{
 			stream:          streamService,
 			metadataService: metadataService,
+			pipeline:        pipeline,
 		}, func() {
 			moduleDeferFunc()
 			metaDeferFunc()
