@@ -32,6 +32,7 @@ import (
 	"time"
 
 	"github.com/apache/skywalking-banyandb/api/common"
+	"github.com/apache/skywalking-banyandb/banyand/internal/storage"
 	"github.com/apache/skywalking-banyandb/pkg/fs"
 	"github.com/apache/skywalking-banyandb/pkg/index/inverted"
 	"github.com/apache/skywalking-banyandb/pkg/logger"
@@ -43,8 +44,6 @@ import (
 
 const (
 	snapshotSuffix = ".snp"
-	filePermission = 0o600
-	dirPermission  = 0o700
 )
 
 type tsTable struct {
@@ -133,7 +132,7 @@ func (tst *tsTable) mustWriteSnapshot(snapshot uint64, partNames []string) {
 		logger.Panicf("cannot marshal partNames to JSON: %s", err)
 	}
 	snapshotPath := filepath.Join(tst.root, snapshotName(snapshot))
-	lf, err := tst.fileSystem.CreateLockFile(snapshotPath, filePermission)
+	lf, err := tst.fileSystem.CreateLockFile(snapshotPath, storage.FilePerm)
 	if err != nil {
 		logger.Panicf("cannot create lock file %s: %s", snapshotPath, err)
 	}
