@@ -75,21 +75,39 @@ var VerifyFn = func(innerGm gm.Gomega, sharedContext helpers.SharedContext, args
 	innerGm.Expect(err).NotTo(gm.HaveOccurred())
 	want := &measurev1.QueryResponse{}
 	helpers.UnmarshalYAML(ww, want)
-	innerGm.Expect(cmp.Equal(resp, want,
+	for i := range resp.DataPoints {
+		if resp.DataPoints[i].Timestamp != nil {
+			innerGm.Expect(resp.DataPoints[i].Version).Should(gm.BeNumerically(">", 0))
+			innerGm.Expect(resp.DataPoints[i].Sid).Should(gm.BeNumerically(">", 0))
+		}
+	}
+	success := innerGm.Expect(cmp.Equal(resp, want,
 		protocmp.IgnoreUnknown(),
 		protocmp.IgnoreFields(&measurev1.DataPoint{}, "timestamp"),
+		protocmp.IgnoreFields(&measurev1.DataPoint{}, "version"),
+		protocmp.IgnoreFields(&measurev1.DataPoint{}, "sid"),
 		protocmp.Transform())).
 		To(gm.BeTrue(), func() string {
-			j, err := protojson.Marshal(resp)
+			var j []byte
+			j, err = protojson.Marshal(resp)
 			if err != nil {
 				return err.Error()
 			}
-			y, err := yaml.JSONToYAML(j)
+			var y []byte
+			y, err = yaml.JSONToYAML(j)
 			if err != nil {
 				return err.Error()
 			}
 			return string(y)
 		})
+	if !success {
+		return
+	}
+	query.Trace = true
+	resp, err = c.Query(ctx, query)
+	innerGm.Expect(err).NotTo(gm.HaveOccurred())
+	innerGm.Expect(resp.Trace).NotTo(gm.BeNil())
+	innerGm.Expect(resp.Trace.GetSpans()).NotTo(gm.BeEmpty())
 }
 
 //go:embed testdata/*.json
@@ -136,3 +154,25 @@ func Write(conn *grpclib.ClientConn, name, group, dataFile string,
 		return err
 	}, flags.EventuallyTimeout).Should(gm.Equal(io.EOF))
 }
+
+// WriteOnly write data into the server and return the write client.
+func WriteOnly(conn *grpclib.ClientConn, name, group, dataFile string,
+	baseTime time.Time, interval time.Duration,
+) measurev1.MeasureService_WriteClient {
+	metadata := &commonv1.Metadata{
+		Name:  name,
+		Group: group,
+	}
+
+	schema := databasev1.NewMeasureRegistryServiceClient(conn)
+	resp, err := schema.Get(context.Background(), &databasev1.MeasureRegistryServiceGetRequest{Metadata: metadata})
+	gm.Expect(err).NotTo(gm.HaveOccurred())
+	metadata = resp.GetMeasure().GetMetadata()
+
+	c := measurev1.NewMeasureServiceClient(conn)
+	ctx := context.Background()
+	writeClient, err := c.Write(ctx)
+	gm.Expect(err).NotTo(gm.HaveOccurred())
+	loadData(metadata, writeClient, dataFile, baseTime, interval)
+	return writeClient
+}
