@@ -46,6 +46,7 @@ type Args struct {
 	Duration  time.Duration
 	WantEmpty bool
 	WantErr   bool
+	DisOrder  bool
 }
 
 // UnmarshalYAML decodes YAML raw bytes to proto.Message.
@@ -69,3 +70,19 @@ func TimeRange(args Args, shardContext SharedContext) *modelv1.TimeRange {
 		End:   timestamppb.New(b.Add(args.Duration)),
 	}
 }
+
+// BackupSharedContext is the context shared between test cases in the snapshot testing.
+type BackupSharedContext struct {
+	DataAddr   string
+	Connection *grpclib.ClientConn
+	RootDir    string
+}
+
+// LifecycleSharedContext is the context shared between test cases in the lifecycle testing.
+type LifecycleSharedContext struct {
+	DataAddr   string
+	Connection *grpclib.ClientConn
+	SrcDir     string
+	DestDir    string
+	EtcdAddr   string
+}
