@@ -111,11 +111,13 @@ func standaloneServer(path string, ports []int, schemaLoaders []SchemaLoader, ce
 		"--stream-root-path=" + path,
 		"--measure-root-path=" + path,
 		"--metadata-root-path=" + path,
+		"--property-root-path=" + path,
 		fmt.Sprintf("--etcd-listen-client-url=%s", endpoint), fmt.Sprintf("--etcd-listen-peer-url=http://%s:%d", host, ports[3]),
 	}
 	tlsEnabled := false
 	if certFile != "" && keyFile != "" {
-		ff = append(ff, "--tls=true", "--cert-file="+certFile, "--key-file="+keyFile, "--http-grpc-cert-file="+certFile)
+		ff = append(ff, "--tls=true", "--cert-file="+certFile, "--key-file="+keyFile, "--http-grpc-cert-file="+certFile,
+			"--http-tls=true", "--http-cert-file="+certFile, "--http-key-file="+keyFile)
 		tlsEnabled = true
 	}
 	if len(flags) > 0 {
@@ -129,12 +131,13 @@ func standaloneServer(path string, ports []int, schemaLoaders []SchemaLoader, ce
 		gomega.Expect(err).NotTo(gomega.HaveOccurred())
 		gomega.Eventually(helpers.HealthCheck(addr, 10*time.Second, 10*time.Second, grpclib.WithTransportCredentials(creds)), testflags.EventuallyTimeout).
 			Should(gomega.Succeed())
+		gomega.Eventually(helpers.HTTPHealthCheck(httpAddr, certFile), testflags.EventuallyTimeout).Should(gomega.Succeed())
 	} else {
 		gomega.Eventually(
 			helpers.HealthCheck(addr, 10*time.Second, 10*time.Second, grpclib.WithTransportCredentials(insecure.NewCredentials())),
 			testflags.EventuallyTimeout).Should(gomega.Succeed())
+		gomega.Eventually(helpers.HTTPHealthCheck(httpAddr, ""), testflags.EventuallyTimeout).Should(gomega.Succeed())
 	}
-	gomega.Eventually(helpers.HTTPHealthCheck(httpAddr), testflags.EventuallyTimeout).Should(gomega.Succeed())
 
 	if schemaLoaders != nil {
 		schemaRegistry, err := schema.NewEtcdSchemaRegistry(
@@ -201,42 +204,67 @@ func CMD(flags ...string) func() {
 	}
 }
 
-// DataNode runs a data node.
-func DataNode(etcdEndpoint string) func() {
+func startDataNode(etcdEndpoint string, flags ...string) (string, string, func(), func()) {
 	path, deferFn, err := test.NewSpace()
 	gomega.Expect(err).NotTo(gomega.HaveOccurred())
+
 	ports, err := test.AllocateFreePorts(1)
 	gomega.Expect(err).NotTo(gomega.HaveOccurred())
+
 	addr := fmt.Sprintf("%s:%d", host, ports[0])
 	nodeHost := "127.0.0.1"
-	closeFn := CMD("data",
+
+	flags = append(flags,
+		"data",
 		"--grpc-host="+host,
 		fmt.Sprintf("--grpc-port=%d", ports[0]),
 		"--stream-root-path="+path,
 		"--measure-root-path="+path,
+		"--property-root-path="+path,
 		"--etcd-endpoints", etcdEndpoint,
 		"--node-host-provider", "flag",
-		"--node-host", nodeHost)
+		"--node-host", nodeHost,
+	)
+
+	closeFn := CMD(flags...)
+
 	gomega.Eventually(
 		helpers.HealthCheck(addr, 10*time.Second, 10*time.Second, grpclib.WithTransportCredentials(insecure.NewCredentials())),
 		testflags.EventuallyTimeout).Should(gomega.Succeed())
+
 	gomega.Eventually(func() (map[string]*databasev1.Node, error) {
 		return helpers.ListKeys(etcdEndpoint, fmt.Sprintf("/%s/nodes/%s:%d", metadata.DefaultNamespace, nodeHost, ports[0]))
 	}, testflags.EventuallyTimeout).Should(gomega.HaveLen(1))
+
+	return addr, path, closeFn, deferFn
+}
+
+// DataNode runs a data node.
+func DataNode(etcdEndpoint string, flags ...string) func() {
+	_, _, closeFn, deferFn := startDataNode(etcdEndpoint, flags...)
 	return func() {
 		closeFn()
 		deferFn()
 	}
 }
 
+// DataNodeWithAddrAndDir runs a data node and returns the address and root path.
+func DataNodeWithAddrAndDir(etcdEndpoint string, flags ...string) (string, string, func()) {
+	addr, dir, closeFn, deferFn := startDataNode(etcdEndpoint, flags...)
+	return addr, dir, func() {
+		closeFn()
+		deferFn()
+	}
+}
+
 // LiaisonNode runs a liaison node.
-func LiaisonNode(etcdEndpoint string) (string, func()) {
+func LiaisonNode(etcdEndpoint string, flags ...string) (string, func()) {
 	ports, err := test.AllocateFreePorts(2)
 	gomega.Expect(err).NotTo(gomega.HaveOccurred())
 	addr := fmt.Sprintf("%s:%d", host, ports[0])
 	httpAddr := fmt.Sprintf("%s:%d", host, ports[1])
 	nodeHost := "127.0.0.1"
-	closeFn := CMD("liaison",
+	flags = append(flags, "liaison",
 		"--grpc-host="+host,
 		fmt.Sprintf("--grpc-port=%d", ports[0]),
 		"--http-host="+host,
@@ -244,8 +272,10 @@ func LiaisonNode(etcdEndpoint string) (string, func()) {
 		"--http-grpc-addr="+addr,
 		"--etcd-endpoints", etcdEndpoint,
 		"--node-host-provider", "flag",
-		"--node-host", nodeHost)
-	gomega.Eventually(helpers.HTTPHealthCheck(httpAddr), testflags.EventuallyTimeout).Should(gomega.Succeed())
+		"--node-host", nodeHost,
+	)
+	closeFn := CMD(flags...)
+	gomega.Eventually(helpers.HTTPHealthCheck(httpAddr, ""), testflags.EventuallyTimeout).Should(gomega.Succeed())
 	gomega.Eventually(func() (map[string]*databasev1.Node, error) {
 		return helpers.ListKeys(etcdEndpoint, fmt.Sprintf("/%s/nodes/%s:%d", metadata.DefaultNamespace, nodeHost, ports[0]))
 	}, testflags.EventuallyTimeout).Should(gomega.HaveLen(1))
